@@ -8,7 +8,6 @@ from .aggregates import __all__ as aggregates_all
 from .constraints import *  # NOQA
 from .constraints import __all__ as constraints_all
 from .db import (
-    DEFAULT_DB_ALIAS,
     PLAIN_VERSION_PICKLE_KEY,
     DatabaseError,
     DataError,
@@ -20,10 +19,8 @@ from .db import (
     OperationalError,
     ProgrammingError,
     close_old_connections,
-    connection,
-    connections,
+    db_connection,
     reset_queries,
-    router,
 )
 from .deletion import (
     CASCADE,
@@ -59,25 +56,23 @@ from .expressions import (
 from .fields import *  # NOQA
 from .fields import __all__ as fields_all
 from .fields.json import JSONField
-from .fields.proxy import OrderWrt
 from .indexes import *  # NOQA
 from .indexes import __all__ as indexes_all
 from .lookups import Lookup, Transform
 from .manager import Manager
 from .query import Prefetch, QuerySet, prefetch_related_objects
 from .query_utils import FilteredRelation, Q
+from .registry import models_registry, register_model
 
 # Imports that would create circular imports if sorted
 from .base import DEFERRED, Model  # isort:skip
 from .fields.related import (  # isort:skip
     ForeignKey,
     ForeignObject,
-    OneToOneField,
     ManyToManyField,
     ForeignObjectRel,
     ManyToOneRel,
     ManyToManyRel,
-    OneToOneRel,
 )
 
 
@@ -110,7 +105,6 @@ __all__ += [
     "Window",
     "WindowFrame",
     "JSONField",
-    "OrderWrt",
     "Lookup",
     "Transform",
     "Manager",
@@ -123,19 +117,15 @@ __all__ += [
     "FilteredRelation",
     "ForeignKey",
     "ForeignObject",
-    "OneToOneField",
     "ManyToManyField",
     "ForeignObjectRel",
     "ManyToOneRel",
     "ManyToManyRel",
-    "OneToOneRel",
 ]
 
 # DB-related exports
 __all__ += [
-    "connection",
-    "connections",
-    "router",
+    "db_connection",
     "reset_queries",
     "close_old_connections",
     "DatabaseError",
@@ -147,6 +137,8 @@ __all__ += [
     "Error",
     "InterfaceError",
     "OperationalError",
-    "DEFAULT_DB_ALIAS",
     "PLAIN_VERSION_PICKLE_KEY",
 ]
+
+# Registry exports
+__all__ += ["register_model", "models_registry"]
