@@ -7,7 +7,6 @@ import sqlparse
 
 from plain.models.backends import utils
 from plain.models.db import NotSupportedError
-from plain.runtime import settings
 from plain.utils import timezone
 from plain.utils.encoding import force_str
 
@@ -82,17 +81,6 @@ class BaseDatabaseOperations:
             "format_for_duration_arithmetic() method."
         )
 
-    def cache_key_culling_sql(self):
-        """
-        Return an SQL query that retrieves the first cache key greater than the
-        n smallest.
-
-        This is used by the 'db' cache backend to determine where to start
-        culling.
-        """
-        cache_key = self.quote_name("cache_key")
-        return f"SELECT {cache_key} FROM %s ORDER BY {cache_key} LIMIT 1 OFFSET %%s"
-
     def unification_cast_sql(self, output_field):
         """
         Given a field instance, return the SQL that casts the result of a union
@@ -234,7 +222,7 @@ class BaseDatabaseOperations:
         """
         return "FOR{} UPDATE{}{}{}".format(
             " NO KEY" if no_key else "",
-            " OF %s" % ", ".join(of) if of else "",
+            " OF {}".format(", ".join(of)) if of else "",
             " NOWAIT" if nowait else "",
             " SKIP LOCKED" if skip_locked else "",
         )
@@ -253,8 +241,8 @@ class BaseDatabaseOperations:
         return " ".join(
             sql
             for sql in (
-                ("LIMIT %d" % limit) if limit else None,
-                ("OFFSET %d" % offset) if offset else None,
+                ("LIMIT %d" % limit) if limit else None,  # noqa: UP031
+                ("OFFSET %d" % offset) if offset else None,  # noqa: UP031
             )
             if sql
         )
@@ -345,13 +333,6 @@ class BaseDatabaseOperations:
             if statement
         ]
 
-    def process_clob(self, value):
-        """
-        Return the value of a CLOB column, for backends that return a locator
-        object that requires additional processing.
-        """
-        return value
-
     def return_insert_columns(self, fields):
         """
         For backends that support returning columns as part of an insert query,
@@ -398,19 +379,19 @@ class BaseDatabaseOperations:
         "uses_savepoints" feature is True. The "sid" parameter is a string
         for the savepoint id.
         """
-        return "SAVEPOINT %s" % self.quote_name(sid)
+        return f"SAVEPOINT {self.quote_name(sid)}"
 
     def savepoint_commit_sql(self, sid):
         """
         Return the SQL for committing the given savepoint.
         """
-        return "RELEASE SAVEPOINT %s" % self.quote_name(sid)
+        return f"RELEASE SAVEPOINT {self.quote_name(sid)}"
 
     def savepoint_rollback_sql(self, sid):
         """
         Return the SQL for rolling back the given savepoint.
         """
-        return "ROLLBACK TO SAVEPOINT %s" % self.quote_name(sid)
+        return f"ROLLBACK TO SAVEPOINT {self.quote_name(sid)}"
 
     def set_time_zone_sql(self):
         """
@@ -420,47 +401,6 @@ class BaseDatabaseOperations:
         """
         return ""
 
-    def sequence_reset_by_name_sql(self, style, sequences):
-        """
-        Return a list of the SQL statements required to reset sequences
-        passed in `sequences`.
-
-        The `style` argument is a Style object as returned by either
-        color_style() or no_style() in plain.internal.legacy.management.color.
-        """
-        return []
-
-    def sequence_reset_sql(self, style, model_list):
-        """
-        Return a list of the SQL statements required to reset sequences for
-        the given models.
-
-        The `style` argument is a Style object as returned by either
-        color_style() or no_style() in plain.internal.legacy.management.color.
-        """
-        return []  # No sequence reset required by default.
-
-    def start_transaction_sql(self):
-        """Return the SQL statement required to start a transaction."""
-        return "BEGIN;"
-
-    def end_transaction_sql(self, success=True):
-        """Return the SQL statement required to end a transaction."""
-        if not success:
-            return "ROLLBACK;"
-        return "COMMIT;"
-
-    def tablespace_sql(self, tablespace, inline=False):
-        """
-        Return the SQL that will be used in a query to define the tablespace.
-
-        Return '' if the backend doesn't support tablespaces.
-
-        If `inline` is True, append the SQL to a row; otherwise append it to
-        the entire CREATE TABLE or CREATE INDEX statement.
-        """
-        return ""
-
     def prep_for_like_query(self, x):
         """Prepare a value for use in a LIKE query."""
         return str(x).replace("\\", "\\\\").replace("%", r"\%").replace("_", r"\_")
@@ -591,10 +531,12 @@ class BaseDatabaseOperations:
         else:
             first = datetime.datetime(value, 1, 1)
             second = datetime.datetime(value, 12, 31, 23, 59, 59, 999999)
-        if settings.USE_TZ:
-            tz = timezone.get_current_timezone()
-            first = timezone.make_aware(first, tz)
-            second = timezone.make_aware(second, tz)
+
+        # Make sure that datetimes are aware in the current timezone
+        tz = timezone.get_current_timezone()
+        first = timezone.make_aware(first, tz)
+        second = timezone.make_aware(second, tz)
+
         first = self.adapt_datetimefield_value(first)
         second = self.adapt_datetimefield_value(second)
         return [first, second]
@@ -637,7 +579,7 @@ class BaseDatabaseOperations:
         can vary between backends (e.g., Oracle with %% and &) and between
         subexpression types (e.g., date expressions).
         """
-        conn = " %s " % connector
+        conn = f" {connector} "
         return conn.join(sub_expressions)
 
     def combine_duration_expression(self, connector, sub_expressions):
@@ -671,20 +613,19 @@ class BaseDatabaseOperations:
             rhs_sql, rhs_params = rhs
             return f"({lhs_sql} - {rhs_sql})", (*lhs_params, *rhs_params)
         raise NotSupportedError(
-            "This backend does not support %s subtraction." % internal_type
+            f"This backend does not support {internal_type} subtraction."
         )
 
     def window_frame_start(self, start):
         if isinstance(start, int):
             if start < 0:
-                return "%d %s" % (abs(start), self.PRECEDING)
+                return "%d %s" % (abs(start), self.PRECEDING)  # noqa: UP031
             elif start == 0:
                 return self.CURRENT_ROW
         elif start is None:
             return self.UNBOUNDED_PRECEDING
         raise ValueError(
-            "start argument must be a negative integer, zero, or None, but got '%s'."
-            % start
+            f"start argument must be a negative integer, zero, or None, but got '{start}'."
         )
 
     def window_frame_end(self, end):
@@ -692,12 +633,11 @@ class BaseDatabaseOperations:
             if end == 0:
                 return self.CURRENT_ROW
             elif end > 0:
-                return "%d %s" % (end, self.FOLLOWING)
+                return "%d %s" % (end, self.FOLLOWING)  # noqa: UP031
         elif end is None:
             return self.UNBOUNDED_FOLLOWING
         raise ValueError(
-            "end argument must be a positive integer, zero, or None, but got '%s'."
-            % end
+            f"end argument must be a positive integer, zero, or None, but got '{end}'."
         )
 
     def window_frame_rows_start_end(self, start=None, end=None):
@@ -715,8 +655,8 @@ class BaseDatabaseOperations:
             (start and start < 0) or (end and end > 0)
         ):
             raise NotSupportedError(
-                "%s only supports UNBOUNDED together with PRECEDING and "
-                "FOLLOWING." % self.connection.display_name
+                f"{self.connection.display_name} only supports UNBOUNDED together with PRECEDING and "
+                "FOLLOWING."
             )
         return start_, end_
 
@@ -729,16 +669,20 @@ class BaseDatabaseOperations:
             supported_formats = self.connection.features.supported_explain_formats
             normalized_format = format.upper()
             if normalized_format not in supported_formats:
-                msg = "%s is not a recognized format." % normalized_format
+                msg = f"{normalized_format} is not a recognized format."
                 if supported_formats:
-                    msg += " Allowed formats: %s" % ", ".join(sorted(supported_formats))
+                    msg += " Allowed formats: {}".format(
+                        ", ".join(sorted(supported_formats))
+                    )
                 else:
                     msg += (
                         f" {self.connection.display_name} does not support any formats."
                     )
                 raise ValueError(msg)
         if options:
-            raise ValueError("Unknown options: %s" % ", ".join(sorted(options.keys())))
+            raise ValueError(
+                "Unknown options: {}".format(", ".join(sorted(options.keys())))
+            )
         return self.explain_prefix
 
     def insert_statement(self, on_conflict=None):
