@@ -15,7 +15,6 @@ from plain.models.constraints import Deferrable
 from plain.models.indexes import Index
 from plain.models.sql import Query
 from plain.models.transaction import TransactionManagementError, atomic
-from plain.runtime import settings
 from plain.utils import timezone
 
 logger = logging.getLogger("plain.models.backends.schema")
@@ -44,7 +43,6 @@ def _all_related_fields(model):
             forward=False,
             reverse=True,
             include_hidden=True,
-            include_parents=False,
         ),
         key=operator.attrgetter("name"),
     )
@@ -83,7 +81,6 @@ class BaseDatabaseSchemaEditor:
     # Overrideable SQL templates
     sql_create_table = "CREATE TABLE %(table)s (%(definition)s)"
     sql_rename_table = "ALTER TABLE %(old_table)s RENAME TO %(new_table)s"
-    sql_retablespace_table = "ALTER TABLE %(table)s SET TABLESPACE %(new_tablespace)s"
     sql_delete_table = "DROP TABLE %(table)s CASCADE"
 
     sql_create_column = "ALTER TABLE %(table)s ADD COLUMN %(column)s %(definition)s"
@@ -140,8 +137,6 @@ class BaseDatabaseSchemaEditor:
     )
     sql_delete_pk = sql_delete_constraint
 
-    sql_delete_procedure = "DROP PROCEDURE %(procedure)s"
-
     sql_alter_table_comment = "COMMENT ON TABLE %(table)s IS %(comment)s"
     sql_alter_column_comment = "COMMENT ON COLUMN %(table)s.%(column)s IS %(comment)s"
 
@@ -157,7 +152,7 @@ class BaseDatabaseSchemaEditor:
     def __enter__(self):
         self.deferred_sql = []
         if self.atomic_migration:
-            self.atomic = atomic(self.connection.alias)
+            self.atomic = atomic()
             self.atomic.__enter__()
         return self
 
@@ -221,7 +216,7 @@ class BaseDatabaseSchemaEditor:
             # Autoincrement SQL (for backends with inline variant).
             col_type_suffix = field.db_type_suffix(connection=self.connection)
             if col_type_suffix:
-                definition += " %s" % col_type_suffix
+                definition += f" {col_type_suffix}"
             params.extend(extra_params)
             # FK.
             if field.remote_field and field.db_constraint:
@@ -266,12 +261,6 @@ class BaseDatabaseSchemaEditor:
                 if constraint
             ),
         }
-        if model._meta.db_tablespace:
-            tablespace_sql = self.connection.ops.tablespace_sql(
-                model._meta.db_tablespace
-            )
-            if tablespace_sql:
-                sql += " " + tablespace_sql
         return sql, params
 
     # Field <-> database mapping functions
@@ -285,7 +274,7 @@ class BaseDatabaseSchemaEditor:
         if self.connection.features.supports_comments_inline and field.db_comment:
             yield self._comment_sql(field.db_comment)
         # Work out nullability.
-        null = field.null
+        null = field.allow_null
         # Include a default value, if requested.
         include_default = (
             include_default
@@ -308,30 +297,14 @@ class BaseDatabaseSchemaEditor:
                 else:
                     yield column_default
                     params.append(default_value)
-        # Oracle treats the empty string ('') as null, so coerce the null
-        # option whenever '' is a possible value.
-        if (
-            field.empty_strings_allowed
-            and not field.primary_key
-            and self.connection.features.interprets_empty_strings_as_nulls
-        ):
-            null = True
+
         if not null:
             yield "NOT NULL"
         elif not self.connection.features.implied_column_null:
             yield "NULL"
+
         if field.primary_key:
             yield "PRIMARY KEY"
-        elif field.unique:
-            yield "UNIQUE"
-        # Optionally add the tablespace if it's an implicitly indexed column.
-        tablespace = field.db_tablespace or model._meta.db_tablespace
-        if (
-            tablespace
-            and self.connection.features.supports_tablespaces
-            and field.unique
-        ):
-            yield self.connection.ops.tablespace_sql(tablespace, inline=True)
 
     def column_sql(self, model, field, include_default=False):
         """
@@ -395,7 +368,9 @@ class BaseDatabaseSchemaEditor:
         # This method allows testing its logic without a connection.
         if field.has_default():
             default = field.get_default()
-        elif not field.null and field.blank and field.empty_strings_allowed:
+        elif (
+            not field.allow_null and not field.required and field.empty_strings_allowed
+        ):
             if field.get_internal_type() == "BinaryField":
                 default = b""
             else:
@@ -460,17 +435,8 @@ class BaseDatabaseSchemaEditor:
         # Add any field index (deferred as SQLite _remake_table needs it).
         self.deferred_sql.extend(self._model_indexes_sql(model))
 
-        # Make M2M tables
-        for field in model._meta.local_many_to_many:
-            if field.remote_field.through._meta.auto_created:
-                self.create_model(field.remote_field.through)
-
     def delete_model(self, model):
         """Delete a model from the database."""
-        # Handle auto-created intermediary models
-        for field in model._meta.local_many_to_many:
-            if field.remote_field.through._meta.auto_created:
-                self.delete_model(field.remote_field.through)
 
         # Delete the table
         self.execute(
@@ -558,25 +524,11 @@ class BaseDatabaseSchemaEditor:
             }
         )
 
-    def alter_db_tablespace(self, model, old_db_tablespace, new_db_tablespace):
-        """Move a model's table between tablespaces."""
-        self.execute(
-            self.sql_retablespace_table
-            % {
-                "table": self.quote_name(model._meta.db_table),
-                "old_tablespace": self.quote_name(old_db_tablespace),
-                "new_tablespace": self.quote_name(new_db_tablespace),
-            }
-        )
-
     def add_field(self, model, field):
         """
         Create a field on a model. Usually involves adding a column, but may
         involve adding a table instead (for M2M fields).
         """
-        # Special-case implicit M2M tables
-        if field.many_to_many and field.remote_field.through._meta.auto_created:
-            return self.create_model(field.remote_field.through)
         # Get the column's definition
         definition, params = self.column_sql(model, field, include_default=True)
         # It might not actually have a column behind it
@@ -603,9 +555,7 @@ class BaseDatabaseSchemaEditor:
                 namespace, _ = split_identifier(model._meta.db_table)
                 definition += " " + self.sql_create_column_inline_fk % {
                     "name": self._fk_constraint_name(model, field, constraint_suffix),
-                    "namespace": "%s." % self.quote_name(namespace)
-                    if namespace
-                    else "",
+                    "namespace": f"{self.quote_name(namespace)}." if namespace else "",
                     "column": self.quote_name(field.column),
                     "to_table": self.quote_name(to_table),
                     "to_column": self.quote_name(to_column),
@@ -660,9 +610,6 @@ class BaseDatabaseSchemaEditor:
         Remove a field from a model. Usually involves deleting a column,
         but for M2Ms may involve deleting a table.
         """
-        # Special-case implicit M2M tables
-        if field.many_to_many and field.remote_field.through._meta.auto_created:
-            return self.delete_model(field.remote_field.through)
         # It might not actually have a column behind it
         if field.db_parameters(connection=self.connection)["type"] is None:
             return
@@ -706,39 +653,21 @@ class BaseDatabaseSchemaEditor:
             new_type is None and new_field.remote_field is None
         ):
             raise ValueError(
-                "Cannot alter field {} into {} - they do not properly define "
-                "db_type (are you using a badly-written custom field?)".format(
-                    old_field, new_field
-                ),
+                f"Cannot alter field {old_field} into {new_field} - they do not properly define "
+                "db_type (are you using a badly-written custom field?)",
             )
         elif (
             old_type is None
             and new_type is None
-            and (
-                old_field.remote_field.through
-                and new_field.remote_field.through
-                and old_field.remote_field.through._meta.auto_created
-                and new_field.remote_field.through._meta.auto_created
-            )
-        ):
-            return self._alter_many_to_many(model, old_field, new_field, strict)
-        elif (
-            old_type is None
-            and new_type is None
-            and (
-                old_field.remote_field.through
-                and new_field.remote_field.through
-                and not old_field.remote_field.through._meta.auto_created
-                and not new_field.remote_field.through._meta.auto_created
-            )
+            and (old_field.remote_field.through and new_field.remote_field.through)
         ):
             # Both sides have through models; this is a no-op.
             return
         elif old_type is None or new_type is None:
             raise ValueError(
-                "Cannot alter field {} into {} - they are not compatible types "
+                f"Cannot alter field {old_field} into {new_field} - they are not compatible types "
                 "(you cannot alter to or from M2M fields, or add or remove "
-                "through= on M2M fields)".format(old_field, new_field)
+                "through= on M2M fields)"
             )
 
         self._alter_field(
@@ -792,18 +721,15 @@ class BaseDatabaseSchemaEditor:
             )
             if strict and len(fk_names) != 1:
                 raise ValueError(
-                    "Found wrong number ({}) of foreign key constraints for {}.{}".format(
-                        len(fk_names),
-                        model._meta.db_table,
-                        old_field.column,
-                    )
+                    f"Found wrong number ({len(fk_names)}) of foreign key constraints for {model._meta.db_table}.{old_field.column}"
                 )
             for fk_name in fk_names:
                 fks_dropped.add((old_field.column,))
                 self.execute(self._delete_fk_sql(model, fk_name))
         # Has unique been removed?
-        if old_field.unique and (
-            not new_field.unique or self._field_became_primary_key(old_field, new_field)
+        if old_field.primary_key and (
+            not new_field.primary_key
+            or self._field_became_primary_key(old_field, new_field)
         ):
             # Find the unique constraint for this field
             meta_constraint_names = {
@@ -818,11 +744,7 @@ class BaseDatabaseSchemaEditor:
             )
             if strict and len(constraint_names) != 1:
                 raise ValueError(
-                    "Found wrong number ({}) of unique constraints for {}.{}".format(
-                        len(constraint_names),
-                        model._meta.db_table,
-                        old_field.column,
-                    )
+                    f"Found wrong number ({len(constraint_names)}) of unique constraints for {model._meta.db_table}.{old_field.column}"
                 )
             for constraint_name in constraint_names:
                 self.execute(self._delete_unique_sql(model, constraint_name))
@@ -832,10 +754,7 @@ class BaseDatabaseSchemaEditor:
         new_collation = new_db_params.get("collation")
         drop_foreign_keys = (
             self.connection.features.supports_foreign_keys
-            and (
-                (old_field.primary_key and new_field.primary_key)
-                or (old_field.unique and new_field.unique)
-            )
+            and (old_field.primary_key and new_field.primary_key)
             and ((old_type != new_type) or (old_collation != new_collation))
         )
         if drop_foreign_keys:
@@ -852,15 +771,18 @@ class BaseDatabaseSchemaEditor:
         # will now be used in lieu of an index. The following lines from the
         # truth table show all True cases; the rest are False:
         #
-        # old_field.db_index | old_field.unique | new_field.db_index | new_field.unique
+        # old_field.db_index | old_field.primary_key | new_field.db_index | new_field.primary_key
         # ------------------------------------------------------------------------------
         # True               | False            | False              | False
         # True               | False            | False              | True
         # True               | False            | True               | True
         if (
-            old_field.db_index
-            and not old_field.unique
-            and (not new_field.db_index or new_field.unique)
+            (old_field.remote_field and old_field.db_index)
+            and not old_field.primary_key
+            and (
+                not (new_field.remote_field and new_field.db_index)
+                or new_field.primary_key
+            )
         ):
             # Find the index for this field
             meta_index_names = {index.name for index in model._meta.indexes}
@@ -893,11 +815,7 @@ class BaseDatabaseSchemaEditor:
             )
             if strict and len(constraint_names) != 1:
                 raise ValueError(
-                    "Found wrong number ({}) of check constraints for {}.{}".format(
-                        len(constraint_names),
-                        model._meta.db_table,
-                        old_field.column,
-                    )
+                    f"Found wrong number ({len(constraint_names)}) of check constraints for {model._meta.db_table}.{old_field.column}"
                 )
             for constraint_name in constraint_names:
                 self.execute(self._delete_check_sql(model, constraint_name))
@@ -944,7 +862,7 @@ class BaseDatabaseSchemaEditor:
         #  4. Drop the default again.
         # Default change?
         needs_database_default = False
-        if old_field.null and not new_field.null:
+        if old_field.allow_null and not new_field.allow_null:
             old_default = self.effective_default(old_field)
             new_default = self.effective_default(new_field)
             if (
@@ -957,13 +875,13 @@ class BaseDatabaseSchemaEditor:
                     self._alter_column_default_sql(model, old_field, new_field)
                 )
         # Nullability change?
-        if old_field.null != new_field.null:
+        if old_field.allow_null != new_field.allow_null:
             fragment = self._alter_column_null_sql(model, old_field, new_field)
             if fragment:
                 null_actions.append(fragment)
         # Only if we have a default and there is a change from NULL to NOT NULL
         four_way_default_alteration = new_field.has_default() and (
-            old_field.null and not new_field.null
+            old_field.allow_null and not new_field.allow_null
         )
         if actions or null_actions:
             if not four_way_default_alteration:
@@ -1012,22 +930,23 @@ class BaseDatabaseSchemaEditor:
         # If primary_key changed to False, delete the primary key constraint.
         if old_field.primary_key and not new_field.primary_key:
             self._delete_primary_key(model, strict)
-        # Added a unique?
-        if self._unique_should_be_added(old_field, new_field):
-            self.execute(self._create_unique_sql(model, [new_field]))
+
         # Added an index? Add an index if db_index switched to True or a unique
         # constraint will no longer be used in lieu of an index. The following
         # lines from the truth table show all True cases; the rest are False:
         #
-        # old_field.db_index | old_field.unique | new_field.db_index | new_field.unique
+        # old_field.db_index | old_field.primary_key | new_field.db_index | new_field.primary_key
         # ------------------------------------------------------------------------------
         # False              | False            | True               | False
         # False              | True             | True               | False
         # True               | True             | True               | False
         if (
-            (not old_field.db_index or old_field.unique)
-            and new_field.db_index
-            and not new_field.unique
+            (
+                not (old_field.remote_field and old_field.db_index)
+                or old_field.primary_key
+            )
+            and (new_field.remote_field and new_field.db_index)
+            and not new_field.primary_key
         ):
             self.execute(self._create_index_sql(model, fields=[new_field]))
         # Type alteration on primary key? Then we need to alter the column
@@ -1115,27 +1034,20 @@ class BaseDatabaseSchemaEditor:
         Return a (sql, params) fragment to set a column to null or non-null
         as required by new_field, or None if no changes are required.
         """
-        if (
-            self.connection.features.interprets_empty_strings_as_nulls
-            and new_field.empty_strings_allowed
-        ):
-            # The field is nullable in the database anyway, leave it alone.
-            return
-        else:
-            new_db_params = new_field.db_parameters(connection=self.connection)
-            sql = (
-                self.sql_alter_column_null
-                if new_field.null
-                else self.sql_alter_column_not_null
-            )
-            return (
-                sql
-                % {
-                    "column": self.quote_name(new_field.column),
-                    "type": new_db_params["type"],
-                },
-                [],
-            )
+        new_db_params = new_field.db_parameters(connection=self.connection)
+        sql = (
+            self.sql_alter_column_null
+            if new_field.allow_null
+            else self.sql_alter_column_not_null
+        )
+        return (
+            sql
+            % {
+                "column": self.quote_name(new_field.column),
+                "type": new_db_params["type"],
+            },
+            [],
+        )
 
     def _alter_column_default_sql(self, model, old_field, new_field, drop=False):
         """
@@ -1159,7 +1071,7 @@ class BaseDatabaseSchemaEditor:
 
         new_db_params = new_field.db_parameters(connection=self.connection)
         if drop:
-            if new_field.null:
+            if new_field.allow_null:
                 sql = self.sql_alter_column_no_default_null
             else:
                 sql = self.sql_alter_column_no_default
@@ -1275,9 +1187,8 @@ class BaseDatabaseSchemaEditor:
         and a unique digest and suffix.
         """
         _, table_name = split_identifier(table_name)
-        hash_suffix_part = "{}{}".format(
-            names_digest(table_name, *column_names, length=8),
-            suffix,
+        hash_suffix_part = (
+            f"{names_digest(table_name, *column_names, length=8)}{suffix}"
         )
         max_length = self.connection.ops.max_name_length() or 200
         # If everything fits into max_length, use that name.
@@ -1298,21 +1209,9 @@ class BaseDatabaseSchemaEditor:
         # Prepend D if needed to prevent the name from starting with an
         # underscore or a number (not permitted on Oracle).
         if index_name[0] == "_" or index_name[0].isdigit():
-            index_name = "D%s" % index_name[:-1]
+            index_name = f"D{index_name[:-1]}"
         return index_name
 
-    def _get_index_tablespace_sql(self, model, fields, db_tablespace=None):
-        if db_tablespace is None:
-            if len(fields) == 1 and fields[0].db_tablespace:
-                db_tablespace = fields[0].db_tablespace
-            elif settings.DEFAULT_INDEX_TABLESPACE:
-                db_tablespace = settings.DEFAULT_INDEX_TABLESPACE
-            elif model._meta.db_tablespace:
-                db_tablespace = model._meta.db_tablespace
-        if db_tablespace is not None:
-            return " " + self.connection.ops.tablespace_sql(db_tablespace)
-        return ""
-
     def _index_condition_sql(self, condition):
         if condition:
             return " WHERE " + condition
@@ -1334,7 +1233,6 @@ class BaseDatabaseSchemaEditor:
         name=None,
         suffix="",
         using="",
-        db_tablespace=None,
         col_suffixes=(),
         sql=None,
         opclasses=(),
@@ -1349,12 +1247,7 @@ class BaseDatabaseSchemaEditor:
         """
         fields = fields or []
         expressions = expressions or []
-        compiler = Query(model, alias_cols=False).get_compiler(
-            connection=self.connection,
-        )
-        tablespace_sql = self._get_index_tablespace_sql(
-            model, fields, db_tablespace=db_tablespace
-        )
+        compiler = Query(model, alias_cols=False).get_compiler()
         columns = [field.column for field in fields]
         sql_create_index = sql or self.sql_create_index
         table = model._meta.db_table
@@ -1375,7 +1268,7 @@ class BaseDatabaseSchemaEditor:
                 if columns
                 else Expressions(table, expressions, compiler, self.quote_value)
             ),
-            extra=tablespace_sql,
+            extra="",
             condition=self._index_condition_sql(condition),
             include=self._index_include_sql(model, include),
         )
@@ -1402,8 +1295,6 @@ class BaseDatabaseSchemaEditor:
         """
         Return a list of all index SQL statements (field indexes, Meta.indexes) for the specified model.
         """
-        if not model._meta.managed or model._meta.swapped:
-            return []
         output = []
         for field in model._meta.local_fields:
             output.extend(self._field_indexes_sql(model, field))
@@ -1443,18 +1334,11 @@ class BaseDatabaseSchemaEditor:
         ) or (old_path, old_args, old_kwargs) != (new_path, new_args, new_kwargs)
 
     def _field_should_be_indexed(self, model, field):
-        return field.db_index and not field.unique
+        return (field.remote_field and field.db_index) and not field.primary_key
 
     def _field_became_primary_key(self, old_field, new_field):
         return not old_field.primary_key and new_field.primary_key
 
-    def _unique_should_be_added(self, old_field, new_field):
-        return (
-            not new_field.primary_key
-            and new_field.unique
-            and (not old_field.unique or old_field.primary_key)
-        )
-
     def _rename_field_sql(self, table, old_field, new_field, new_type):
         return self.sql_rename_column % {
             "table": self.quote_name(table),
@@ -1572,9 +1456,7 @@ class BaseDatabaseSchemaEditor:
         ):
             return None
 
-        compiler = Query(model, alias_cols=False).get_compiler(
-            connection=self.connection
-        )
+        compiler = Query(model, alias_cols=False).get_compiler()
         table = model._meta.db_table
         columns = [field.column for field in fields]
         if name is None:
@@ -1717,10 +1599,7 @@ class BaseDatabaseSchemaEditor:
         constraint_names = self._constraint_names(model, primary_key=True)
         if strict and len(constraint_names) != 1:
             raise ValueError(
-                "Found wrong number ({}) of PK constraints for {}".format(
-                    len(constraint_names),
-                    model._meta.db_table,
-                )
+                f"Found wrong number ({len(constraint_names)}) of PK constraints for {model._meta.db_table}"
             )
         for constraint_name in constraint_names:
             self.execute(self._delete_primary_key_sql(model, constraint_name))
@@ -1742,10 +1621,3 @@ class BaseDatabaseSchemaEditor:
 
     def _collate_sql(self, collation, old_collation=None, table_name=None):
         return "COLLATE " + self.quote_name(collation) if collation else ""
-
-    def remove_procedure(self, procedure_name, param_types=()):
-        sql = self.sql_delete_procedure % {
-            "procedure": self.quote_name(procedure_name),
-            "param_types": ",".join(param_types),
-        }
-        self.execute(sql)
