@@ -5,7 +5,6 @@ from plain.models.backends.utils import split_tzname_delta
 from plain.models.constants import OnConflict
 from plain.models.expressions import Exists, ExpressionWrapper
 from plain.models.lookups import Lookup
-from plain.runtime import settings
 from plain.utils import timezone
 from plain.utils.encoding import force_str
 from plain.utils.regex_helper import _lazy_re_compile
@@ -91,7 +90,7 @@ class DatabaseOperations(BaseDatabaseOperations):
         return f"{sign}{offset}" if offset else tzname
 
     def _convert_sql_to_tz(self, sql, params, tzname):
-        if tzname and settings.USE_TZ and self.connection.timezone_name != tzname:
+        if tzname and self.connection.timezone_name != tzname:
             return f"CONVERT_TZ({sql}, %s, %s)", (
                 *params,
                 self.connection.timezone_name,
@@ -157,7 +156,7 @@ class DatabaseOperations(BaseDatabaseOperations):
         return cursor.fetchall()
 
     def format_for_duration_arithmetic(self, sql):
-        return "INTERVAL %s MICROSECOND" % sql
+        return f"INTERVAL {sql} MICROSECOND"
 
     def force_no_ordering(self):
         """
@@ -184,7 +183,7 @@ class DatabaseOperations(BaseDatabaseOperations):
     def quote_name(self, name):
         if name.startswith("`") and name.endswith("`"):
             return name  # Quoting once is enough.
-        return "`%s`" % name
+        return f"`{name}`"
 
     def return_insert_columns(self, fields):
         # MySQL and MariaDB < 10.5.0 don't support an INSERT...RETURNING
@@ -192,24 +191,10 @@ class DatabaseOperations(BaseDatabaseOperations):
         if not fields:
             return "", ()
         columns = [
-            "{}.{}".format(
-                self.quote_name(field.model._meta.db_table),
-                self.quote_name(field.column),
-            )
+            f"{self.quote_name(field.model._meta.db_table)}.{self.quote_name(field.column)}"
             for field in fields
         ]
-        return "RETURNING %s" % ", ".join(columns), ()
-
-    def sequence_reset_by_name_sql(self, style, sequences):
-        return [
-            "{} {} {} {} = 1;".format(
-                style.SQL_KEYWORD("ALTER"),
-                style.SQL_KEYWORD("TABLE"),
-                style.SQL_FIELD(self.quote_name(sequence_info["table"])),
-                style.SQL_FIELD("AUTO_INCREMENT"),
-            )
-            for sequence_info in sequences
-        ]
+        return "RETURNING {}".format(", ".join(columns)), ()
 
     def validate_autopk_value(self, value):
         # Zero in AUTO_INCREMENT field does not work without the
@@ -230,13 +215,7 @@ class DatabaseOperations(BaseDatabaseOperations):
 
         # MySQL doesn't support tz-aware datetimes
         if timezone.is_aware(value):
-            if settings.USE_TZ:
-                value = timezone.make_naive(value, self.connection.timezone)
-            else:
-                raise ValueError(
-                    "MySQL backend does not support timezone-aware datetimes when "
-                    "USE_TZ is False."
-                )
+            value = timezone.make_naive(value, self.connection.timezone)
         return str(value)
 
     def adapt_timefield_value(self, value):
@@ -261,17 +240,17 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def bulk_insert_sql(self, fields, placeholder_rows):
         placeholder_rows_sql = (", ".join(row) for row in placeholder_rows)
-        values_sql = ", ".join("(%s)" % sql for sql in placeholder_rows_sql)
+        values_sql = ", ".join(f"({sql})" for sql in placeholder_rows_sql)
         return "VALUES " + values_sql
 
     def combine_expression(self, connector, sub_expressions):
         if connector == "^":
-            return "POW(%s)" % ",".join(sub_expressions)
+            return "POW({})".format(",".join(sub_expressions))
         # Convert the result to a signed integer since MySQL's binary operators
         # return an unsigned integer.
         elif connector in ("&", "|", "<<", "#"):
             connector = "^" if connector == "#" else connector
-            return "CONVERT(%s, SIGNED)" % connector.join(sub_expressions)
+            return f"CONVERT({connector.join(sub_expressions)}, SIGNED)"
         elif connector == ">>":
             lhs, rhs = sub_expressions
             return f"FLOOR({lhs} / POW(2, {rhs}))"
@@ -283,8 +262,7 @@ class DatabaseOperations(BaseDatabaseOperations):
         if internal_type == "BooleanField":
             converters.append(self.convert_booleanfield_value)
         elif internal_type == "DateTimeField":
-            if settings.USE_TZ:
-                converters.append(self.convert_datetimefield_value)
+            converters.append(self.convert_datetimefield_value)
         elif internal_type == "UUIDField":
             converters.append(self.convert_uuidfield_value)
         return converters
@@ -348,7 +326,7 @@ class DatabaseOperations(BaseDatabaseOperations):
             )
         if format and not (analyze and not self.connection.mysql_is_mariadb):
             # Only MariaDB supports the analyze option with formats.
-            prefix += " FORMAT=%s" % format
+            prefix += f" FORMAT={format}"
         return prefix
 
     def regex_lookup(self, lookup_type):
@@ -359,7 +337,7 @@ class DatabaseOperations(BaseDatabaseOperations):
             return "%s REGEXP %s"
 
         match_option = "c" if lookup_type == "regex" else "i"
-        return "REGEXP_LIKE(%%s, %%s, '%s')" % match_option
+        return f"REGEXP_LIKE(%s, %s, '{match_option}')"
 
     def insert_statement(self, on_conflict=None):
         if on_conflict == OnConflict.IGNORE:
