@@ -93,7 +93,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         interface.
         """
         cursor.execute(
-            "PRAGMA table_info(%s)" % self.connection.ops.quote_name(table_name)
+            f"PRAGMA table_info({self.connection.ops.quote_name(table_name)})"
         )
         table_info = cursor.fetchall()
         if not table_info:
@@ -103,7 +103,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         if self.connection.features.can_introspect_json_field:
             for line in table_info:
                 column = line[1]
-                json_constraint_sql = '%%json_valid("%s")%%' % column
+                json_constraint_sql = f'%json_valid("{column}")%'
                 has_json_constraint = cursor.execute(
                     """
                     SELECT sql
@@ -144,7 +144,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         representing all foreign keys in the given table.
         """
         cursor.execute(
-            "PRAGMA foreign_key_list(%s)" % self.connection.ops.quote_name(table_name)
+            f"PRAGMA foreign_key_list({self.connection.ops.quote_name(table_name)})"
         )
         return {
             column_name: (ref_column_name, ref_table_name)
@@ -160,7 +160,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
 
     def get_primary_key_columns(self, cursor, table_name):
         cursor.execute(
-            "PRAGMA table_info(%s)" % self.connection.ops.quote_name(table_name)
+            f"PRAGMA table_info({self.connection.ops.quote_name(table_name)})"
         )
         return [name for _, name, *_, pk in cursor.fetchall() if pk]
 
@@ -289,7 +289,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                 else:
                     unnamed_constrains_index += 1
                     constraints[
-                        "__unnamed_constraint_%s__" % unnamed_constrains_index
+                        f"__unnamed_constraint_{unnamed_constrains_index}__"
                     ] = unique
             if check:
                 if constraint_name:
@@ -297,7 +297,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                 else:
                     unnamed_constrains_index += 1
                     constraints[
-                        "__unnamed_constraint_%s__" % unnamed_constrains_index
+                        f"__unnamed_constraint_{unnamed_constrains_index}__"
                     ] = check
             if end_token.match(sqlparse.tokens.Punctuation, ")"):
                 break
@@ -312,9 +312,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         # Find inline check constraints.
         try:
             table_schema = cursor.execute(
-                "SELECT sql FROM sqlite_master WHERE type='table' and name={}".format(
-                    self.connection.ops.quote_name(table_name)
-                )
+                f"SELECT sql FROM sqlite_master WHERE type='table' and name={self.connection.ops.quote_name(table_name)}"
             ).fetchone()[0]
         except TypeError:
             # table_name is a view.
@@ -327,7 +325,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
 
         # Get the index info
         cursor.execute(
-            "PRAGMA index_list(%s)" % self.connection.ops.quote_name(table_name)
+            f"PRAGMA index_list({self.connection.ops.quote_name(table_name)})"
         )
         for row in cursor.fetchall():
             # SQLite 3.8.9+ has 5 columns, however older versions only give 3
@@ -335,7 +333,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             number, index, unique = row[:3]
             cursor.execute(
                 "SELECT sql FROM sqlite_master "
-                "WHERE type='index' AND name=%s" % self.connection.ops.quote_name(index)
+                f"WHERE type='index' AND name={self.connection.ops.quote_name(index)}"
             )
             # There's at most one row.
             (sql,) = cursor.fetchone() or (None,)
@@ -350,7 +348,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                 continue
             # Get the index info for that index
             cursor.execute(
-                "PRAGMA index_info(%s)" % self.connection.ops.quote_name(index)
+                f"PRAGMA index_info({self.connection.ops.quote_name(index)})"
             )
             for index_rank, column_rank, column in cursor.fetchall():
                 if index not in constraints:
