@@ -2,61 +2,58 @@ import os
 import subprocess
 import sys
 import time
-from importlib import import_module
 from itertools import takewhile
 
 import click
 
-from plain.models import migrations
-from plain.models.db import DEFAULT_DB_ALIAS, OperationalError, connections, router
-from plain.models.migrations.autodetector import MigrationAutodetector
-from plain.models.migrations.executor import MigrationExecutor
-from plain.models.migrations.loader import AmbiguityError, MigrationLoader
-from plain.models.migrations.migration import Migration, SwappableTuple
-from plain.models.migrations.optimizer import MigrationOptimizer
-from plain.models.migrations.questioner import (
+from plain.cli import register_cli
+from plain.packages import packages_registry
+from plain.runtime import settings
+from plain.utils.text import Truncator
+
+from . import migrations
+from .backups.cli import cli as backups_cli
+from .backups.cli import create_backup
+from .db import OperationalError, db_connection
+from .migrations.autodetector import MigrationAutodetector
+from .migrations.executor import MigrationExecutor
+from .migrations.loader import AmbiguityError, MigrationLoader
+from .migrations.migration import Migration, SettingsTuple
+from .migrations.optimizer import MigrationOptimizer
+from .migrations.questioner import (
     InteractiveMigrationQuestioner,
     MigrationQuestioner,
     NonInteractiveMigrationQuestioner,
 )
-from plain.models.migrations.recorder import MigrationRecorder
-from plain.models.migrations.state import ModelState, ProjectState
-from plain.models.migrations.utils import get_migration_name_timestamp
-from plain.models.migrations.writer import MigrationWriter
-from plain.packages import packages
-from plain.runtime import __version__, settings
-from plain.utils.module_loading import module_has_submodule
-from plain.utils.text import Truncator
+from .migrations.recorder import MigrationRecorder
+from .migrations.state import ModelState, ProjectState
+from .migrations.utils import get_migration_name_timestamp
+from .migrations.writer import MigrationWriter
+from .registry import models_registry
 
 
+@register_cli("models")
 @click.group()
 def cli():
     pass
 
 
+cli.add_command(backups_cli)
+
+
 @cli.command()
-@click.option(
-    "--database",
-    default=DEFAULT_DB_ALIAS,
-    help=(
-        "Nominates a database onto which to open a shell. Defaults to the "
-        '"default" database.'
-    ),
-)
 @click.argument("parameters", nargs=-1)
-def db_shell(database, parameters):
+def db_shell(parameters):
     """Runs the command-line client for specified database, or the default database if none is provided."""
-    connection = connections[database]
     try:
-        connection.client.runshell(parameters)
+        db_connection.client.runshell(parameters)
     except FileNotFoundError:
         # Note that we're assuming the FileNotFoundError relates to the
         # command missing. It could be raised for some other reason, in
         # which case this error message would be inaccurate. Still, this
         # message catches the common case.
         click.secho(
-            "You appear not to have the %r program installed or on your path."
-            % connection.client.executable_name,
+            f"You appear not to have the {db_connection.client.executable_name!r} program installed or on your path.",
             fg="red",
             err=True,
         )
@@ -79,25 +76,27 @@ def db_wait():
     attempts = 0
     while True:
         attempts += 1
-        waiting_for = []
+        waiting_for = False
 
-        for conn in connections.all():
-            try:
-                conn.ensure_connection()
-            except OperationalError:
-                waiting_for.append(conn.alias)
+        try:
+            db_connection.ensure_connection()
+        except OperationalError:
+            waiting_for = True
 
         if waiting_for:
-            click.secho(
-                f"Waiting for database (attempt {attempts}): {', '.join(waiting_for)}",
-                fg="yellow",
-            )
+            if attempts > 1:
+                # After the first attempt, start printing them
+                click.secho(
+                    f"Waiting for database (attempt {attempts})",
+                    fg="yellow",
+                )
             time.sleep(1.5)
         else:
-            click.secho(f"Database ready: {', '.join(connections)}", fg="green")
+            click.secho("Database ready", fg="green")
             break
 
 
+@register_cli("makemigrations")
 @cli.command()
 @click.argument("package_labels", nargs=-1)
 @click.option(
@@ -120,11 +119,6 @@ def db_wait():
     is_flag=True,
     help="Exit with a non-zero status if model changes are missing migrations and don't actually write them.",
 )
-@click.option(
-    "--update",
-    is_flag=True,
-    help="Merge model changes into the latest migration and optimize the resulting operations.",
-)
 @click.option(
     "-v",
     "--verbosity",
@@ -133,7 +127,7 @@ def db_wait():
     help="Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output",
 )
 def makemigrations(
-    package_labels, dry_run, merge, empty, no_input, name, check, update, verbosity
+    package_labels, dry_run, merge, empty, no_input, name, check, verbosity
 ):
     """Creates new migration(s) for packages."""
 
@@ -201,77 +195,6 @@ def makemigrations(
                     )
                     log(writer.as_string(), level=3)
 
-    def write_to_last_migration_files(changes):
-        """Write changes to the last migration file for each package."""
-        loader = MigrationLoader(connections[DEFAULT_DB_ALIAS])
-        new_changes = {}
-        update_previous_migration_paths = {}
-        for package_label, package_migrations in changes.items():
-            leaf_migration_nodes = loader.graph.leaf_nodes(app=package_label)
-            if len(leaf_migration_nodes) == 0:
-                raise click.ClickException(
-                    f"Package {package_label} has no migration, cannot update last migration."
-                )
-            leaf_migration_node = leaf_migration_nodes[0]
-            leaf_migration = loader.graph.nodes[leaf_migration_node]
-
-            if leaf_migration.replaces:
-                raise click.ClickException(
-                    f"Cannot update squash migration '{leaf_migration}'."
-                )
-            if leaf_migration_node in loader.applied_migrations:
-                raise click.ClickException(
-                    f"Cannot update applied migration '{leaf_migration}'."
-                )
-
-            depending_migrations = [
-                migration
-                for migration in loader.disk_migrations.values()
-                if leaf_migration_node in migration.dependencies
-            ]
-            if depending_migrations:
-                formatted_migrations = ", ".join(
-                    [f"'{migration}'" for migration in depending_migrations]
-                )
-                raise click.ClickException(
-                    f"Cannot update migration '{leaf_migration}' that migrations "
-                    f"{formatted_migrations} depend on."
-                )
-
-            for migration in package_migrations:
-                leaf_migration.operations.extend(migration.operations)
-                for dependency in migration.dependencies:
-                    if isinstance(dependency, SwappableTuple):
-                        if settings.AUTH_USER_MODEL == dependency.setting:
-                            leaf_migration.dependencies.append(
-                                ("__setting__", "AUTH_USER_MODEL")
-                            )
-                        else:
-                            leaf_migration.dependencies.append(dependency)
-                    elif dependency[0] != migration.package_label:
-                        leaf_migration.dependencies.append(dependency)
-
-            optimizer = MigrationOptimizer()
-            leaf_migration.operations = optimizer.optimize(
-                leaf_migration.operations, package_label
-            )
-
-            previous_migration_path = MigrationWriter(leaf_migration).path
-            suggested_name = (
-                leaf_migration.name[:4] + "_" + leaf_migration.suggest_name()
-            )
-            new_name = (
-                suggested_name
-                if leaf_migration.name != suggested_name
-                else leaf_migration.name + "_updated"
-            )
-            leaf_migration.name = new_name
-
-            new_changes[package_label] = [leaf_migration]
-            update_previous_migration_paths[package_label] = previous_migration_path
-
-        write_migration_files(new_changes, update_previous_migration_paths)
-
     def handle_merge(loader, conflicts):
         """Handle merging conflicting migrations."""
         if interactive:
@@ -374,7 +297,7 @@ def makemigrations(
     has_bad_labels = False
     for package_label in package_labels:
         try:
-            packages.get_package_config(package_label)
+            packages_registry.get_package_config(package_label)
         except LookupError as err:
             click.echo(str(err), err=True)
             has_bad_labels = True
@@ -385,21 +308,8 @@ def makemigrations(
     loader = MigrationLoader(None, ignore_no_migrations=True)
 
     # Raise an error if any migrations are applied before their dependencies.
-    consistency_check_labels = {
-        config.label for config in packages.get_package_configs()
-    }
-    # Non-default databases are only checked if database routers used.
-    aliases_to_check = connections if settings.DATABASE_ROUTERS else [DEFAULT_DB_ALIAS]
-    for alias in sorted(aliases_to_check):
-        connection = connections[alias]
-        if connection.settings_dict["ENGINE"] != "plain.models.backends.dummy" and any(
-            router.allow_migrate(
-                connection.alias, package_label, model_name=model._meta.object_name
-            )
-            for package_label in consistency_check_labels
-            for model in packages.get_package_config(package_label).get_models()
-        ):
-            loader.check_consistent_history(connection)
+    # Only the default db_connection is supported.
+    loader.check_consistent_history(db_connection)
 
     # Check for conflicts
     conflicts = loader.detect_conflicts()
@@ -441,7 +351,7 @@ def makemigrations(
     # Set up autodetector
     autodetector = MigrationAutodetector(
         loader.project_state(),
-        ProjectState.from_packages(packages),
+        ProjectState.from_models_registry(models_registry),
         questioner,
     )
 
@@ -481,27 +391,14 @@ def makemigrations(
     else:
         if check_changes:
             sys.exit(1)
-        if update:
-            write_to_last_migration_files(changes)
-        else:
-            write_migration_files(changes)
+
+        write_migration_files(changes)
 
 
+@register_cli("migrate")
 @cli.command()
 @click.argument("package_label", required=False)
 @click.argument("migration_name", required=False)
-@click.option(
-    "--noinput",
-    "--no-input",
-    "no_input",
-    is_flag=True,
-    help="Tells Plain to NOT prompt the user for input of any kind.",
-)
-@click.option(
-    "--database",
-    default=DEFAULT_DB_ALIAS,
-    help="Nominates a database to synchronize. Defaults to the 'default' database.",
-)
 @click.option(
     "--fake", is_flag=True, help="Mark migrations as run without actually running them."
 )
@@ -522,12 +419,16 @@ def makemigrations(
     help="Exits with a non-zero status if unapplied migrations exist and does not actually apply migrations.",
 )
 @click.option(
-    "--run-syncdb", is_flag=True, help="Creates tables for packages without migrations."
+    "--backup/--no-backup",
+    "backup",
+    is_flag=True,
+    default=None,
+    help="Explicitly enable/disable pre-migration backups.",
 )
 @click.option(
     "--prune",
     is_flag=True,
-    help="Delete nonexistent migrations from the django_migrations table.",
+    help="Delete nonexistent migrations from the plainmigrations table.",
 )
 @click.option(
     "-v",
@@ -539,13 +440,11 @@ def makemigrations(
 def migrate(
     package_label,
     migration_name,
-    no_input,
-    database,
     fake,
     fake_initial,
     plan,
     check_unapplied,
-    run_syncdb,
+    backup,
     prune,
     verbosity,
 ):
@@ -553,130 +452,45 @@ def migrate(
 
     def migration_progress_callback(action, migration=None, fake=False):
         if verbosity >= 1:
-            compute_time = verbosity > 1
             if action == "apply_start":
-                if compute_time:
-                    start = time.monotonic()
                 click.echo(f"  Applying {migration}...", nl=False)
             elif action == "apply_success":
-                elapsed = f" ({time.monotonic() - start:.3f}s)" if compute_time else ""
                 if fake:
-                    click.echo(click.style(f" FAKED{elapsed}", fg="green"))
+                    click.echo(click.style(" FAKED", fg="green"))
                 else:
-                    click.echo(click.style(f" OK{elapsed}", fg="green"))
-            elif action == "unapply_start":
-                if compute_time:
-                    start = time.monotonic()
-                click.echo(f"  Unapplying {migration}...", nl=False)
-            elif action == "unapply_success":
-                elapsed = f" ({time.monotonic() - start:.3f}s)" if compute_time else ""
-                if fake:
-                    click.echo(click.style(f" FAKED{elapsed}", fg="green"))
-                else:
-                    click.echo(click.style(f" OK{elapsed}", fg="green"))
+                    click.echo(click.style(" OK", fg="green"))
             elif action == "render_start":
-                if compute_time:
-                    start = time.monotonic()
                 click.echo("  Rendering model states...", nl=False)
             elif action == "render_success":
-                elapsed = f" ({time.monotonic() - start:.3f}s)" if compute_time else ""
-                click.echo(click.style(f" DONE{elapsed}", fg="green"))
+                click.echo(click.style(" DONE", fg="green"))
 
-    def sync_packages(connection, package_labels):
-        """Run the old syncdb-style operation on a list of package_labels."""
-        with connection.cursor() as cursor:
-            tables = connection.introspection.table_names(cursor)
-
-        # Build the manifest of packages and models that are to be synchronized.
-        all_models = [
-            (
-                package_config.label,
-                router.get_migratable_models(
-                    package_config, connection.alias, include_auto_created=False
-                ),
-            )
-            for package_config in packages.get_package_configs()
-            if package_config.models_module is not None
-            and package_config.label in package_labels
-        ]
-
-        def model_installed(model):
-            opts = model._meta
-            converter = connection.introspection.identifier_converter
-            return not (
-                (converter(opts.db_table) in tables)
-                or (
-                    opts.auto_created
-                    and converter(opts.auto_created._meta.db_table) in tables
-                )
-            )
-
-        manifest = {
-            package_name: list(filter(model_installed, model_list))
-            for package_name, model_list in all_models
-        }
-
-        # Create the tables for each model
-        if verbosity >= 1:
-            click.echo("  Creating tables...", color="cyan")
-        with connection.schema_editor() as editor:
-            for package_name, model_list in manifest.items():
-                for model in model_list:
-                    # Never install unmanaged models, etc.
-                    if not model._meta.can_migrate(connection):
-                        continue
-                    if verbosity >= 3:
-                        click.echo(
-                            f"    Processing {package_name}.{model._meta.object_name} model"
-                        )
-                    if verbosity >= 1:
-                        click.echo(f"    Creating table {model._meta.db_table}")
-                    editor.create_model(model)
-
-            # Deferred SQL is executed when exiting the editor's context.
-            if verbosity >= 1:
-                click.echo("    Running deferred SQL...", color="cyan")
-
-    def describe_operation(operation, backwards):
+    def describe_operation(operation):
         """Return a string that describes a migration operation for --plan."""
         prefix = ""
         is_error = False
         if hasattr(operation, "code"):
-            code = operation.reverse_code if backwards else operation.code
+            code = operation.code
             action = (code.__doc__ or "") if code else None
         elif hasattr(operation, "sql"):
-            action = operation.reverse_sql if backwards else operation.sql
+            action = operation.sql
         else:
             action = ""
-            if backwards:
-                prefix = "Undo "
         if action is not None:
             action = str(action).replace("\n", "")
-        elif backwards:
-            action = "IRREVERSIBLE"
-            is_error = True
         if action:
             action = " -> " + action
         truncated = Truncator(action)
         return prefix + operation.describe() + truncated.chars(40), is_error
 
-    # Import the 'management' module within each installed package, to register
-    # dispatcher events.
-    for package_config in packages.get_package_configs():
-        if module_has_submodule(package_config.module, "management"):
-            import_module(".management", package_config.name)
-
     # Get the database we're operating from
-    connection = connections[database]
-
     # Hook for backends needing any database preparation
-    connection.prepare_database()
+    db_connection.prepare_database()
 
     # Work out which packages have migrations and which do not
-    executor = MigrationExecutor(connection, migration_progress_callback)
+    executor = MigrationExecutor(db_connection, migration_progress_callback)
 
     # Raise an error if any migrations are applied before their dependencies.
-    executor.loader.check_consistent_history(connection)
+    executor.loader.check_consistent_history(db_connection)
 
     # Before anything else, see if there's conflicting packages and drop out
     # hard if there are any
@@ -688,52 +502,45 @@ def migrate(
         )
         raise click.ClickException(
             "Conflicting migrations detected; multiple leaf nodes in the "
-            "migration graph: (%s).\nTo fix them run "
-            "'python manage.py makemigrations --merge'" % name_str
+            f"migration graph: ({name_str}).\nTo fix them run "
+            "'python manage.py makemigrations --merge'"
         )
 
     # If they supplied command line arguments, work out what they mean.
     target_package_labels_only = True
     if package_label:
         try:
-            packages.get_package_config(package_label)
+            packages_registry.get_package_config(package_label)
         except LookupError as err:
             raise click.ClickException(str(err))
-        if run_syncdb:
-            if package_label in executor.loader.migrated_packages:
-                raise click.ClickException(
-                    f"Can't use run_syncdb with package '{package_label}' as it has migrations."
-                )
-        elif package_label not in executor.loader.migrated_packages:
+
+        if package_label not in executor.loader.migrated_packages:
             raise click.ClickException(
                 f"Package '{package_label}' does not have migrations."
             )
 
     if package_label and migration_name:
-        if migration_name == "zero":
-            targets = [(package_label, None)]
-        else:
-            try:
-                migration = executor.loader.get_migration_by_prefix(
-                    package_label, migration_name
-                )
-            except AmbiguityError:
-                raise click.ClickException(
-                    f"More than one migration matches '{migration_name}' in package '{package_label}'. "
-                    "Please be more specific."
-                )
-            except KeyError:
-                raise click.ClickException(
-                    f"Cannot find a migration matching '{migration_name}' from package '{package_label}'."
-                )
-            target = (package_label, migration.name)
-            if (
-                target not in executor.loader.graph.nodes
-                and target in executor.loader.replacements
-            ):
-                incomplete_migration = executor.loader.replacements[target]
-                target = incomplete_migration.replaces[-1]
-            targets = [target]
+        try:
+            migration = executor.loader.get_migration_by_prefix(
+                package_label, migration_name
+            )
+        except AmbiguityError:
+            raise click.ClickException(
+                f"More than one migration matches '{migration_name}' in package '{package_label}'. "
+                "Please be more specific."
+            )
+        except KeyError:
+            raise click.ClickException(
+                f"Cannot find a migration matching '{migration_name}' from package '{package_label}'."
+            )
+        target = (package_label, migration.name)
+        if (
+            target not in executor.loader.graph.nodes
+            and target in executor.loader.replacements
+        ):
+            incomplete_migration = executor.loader.replacements[target]
+            target = incomplete_migration.replaces[-1]
+        targets = [target]
         target_package_labels_only = False
     elif package_label:
         targets = [
@@ -802,10 +609,10 @@ def migrate(
         if not migration_plan:
             click.echo("  No planned migration operations.")
         else:
-            for migration, backwards in migration_plan:
+            for migration in migration_plan:
                 click.echo(str(migration), color="cyan")
                 for operation in migration.operations:
-                    message, is_error = describe_operation(operation, backwards)
+                    message, is_error = describe_operation(operation)
                     if is_error:
                         click.echo("    " + message, fg="yellow")
                     else:
@@ -822,22 +629,10 @@ def migrate(
     if prune:
         return
 
-    # At this point, ignore run_syncdb if there aren't any packages to sync.
-    run_syncdb = run_syncdb and executor.loader.unmigrated_packages
     # Print some useful info
     if verbosity >= 1:
         click.echo("Operations to perform:", color="cyan")
-        if run_syncdb:
-            if package_label:
-                click.echo(
-                    f"  Synchronize unmigrated package: {package_label}", color="yellow"
-                )
-            else:
-                click.echo(
-                    "  Synchronize unmigrated packages: "
-                    + (", ".join(sorted(executor.loader.unmigrated_packages))),
-                    color="yellow",
-                )
+
         if target_package_labels_only:
             click.echo(
                 "  Apply all migrations: "
@@ -845,55 +640,39 @@ def migrate(
                 color="yellow",
             )
         else:
-            if targets[0][1] is None:
-                click.echo(f"  Unapply all migrations: {targets[0][0]}", color="yellow")
-            else:
-                click.echo(
-                    f"  Target specific migration: {targets[0][1]}, from {targets[0][0]}",
-                    color="yellow",
-                )
+            click.echo(
+                f"  Target specific migration: {targets[0][1]}, from {targets[0][0]}",
+                color="yellow",
+            )
 
     pre_migrate_state = executor._create_project_state(with_applied_migrations=True)
 
-    # Run the syncdb phase.
-    if run_syncdb:
-        if verbosity >= 1:
-            click.echo("Synchronizing packages without migrations:", color="cyan")
-        if package_label:
-            sync_packages(connection, [package_label])
-        else:
-            sync_packages(connection, executor.loader.unmigrated_packages)
+    # sql = executor.loader.collect_sql(migration_plan)
+    # pprint(sql)
 
-    # Migrate!
-    if verbosity >= 1:
-        click.echo("Running migrations:", color="cyan")
-    if not migration_plan:
-        if verbosity >= 1:
-            click.echo("  No migrations to apply.")
-            # If there's changes that aren't in migrations yet, tell them
-            # how to fix it.
-            autodetector = MigrationAutodetector(
-                executor.loader.project_state(),
-                ProjectState.from_packages(packages),
+    if migration_plan:
+        if backup or (
+            backup is None
+            and settings.DEBUG
+            and click.confirm(
+                "\nYou are in DEBUG mode. Would you like to make a database backup before running migrations?",
+                default=True,
             )
-            changes = autodetector.changes(graph=executor.loader.graph)
-            if changes:
-                click.echo(
-                    click.style(
-                        f"  Your models in package(s): {', '.join(repr(package) for package in sorted(changes))} "
-                        "have changes that are not yet reflected in a migration, and so won't be applied.",
-                        fg="yellow",
-                    )
-                )
-                click.echo(
-                    click.style(
-                        "  Run 'manage.py makemigrations' to make new "
-                        "migrations, and then re-run 'manage.py migrate' to "
-                        "apply them.",
-                        fg="yellow",
-                    )
-                )
-    else:
+        ):
+            backup_name = f"migrate_{time.strftime('%Y%m%d_%H%M%S')}"
+            # Can't use ctx.invoke because this is called by the test db creation currently,
+            # which doesn't have a context.
+            create_backup.callback(
+                backup_name=backup_name,
+                pg_dump=os.environ.get(
+                    "PG_DUMP", "pg_dump"
+                ),  # Have to this again manually
+            )
+            print()
+
+        if verbosity >= 1:
+            click.echo("Running migrations:", color="cyan")
+
         post_migrate_state = executor.migrate(
             targets,
             plan=migration_plan,
@@ -903,8 +682,8 @@ def migrate(
         )
         # post_migrate signals have access to all models. Ensure that all models
         # are reloaded in case any are delayed.
-        post_migrate_state.clear_delayed_packages_cache()
-        post_migrate_packages = post_migrate_state.packages
+        post_migrate_state.clear_delayed_models_cache()
+        post_migrate_packages = post_migrate_state.models_registry
 
         # Re-render models of real packages to include relationships now that
         # we've got a final state. This wouldn't be necessary if real packages
@@ -916,9 +695,38 @@ def migrate(
                 model_keys.append(model_key)
                 post_migrate_packages.unregister_model(*model_key)
         post_migrate_packages.render_multiple(
-            [ModelState.from_model(packages.get_model(*model)) for model in model_keys]
+            [
+                ModelState.from_model(models_registry.get_model(*model))
+                for model in model_keys
+            ]
         )
 
+    elif verbosity >= 1:
+        click.echo("  No migrations to apply.")
+        # If there's changes that aren't in migrations yet, tell them
+        # how to fix it.
+        autodetector = MigrationAutodetector(
+            executor.loader.project_state(),
+            ProjectState.from_models_registry(models_registry),
+        )
+        changes = autodetector.changes(graph=executor.loader.graph)
+        if changes:
+            click.echo(
+                click.style(
+                    f"  Your models in package(s): {', '.join(repr(package) for package in sorted(changes))} "
+                    "have changes that are not yet reflected in a migration, and so won't be applied.",
+                    fg="yellow",
+                )
+            )
+            click.echo(
+                click.style(
+                    "  Run 'manage.py makemigrations' to make new "
+                    "migrations, and then re-run 'manage.py migrate' to "
+                    "apply them.",
+                    fg="yellow",
+                )
+            )
+
 
 @cli.command()
 @click.argument("package_label")
@@ -938,7 +746,7 @@ def migrate(
 def optimize_migration(package_label, migration_name, check, verbosity):
     """Optimizes the operations for the named migration."""
     try:
-        packages.get_package_config(package_label)
+        packages_registry.get_package_config(package_label)
     except LookupError as err:
         raise click.ClickException(str(err))
 
@@ -988,9 +796,7 @@ def optimize_migration(package_label, migration_name, check, verbosity):
         if migration.replaces:
             raise click.ClickException(
                 "Migration will require manual porting but is already a squashed "
-                "migration.\nTransition to a normal migration first: "
-                f"https://docs.djangoproject.com/en/{__version__}/topics/migrations/"
-                "#squashing-migrations"
+                "migration.\nTransition to a normal migration first."
             )
         # Make a new migration with those operations.
         subclass = type(
@@ -1027,11 +833,6 @@ def optimize_migration(package_label, migration_name, check, verbosity):
 
 @cli.command()
 @click.argument("package_labels", nargs=-1)
-@click.option(
-    "--database",
-    default=DEFAULT_DB_ALIAS,
-    help="Nominates a database to show migrations for. Defaults to the 'default' database.",
-)
 @click.option(
     "--format",
     type=click.Choice(["list", "plan"]),
@@ -1045,29 +846,30 @@ def optimize_migration(package_label, migration_name, check, verbosity):
     default=1,
     help="Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output",
 )
-def show_migrations(package_labels, database, format, verbosity):
+def show_migrations(package_labels, format, verbosity):
     """Shows all available migrations for the current project"""
 
     def _validate_package_names(package_names):
         has_bad_names = False
         for package_name in package_names:
             try:
-                packages.get_package_config(package_name)
+                packages_registry.get_package_config(package_name)
             except LookupError as err:
                 click.echo(str(err), err=True)
                 has_bad_names = True
         if has_bad_names:
             sys.exit(2)
 
-    def show_list(connection, package_names):
+    def show_list(db_connection, package_names):
         """
         Show a list of all migrations on the system, or only those of
         some named packages.
         """
         # Load migrations from disk/DB
-        loader = MigrationLoader(connection, ignore_no_migrations=True)
-        recorder = MigrationRecorder(connection)
+        loader = MigrationLoader(db_connection, ignore_no_migrations=True)
+        recorder = MigrationRecorder(db_connection)
         recorded_migrations = recorder.applied_migrations()
+
         graph = loader.graph
         # If we were passed a list of packages, validate it
         if package_names:
@@ -1105,13 +907,42 @@ def show_migrations(package_labels, database, format, verbosity):
             if not shown:
                 click.secho(" (no migrations)", fg="red")
 
-    def show_plan(connection, package_names):
+        # Find recorded migrations that aren't in the graph (prunable)
+        prunable_migrations = [
+            migration
+            for migration in recorded_migrations
+            if (
+                migration not in loader.disk_migrations
+                and (not package_names or migration[0] in package_names)
+            )
+        ]
+
+        if prunable_migrations:
+            click.echo()
+            click.secho(
+                "Recorded migrations not in migration files (candidates for pruning):",
+                fg="yellow",
+                bold=True,
+            )
+            prunable_by_package = {}
+            for migration in prunable_migrations:
+                package, name = migration
+                if package not in prunable_by_package:
+                    prunable_by_package[package] = []
+                prunable_by_package[package].append(name)
+
+            for package in sorted(prunable_by_package.keys()):
+                click.secho(f"  {package}:", fg="yellow")
+                for name in sorted(prunable_by_package[package]):
+                    click.echo(f"    - {name}")
+
+    def show_plan(db_connection, package_names):
         """
         Show all known migrations (or only those of the specified package_names)
         in the order they will be applied.
         """
         # Load migrations from disk/DB
-        loader = MigrationLoader(connection)
+        loader = MigrationLoader(db_connection)
         graph = loader.graph
         if package_names:
             _validate_package_names(package_names)
@@ -1150,12 +981,11 @@ def show_migrations(package_labels, database, format, verbosity):
             click.secho("(no migrations)", fg="red")
 
     # Get the database we're operating from
-    connection = connections[database]
 
     if format == "plan":
-        show_plan(connection, package_labels)
+        show_plan(db_connection, package_labels)
     else:
-        show_list(connection, package_labels)
+        show_list(db_connection, package_labels)
 
 
 @cli.command()
@@ -1210,12 +1040,12 @@ def squash_migrations(
 
     # Validate package_label
     try:
-        packages.get_package_config(package_label)
+        packages_registry.get_package_config(package_label)
     except LookupError as err:
         raise click.ClickException(str(err))
 
     # Load the current graph state, check the app and migration they asked for exists
-    loader = MigrationLoader(connections[DEFAULT_DB_ALIAS])
+    loader = MigrationLoader(db_connection)
     if package_label not in loader.migrated_packages:
         raise click.ClickException(
             f"Package '{package_label}' does not have migrations (so squashmigrations on it makes no sense)"
@@ -1270,16 +1100,12 @@ def squash_migrations(
         if smigration.replaces:
             raise click.ClickException(
                 "You cannot squash squashed migrations! Please transition it to a "
-                f"normal migration first: https://docs.djangoproject.com/en/{__version__}/"
-                "topics/migrations/#squashing-migrations"
+                "normal migration first"
             )
         operations.extend(smigration.operations)
         for dependency in smigration.dependencies:
-            if isinstance(dependency, SwappableTuple):
-                if settings.AUTH_USER_MODEL == dependency.setting:
-                    dependencies.add(("__setting__", "AUTH_USER_MODEL"))
-                else:
-                    dependencies.add(dependency)
+            if isinstance(dependency, SettingsTuple):
+                dependencies.add(dependency)
             elif dependency[0] != smigration.package_label or first_migration:
                 dependencies.add(dependency)
         first_migration = False
