@@ -2,19 +2,18 @@ import collections.abc
 import copy
 import datetime
 import decimal
+import enum
 import operator
 import uuid
 import warnings
 from base64 import b64decode, b64encode
-from functools import partialmethod, total_ordering
+from functools import cached_property, partialmethod, total_ordering
 
 from plain import exceptions, preflight, validators
 from plain.models.constants import LOOKUP_SEP
-from plain.models.db import connection, connections, router
+from plain.models.db import db_connection
 from plain.models.enums import ChoicesMeta
 from plain.models.query_utils import DeferredAttribute, RegisterLookupMixin
-from plain.packages import packages
-from plain.runtime import settings
 from plain.utils import timezone
 from plain.utils.datastructures import DictWrapper
 from plain.utils.dateparse import (
@@ -24,10 +23,12 @@ from plain.utils.dateparse import (
     parse_time,
 )
 from plain.utils.duration import duration_microseconds, duration_string
-from plain.utils.functional import Promise, cached_property
+from plain.utils.functional import Promise
 from plain.utils.ipv6 import clean_ipv6_address
 from plain.utils.itercompat import is_iterable
 
+from ..registry import models_registry
+
 __all__ = [
     "AutoField",
     "BLANK_CHOICE_DASH",
@@ -53,7 +54,6 @@ __all__ = [
     "PositiveBigIntegerField",
     "PositiveIntegerField",
     "PositiveSmallIntegerField",
-    "SlugField",
     "SmallAutoField",
     "SmallIntegerField",
     "TextField",
@@ -77,7 +77,9 @@ BLANK_CHOICE_DASH = [("", "---------")]
 
 
 def _load_field(package_label, model_name, field_name):
-    return packages.get_model(package_label, model_name)._meta.get_field(field_name)
+    return models_registry.get_model(package_label, model_name)._meta.get_field(
+        field_name
+    )
 
 
 # A guide to Field parameters:
@@ -123,8 +125,8 @@ class Field(RegisterLookupMixin):
     default_validators = []  # Default set of validators
     default_error_messages = {
         "invalid_choice": "Value %(value)r is not a valid choice.",
-        "null": "This field cannot be null.",
-        "blank": "This field cannot be blank.",
+        "allow_null": "This field cannot be null.",
+        "required": "This field is be required.",
         "unique": "A %(model_name)s with this %(field_label)s already exists.",
     }
     system_check_deprecated_details = None
@@ -133,10 +135,9 @@ class Field(RegisterLookupMixin):
     # Attributes that don't affect a column definition.
     # These attributes are ignored when altering the field.
     non_db_attrs = (
-        "blank",
+        "required",
         "choices",
         "db_column",
-        "editable",
         "error_messages",
         "limit_choices_to",
         # Database-level options are not supported, see #21961.
@@ -152,7 +153,6 @@ class Field(RegisterLookupMixin):
     many_to_many = None
     many_to_one = None
     one_to_many = None
-    one_to_one = None
     related_model = None
 
     descriptor_class = DeferredAttribute
@@ -165,41 +165,36 @@ class Field(RegisterLookupMixin):
 
     def __init__(
         self,
-        name=None,
+        *,
         primary_key=False,
         max_length=None,
-        unique=False,
-        blank=False,
-        null=False,
-        db_index=False,
+        required=True,
+        allow_null=False,
         rel=None,
         default=NOT_PROVIDED,
-        editable=True,
         choices=None,
         db_column=None,
-        db_tablespace=None,
         auto_created=False,
         validators=(),
         error_messages=None,
         db_comment=None,
     ):
-        self.name = name
+        self.name = None  # Set by set_attributes_from_name
         self.primary_key = primary_key
-        self.max_length, self._unique = max_length, unique
-        self.blank, self.null = blank, null
+        self.max_length = max_length
+        self.required, self.allow_null = required, allow_null
         self.remote_field = rel
         self.is_relation = self.remote_field is not None
         self.default = default
-        self.editable = editable
         if isinstance(choices, ChoicesMeta):
             choices = choices.choices
+        elif isinstance(choices, enum.EnumMeta):
+            choices = [(member.value, member.name) for member in choices]
         if isinstance(choices, collections.abc.Iterator):
             choices = list(choices)
         self.choices = choices
-        self.db_index = db_index
         self.db_column = db_column
         self.db_comment = db_comment
-        self._db_tablespace = db_tablespace
         self.auto_created = auto_created
 
         # Adjust the appropriate creation counter, and save our local copy.
@@ -230,13 +225,12 @@ class Field(RegisterLookupMixin):
         name = getattr(self, "name", None)
         if name is not None:
             return f"<{path}: {name}>"
-        return "<%s>" % path
+        return f"<{path}>"
 
     def check(self, **kwargs):
         return [
             *self._check_field_name(),
             *self._check_choices(),
-            *self._check_db_index(),
             *self._check_db_comment(**kwargs),
             *self._check_null_allowed_for_primary_keys(),
             *self._check_backend_specific_checks(**kwargs),
@@ -260,7 +254,7 @@ class Field(RegisterLookupMixin):
         elif LOOKUP_SEP in self.name:
             return [
                 preflight.Error(
-                    'Field names must not contain "%s".' % LOOKUP_SEP,
+                    f'Field names must not contain "{LOOKUP_SEP}".',
                     obj=self,
                     id="fields.E002",
                 )
@@ -335,7 +329,7 @@ class Field(RegisterLookupMixin):
             if self.max_length is not None and choice_max_length > self.max_length:
                 return [
                     preflight.Error(
-                        "'max_length' is too small to fit the longest value "
+                        "'max_length' is too small to fit the longest value "  # noqa: UP031
                         "in 'choices' (%d characters)." % choice_max_length,
                         obj=self,
                         id="fields.E009",
@@ -352,54 +346,34 @@ class Field(RegisterLookupMixin):
             )
         ]
 
-    def _check_db_index(self):
-        if self.db_index not in (None, True, False):
-            return [
-                preflight.Error(
-                    "'db_index' must be None, True or False.",
-                    obj=self,
-                    id="fields.E006",
-                )
-            ]
-        else:
-            return []
-
-    def _check_db_comment(self, databases=None, **kwargs):
-        if not self.db_comment or not databases:
+    def _check_db_comment(self, database=False, **kwargs):
+        if not self.db_comment or not database:
             return []
         errors = []
-        for db in databases:
-            if not router.allow_migrate_model(db, self.model):
-                continue
-            connection = connections[db]
-            if not (
-                connection.features.supports_comments
-                or "supports_comments" in self.model._meta.required_db_features
-            ):
-                errors.append(
-                    preflight.Warning(
-                        f"{connection.display_name} does not support comments on "
-                        f"columns (db_comment).",
-                        obj=self,
-                        id="fields.W163",
-                    )
+        if not (
+            db_connection.features.supports_comments
+            or "supports_comments" in self.model._meta.required_db_features
+        ):
+            errors.append(
+                preflight.Warning(
+                    f"{db_connection.display_name} does not support comments on "
+                    f"columns (db_comment).",
+                    obj=self,
+                    id="fields.W163",
                 )
+            )
         return errors
 
     def _check_null_allowed_for_primary_keys(self):
-        if (
-            self.primary_key
-            and self.null
-            and not connection.features.interprets_empty_strings_as_nulls
-        ):
+        if self.primary_key and self.allow_null:
             # We cannot reliably check this for backends like Oracle which
             # consider NULL and '' to be equal (and thus set up
             # character-based fields a little differently).
             return [
                 preflight.Error(
-                    "Primary keys must not have null=True.",
+                    "Primary keys must not have allow_null=True.",
                     hint=(
-                        "Set null=False on the field, or "
+                        "Set allow_null=False on the field, or "
                         "remove primary_key=True argument."
                     ),
                     obj=self,
@@ -409,13 +383,11 @@ class Field(RegisterLookupMixin):
         else:
             return []
 
-    def _check_backend_specific_checks(self, databases=None, **kwargs):
-        if databases is None:
+    def _check_backend_specific_checks(self, database=False, **kwargs):
+        if not database:
             return []
         errors = []
-        for alias in databases:
-            if router.allow_migrate_model(alias, self.model):
-                errors.extend(connections[alias].validation.check_field(self, **kwargs))
+        errors.extend(db_connection.validation.check_field(self, **kwargs))
         return errors
 
     def _check_validators(self):
@@ -441,8 +413,8 @@ class Field(RegisterLookupMixin):
                 preflight.Error(
                     self.system_check_removed_details.get(
                         "msg",
-                        "%s has been removed except for support in historical "
-                        "migrations." % self.__class__.__name__,
+                        f"{self.__class__.__name__} has been removed except for support in historical "
+                        "migrations.",
                     ),
                     hint=self.system_check_removed_details.get("hint"),
                     obj=self,
@@ -453,7 +425,7 @@ class Field(RegisterLookupMixin):
             return [
                 preflight.Warning(
                     self.system_check_deprecated_details.get(
-                        "msg", "%s has been deprecated." % self.__class__.__name__
+                        "msg", f"{self.__class__.__name__} has been deprecated."
                     ),
                     hint=self.system_check_deprecated_details.get("hint"),
                     obj=self,
@@ -522,25 +494,19 @@ class Field(RegisterLookupMixin):
         possibles = {
             "primary_key": False,
             "max_length": None,
-            "unique": False,
-            "blank": False,
-            "null": False,
-            "db_index": False,
+            "required": True,
+            "allow_null": False,
             "default": NOT_PROVIDED,
-            "editable": True,
             "choices": None,
             "db_column": None,
             "db_comment": None,
-            "db_tablespace": None,
             "auto_created": False,
             "validators": [],
             "error_messages": None,
         }
         attr_overrides = {
-            "unique": "_unique",
             "error_messages": "_error_messages",
             "validators": "_validators",
-            "db_tablespace": "_db_tablespace",
         }
         equals_comparison = {"choices", "validators"}
         for name, default in possibles.items():
@@ -705,9 +671,6 @@ class Field(RegisterLookupMixin):
         Validate value and raise ValidationError if necessary. Subclasses
         should override this to provide validation logic.
         """
-        if not self.editable:
-            # Skip validation for non-editable fields.
-            return
 
         if self.choices is not None and value not in self.empty_values:
             for option_key, option_value in self.choices:
@@ -725,11 +688,15 @@ class Field(RegisterLookupMixin):
                 params={"value": value},
             )
 
-        if value is None and not self.null:
-            raise exceptions.ValidationError(self.error_messages["null"], code="null")
+        if value is None and not self.allow_null:
+            raise exceptions.ValidationError(
+                self.error_messages["allow_null"], code="allow_null"
+            )
 
-        if not self.blank and value in self.empty_values:
-            raise exceptions.ValidationError(self.error_messages["blank"], code="blank")
+        if self.required and value in self.empty_values:
+            raise exceptions.ValidationError(
+                self.error_messages["required"], code="required"
+            )
 
     def clean(self, value, model_instance):
         """
@@ -793,8 +760,7 @@ class Field(RegisterLookupMixin):
     def rel_db_type(self, connection):
         """
         Return the data type that a related field pointing to this field should
-        use. For example, this method is called by ForeignKey and OneToOneField
-        to determine its data type.
+        use. For example, this method is called by ForeignKey to determine its data type.
         """
         return self.db_type(connection)
 
@@ -826,14 +792,6 @@ class Field(RegisterLookupMixin):
             return [self.from_db_value]
         return []
 
-    @property
-    def unique(self):
-        return self._unique or self.primary_key
-
-    @property
-    def db_tablespace(self):
-        return self._db_tablespace or settings.DEFAULT_INDEX_TABLESPACE
-
     @property
     def db_returning(self):
         """
@@ -864,20 +822,13 @@ class Field(RegisterLookupMixin):
             # this class, but don't check methods derived from inheritance, to
             # allow overriding inherited choices. For more complex inheritance
             # structures users should override contribute_to_class().
-            if "get_%s_display" % self.name not in cls.__dict__:
+            if f"get_{self.name}_display" not in cls.__dict__:
                 setattr(
                     cls,
-                    "get_%s_display" % self.name,
+                    f"get_{self.name}_display",
                     partialmethod(cls._get_FIELD_display, field=self),
                 )
 
-    def get_filter_kwargs_for_object(self, obj):
-        """
-        Return a dict that when passed as kwargs to self.model.filter(), would
-        yield all instances having the same value for this field as obj has.
-        """
-        return {self.name: getattr(obj, self.attname)}
-
     def get_attname(self):
         return self.name
 
@@ -930,11 +881,7 @@ class Field(RegisterLookupMixin):
                 return self.default
             return lambda: self.default
 
-        if (
-            not self.empty_strings_allowed
-            or self.null
-            and not connection.features.interprets_empty_strings_as_nulls
-        ):
+        if not self.empty_strings_allowed or self.allow_null:
             return return_None
         return str  # return empty string
 
@@ -1013,7 +960,7 @@ class BooleanField(Field):
         return "BooleanField"
 
     def to_python(self, value):
-        if self.null and value in self.empty_values:
+        if self.allow_null and value in self.empty_values:
             return None
         if value in (True, False):
             # 1/0 are equal to True/False. bool() converts former to latter.
@@ -1023,7 +970,7 @@ class BooleanField(Field):
         if value in ("f", "False", "0"):
             return False
         raise exceptions.ValidationError(
-            self.error_messages["invalid_nullable" if self.null else "invalid"],
+            self.error_messages["invalid_nullable" if self.allow_null else "invalid"],
             code="invalid",
             params={"value": value},
         )
@@ -1036,8 +983,8 @@ class BooleanField(Field):
 
 
 class CharField(Field):
-    def __init__(self, *args, db_collation=None, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, *, db_collation=None, **kwargs):
+        super().__init__(**kwargs)
         self.db_collation = db_collation
         if self.max_length is not None:
             self.validators.append(validators.MaxLengthValidator(self.max_length))
@@ -1050,17 +997,17 @@ class CharField(Field):
             return "String (unlimited)"
 
     def check(self, **kwargs):
-        databases = kwargs.get("databases") or []
+        database = kwargs.get("database", False)
         return [
             *super().check(**kwargs),
-            *self._check_db_collation(databases),
+            *self._check_db_collation(database),
             *self._check_max_length_attribute(**kwargs),
         ]
 
     def _check_max_length_attribute(self, **kwargs):
         if self.max_length is None:
             if (
-                connection.features.supports_unlimited_charfield
+                db_connection.features.supports_unlimited_charfield
                 or "supports_unlimited_charfield"
                 in self.model._meta.required_db_features
             ):
@@ -1087,26 +1034,22 @@ class CharField(Field):
         else:
             return []
 
-    def _check_db_collation(self, databases):
+    def _check_db_collation(self, database):
         errors = []
-        for db in databases:
-            if not router.allow_migrate_model(db, self.model):
-                continue
-            connection = connections[db]
-            if not (
-                self.db_collation is None
-                or "supports_collation_on_charfield"
-                in self.model._meta.required_db_features
-                or connection.features.supports_collation_on_charfield
-            ):
-                errors.append(
-                    preflight.Error(
-                        "%s does not support a database collation on "
-                        "CharFields." % connection.display_name,
-                        obj=self,
-                        id="fields.E190",
-                    ),
-                )
+        if database and not (
+            self.db_collation is None
+            or "supports_collation_on_charfield"
+            in self.model._meta.required_db_features
+            or db_connection.features.supports_collation_on_charfield
+        ):
+            errors.append(
+                preflight.Error(
+                    f"{db_connection.display_name} does not support a database collation on "
+                    "CharFields.",
+                    obj=self,
+                    id="fields.E190",
+                ),
+            )
         return errors
 
     def cast_db_type(self, connection):
@@ -1147,8 +1090,7 @@ class CommaSeparatedIntegerField(CharField):
             "historical migrations."
         ),
         "hint": (
-            "Use CharField(validators=[validate_comma_separated_integer_list]) "
-            "instead."
+            "Use CharField(validators=[validate_comma_separated_integer_list]) instead."
         ),
         "id": "fields.E901",
     }
@@ -1156,7 +1098,7 @@ class CommaSeparatedIntegerField(CharField):
 
 def _to_naive(value):
     if timezone.is_aware(value):
-        value = timezone.make_naive(value, datetime.timezone.utc)
+        value = timezone.make_naive(value, datetime.UTC)
     return value
 
 
@@ -1245,12 +1187,11 @@ class DateField(DateTimeCheckMixin, Field):
     }
     description = "Date (without time)"
 
-    def __init__(self, name=None, auto_now=False, auto_now_add=False, **kwargs):
+    def __init__(self, *, auto_now=False, auto_now_add=False, **kwargs):
         self.auto_now, self.auto_now_add = auto_now, auto_now_add
         if auto_now or auto_now_add:
-            kwargs["editable"] = False
-            kwargs["blank"] = True
-        super().__init__(name, **kwargs)
+            kwargs["required"] = False
+        super().__init__(**kwargs)
 
     def _check_fix_default_value(self):
         """
@@ -1278,8 +1219,7 @@ class DateField(DateTimeCheckMixin, Field):
         if self.auto_now_add:
             kwargs["auto_now_add"] = True
         if self.auto_now or self.auto_now_add:
-            del kwargs["editable"]
-            del kwargs["blank"]
+            del kwargs["required"]
         return name, path, args, kwargs
 
     def get_internal_type(self):
@@ -1289,7 +1229,7 @@ class DateField(DateTimeCheckMixin, Field):
         if value is None:
             return value
         if isinstance(value, datetime.datetime):
-            if settings.USE_TZ and timezone.is_aware(value):
+            if timezone.is_aware(value):
                 # Convert aware datetimes to the default time zone
                 # before casting them to dates (#17742).
                 default_timezone = timezone.get_default_timezone()
@@ -1325,17 +1265,17 @@ class DateField(DateTimeCheckMixin, Field):
 
     def contribute_to_class(self, cls, name, **kwargs):
         super().contribute_to_class(cls, name, **kwargs)
-        if not self.null:
+        if not self.allow_null:
             setattr(
                 cls,
-                "get_next_by_%s" % self.name,
+                f"get_next_by_{self.name}",
                 partialmethod(
                     cls._get_next_or_previous_by_FIELD, field=self, is_next=True
                 ),
             )
             setattr(
                 cls,
-                "get_previous_by_%s" % self.name,
+                f"get_previous_by_{self.name}",
                 partialmethod(
                     cls._get_next_or_previous_by_FIELD, field=self, is_next=False
                 ),
@@ -1391,20 +1331,19 @@ class DateTimeField(DateField):
             return value
         if isinstance(value, datetime.date):
             value = datetime.datetime(value.year, value.month, value.day)
-            if settings.USE_TZ:
-                # For backwards compatibility, interpret naive datetimes in
-                # local time. This won't work during DST change, but we can't
-                # do much about it, so we let the exceptions percolate up the
-                # call stack.
-                warnings.warn(
-                    "DateTimeField {}.{} received a naive datetime "
-                    "({}) while time zone support is active.".format(
-                        self.model.__name__, self.name, value
-                    ),
-                    RuntimeWarning,
-                )
-                default_timezone = timezone.get_default_timezone()
-                value = timezone.make_aware(value, default_timezone)
+
+            # For backwards compatibility, interpret naive datetimes in
+            # local time. This won't work during DST change, but we can't
+            # do much about it, so we let the exceptions percolate up the
+            # call stack.
+            warnings.warn(
+                f"DateTimeField {self.model.__name__}.{self.name} received a naive datetime "
+                f"({value}) while time zone support is active.",
+                RuntimeWarning,
+            )
+            default_timezone = timezone.get_default_timezone()
+            value = timezone.make_aware(value, default_timezone)
+
             return value
 
         try:
@@ -1449,7 +1388,7 @@ class DateTimeField(DateField):
     def get_prep_value(self, value):
         value = super().get_prep_value(value)
         value = self.to_python(value)
-        if value is not None and settings.USE_TZ and timezone.is_naive(value):
+        if value is not None and timezone.is_naive(value):
             # For backwards compatibility, interpret naive datetimes in local
             # time. This won't work during DST change, but we can't do much
             # about it, so we let the exceptions percolate up the call stack.
@@ -1486,13 +1425,13 @@ class DecimalField(Field):
 
     def __init__(
         self,
-        name=None,
+        *,
         max_digits=None,
         decimal_places=None,
         **kwargs,
     ):
         self.max_digits, self.decimal_places = max_digits, decimal_places
-        super().__init__(name, **kwargs)
+        super().__init__(**kwargs)
 
     def check(self, **kwargs):
         errors = super().check(**kwargs)
@@ -1681,10 +1620,10 @@ class EmailField(CharField):
     default_validators = [validators.validate_email]
     description = "Email address"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, **kwargs):
         # max_length=254 to be compliant with RFCs 3696 and 5321
         kwargs.setdefault("max_length", 254)
-        super().__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
     def deconstruct(self):
         name, path, args, kwargs = super().deconstruct()
@@ -1744,8 +1683,7 @@ class IntegerField(Field):
         if self.max_length is not None:
             return [
                 preflight.Warning(
-                    "'max_length' is ignored when used with %s."
-                    % self.__class__.__name__,
+                    f"'max_length' is ignored when used with {self.__class__.__name__}.",
                     hint="Remove 'max_length' from field",
                     obj=self,
                     id="fields.W122",
@@ -1756,10 +1694,10 @@ class IntegerField(Field):
     @cached_property
     def validators(self):
         # These validators can't be added at field initialization time since
-        # they're based on values retrieved from `connection`.
+        # they're based on values retrieved from the database connection.
         validators_ = super().validators
         internal_type = self.get_internal_type()
-        min_value, max_value = connection.ops.integer_field_range(internal_type)
+        min_value, max_value = db_connection.ops.integer_field_range(internal_type)
         if min_value is not None and not any(
             (
                 isinstance(validator, validators.MinValueValidator)
@@ -1821,7 +1759,6 @@ class IntegerField(Field):
 
 class BigIntegerField(IntegerField):
     description = "Big (8 byte) integer"
-    MAX_BIGINT = 9223372036854775807
 
     def get_internal_type(self):
         return "BigIntegerField"
@@ -1846,9 +1783,9 @@ class IPAddressField(Field):
         "id": "fields.E900",
     }
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, **kwargs):
         kwargs["max_length"] = 15
-        super().__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
     def deconstruct(self):
         name, path, args, kwargs = super().deconstruct()
@@ -1872,10 +1809,9 @@ class GenericIPAddressField(Field):
 
     def __init__(
         self,
-        name=None,
+        *,
         protocol="both",
         unpack_ipv4=False,
-        *args,
         **kwargs,
     ):
         self.unpack_ipv4 = unpack_ipv4
@@ -1886,19 +1822,21 @@ class GenericIPAddressField(Field):
         ) = validators.ip_address_validators(protocol, unpack_ipv4)
         self.default_error_messages["invalid"] = invalid_error_message
         kwargs["max_length"] = 39
-        super().__init__(name, *args, **kwargs)
+        super().__init__(**kwargs)
 
     def check(self, **kwargs):
         return [
             *super().check(**kwargs),
-            *self._check_blank_and_null_values(**kwargs),
+            *self._check_required_and_null_values(**kwargs),
         ]
 
-    def _check_blank_and_null_values(self, **kwargs):
-        if not getattr(self, "null", False) and getattr(self, "blank", False):
+    def _check_required_and_null_values(self, **kwargs):
+        if not getattr(self, "allow_null", False) and not getattr(
+            self, "required", True
+        ):
             return [
                 preflight.Error(
-                    "GenericIPAddressFields cannot have blank=True if null=False, "
+                    "GenericIPAddressFields cannot have required=False if allow_null=False, "
                     "as blank values are stored as nulls.",
                     obj=self,
                     id="fields.E150",
@@ -1956,22 +1894,21 @@ class NullBooleanField(BooleanField):
     description = "Boolean (Either True, False or None)"
     system_check_removed_details = {
         "msg": (
-            "NullBooleanField is removed except for support in historical "
-            "migrations."
+            "NullBooleanField is removed except for support in historical migrations."
         ),
-        "hint": "Use BooleanField(null=True) instead.",
+        "hint": "Use BooleanField(allow_null=True) instead.",
         "id": "fields.E903",
     }
 
-    def __init__(self, *args, **kwargs):
-        kwargs["null"] = True
-        kwargs["blank"] = True
-        super().__init__(*args, **kwargs)
+    def __init__(self, **kwargs):
+        kwargs["allow_null"] = True
+        kwargs["required"] = False
+        super().__init__(**kwargs)
 
     def deconstruct(self):
         name, path, args, kwargs = super().deconstruct()
-        del kwargs["null"]
-        del kwargs["blank"]
+        del kwargs["allow_null"]
+        del kwargs["required"]
         return name, path, args, kwargs
 
 
@@ -2024,68 +1961,36 @@ class PositiveSmallIntegerField(PositiveIntegerRelDbTypeMixin, SmallIntegerField
         return "PositiveSmallIntegerField"
 
 
-class SlugField(CharField):
-    default_validators = [validators.validate_slug]
-    description = "Slug (up to %(max_length)s)"
-
-    def __init__(
-        self, *args, max_length=50, db_index=True, allow_unicode=False, **kwargs
-    ):
-        self.allow_unicode = allow_unicode
-        if self.allow_unicode:
-            self.default_validators = [validators.validate_unicode_slug]
-        super().__init__(*args, max_length=max_length, db_index=db_index, **kwargs)
-
-    def deconstruct(self):
-        name, path, args, kwargs = super().deconstruct()
-        if kwargs.get("max_length") == 50:
-            del kwargs["max_length"]
-        if self.db_index is False:
-            kwargs["db_index"] = False
-        else:
-            del kwargs["db_index"]
-        if self.allow_unicode is not False:
-            kwargs["allow_unicode"] = self.allow_unicode
-        return name, path, args, kwargs
-
-    def get_internal_type(self):
-        return "SlugField"
-
-
 class TextField(Field):
     description = "Text"
 
-    def __init__(self, *args, db_collation=None, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, *, db_collation=None, **kwargs):
+        super().__init__(**kwargs)
         self.db_collation = db_collation
 
     def check(self, **kwargs):
-        databases = kwargs.get("databases") or []
+        database = kwargs.get("database", False)
         return [
             *super().check(**kwargs),
-            *self._check_db_collation(databases),
+            *self._check_db_collation(database),
         ]
 
-    def _check_db_collation(self, databases):
+    def _check_db_collation(self, database):
         errors = []
-        for db in databases:
-            if not router.allow_migrate_model(db, self.model):
-                continue
-            connection = connections[db]
-            if not (
-                self.db_collation is None
-                or "supports_collation_on_textfield"
-                in self.model._meta.required_db_features
-                or connection.features.supports_collation_on_textfield
-            ):
-                errors.append(
-                    preflight.Error(
-                        "%s does not support a database collation on "
-                        "TextFields." % connection.display_name,
-                        obj=self,
-                        id="fields.E190",
-                    ),
-                )
+        if database and not (
+            self.db_collation is None
+            or "supports_collation_on_textfield"
+            in self.model._meta.required_db_features
+            or db_connection.features.supports_collation_on_textfield
+        ):
+            errors.append(
+                preflight.Error(
+                    f"{db_connection.display_name} does not support a database collation on "
+                    "TextFields.",
+                    obj=self,
+                    id="fields.E190",
+                ),
+            )
         return errors
 
     def db_parameters(self, connection):
@@ -2120,12 +2025,11 @@ class TimeField(DateTimeCheckMixin, Field):
     }
     description = "Time"
 
-    def __init__(self, name=None, auto_now=False, auto_now_add=False, **kwargs):
+    def __init__(self, *, auto_now=False, auto_now_add=False, **kwargs):
         self.auto_now, self.auto_now_add = auto_now, auto_now_add
         if auto_now or auto_now_add:
-            kwargs["editable"] = False
-            kwargs["blank"] = True
-        super().__init__(name, **kwargs)
+            kwargs["required"] = False
+        super().__init__(**kwargs)
 
     def _check_fix_default_value(self):
         """
@@ -2156,8 +2060,7 @@ class TimeField(DateTimeCheckMixin, Field):
         if self.auto_now_add is not False:
             kwargs["auto_now_add"] = self.auto_now_add
         if self.auto_now or self.auto_now_add:
-            del kwargs["blank"]
-            del kwargs["editable"]
+            del kwargs["required"]
         return name, path, args, kwargs
 
     def get_internal_type(self):
@@ -2218,9 +2121,9 @@ class URLField(CharField):
     default_validators = [validators.URLValidator()]
     description = "URL"
 
-    def __init__(self, name=None, **kwargs):
+    def __init__(self, **kwargs):
         kwargs.setdefault("max_length", 200)
-        super().__init__(name, **kwargs)
+        super().__init__(**kwargs)
 
     def deconstruct(self):
         name, path, args, kwargs = super().deconstruct()
@@ -2233,9 +2136,8 @@ class BinaryField(Field):
     description = "Raw binary data"
     empty_values = [None, b""]
 
-    def __init__(self, *args, **kwargs):
-        kwargs.setdefault("editable", False)
-        super().__init__(*args, **kwargs)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         if self.max_length is not None:
             self.validators.append(validators.MaxLengthValidator(self.max_length))
 
@@ -2254,14 +2156,6 @@ class BinaryField(Field):
             ]
         return []
 
-    def deconstruct(self):
-        name, path, args, kwargs = super().deconstruct()
-        if self.editable:
-            kwargs["editable"] = True
-        else:
-            del kwargs["editable"]
-        return name, path, args, kwargs
-
     def get_internal_type(self):
         return "BinaryField"
 
@@ -2344,7 +2238,7 @@ class AutoFieldMixin:
     db_returning = True
 
     def __init__(self, *args, **kwargs):
-        kwargs["blank"] = True
+        kwargs["required"] = False
         super().__init__(*args, **kwargs)
 
     def check(self, **kwargs):
@@ -2367,7 +2261,7 @@ class AutoFieldMixin:
 
     def deconstruct(self):
         name, path, args, kwargs = super().deconstruct()
-        del kwargs["blank"]
+        del kwargs["required"]
         kwargs["primary_key"] = True
         return name, path, args, kwargs
 
@@ -2383,8 +2277,7 @@ class AutoFieldMixin:
     def contribute_to_class(self, cls, name, **kwargs):
         if cls._meta.auto_field:
             raise ValueError(
-                "Model %s can't have more than one auto-generated field."
-                % cls._meta.label
+                f"Model {cls._meta.label} can't have more than one auto-generated field."
             )
         super().contribute_to_class(cls, name, **kwargs)
         cls._meta.auto_field = self
