@@ -2,6 +2,7 @@
 Helper functions for creating Form classes from Plain models
 and database field objects.
 """
+
 from itertools import chain
 
 from plain.exceptions import (
@@ -13,7 +14,6 @@ from plain.exceptions import (
 from plain.forms import fields
 from plain.forms.fields import ChoiceField, Field
 from plain.forms.forms import BaseForm, DeclarativeFieldsMetaclass
-from plain.models.utils import AltersData
 
 __all__ = (
     "ModelForm",
@@ -37,11 +37,7 @@ def construct_instance(form, instance, fields=None):
     cleaned_data = form.cleaned_data
     file_field_list = []
     for f in opts.fields:
-        if (
-            not f.editable
-            or isinstance(f, models.AutoField)
-            or f.name not in cleaned_data
-        ):
+        if isinstance(f, models.AutoField) or f.name not in cleaned_data:
             continue
         if fields is not None and f.name not in fields:
             continue
@@ -79,9 +75,7 @@ def model_to_dict(instance, fields=None):
     """
     opts = instance._meta
     data = {}
-    for f in chain(opts.concrete_fields, opts.private_fields, opts.many_to_many):
-        if not getattr(f, "editable", False):
-            continue
+    for f in chain(opts.concrete_fields, opts.many_to_many):
         if fields is not None and f.name not in fields:
             continue
         data[f.name] = f.value_from_object(instance)
@@ -113,22 +107,8 @@ def fields_for_model(
     field_dict = {}
     ignored = []
     opts = model._meta
-    # Avoid circular import
-    from plain.models.fields import Field as ModelField
 
-    sortable_private_fields = [
-        f for f in opts.private_fields if isinstance(f, ModelField)
-    ]
-    for f in sorted(
-        chain(opts.concrete_fields, sortable_private_fields, opts.many_to_many)
-    ):
-        if not getattr(f, "editable", False):
-            if fields is not None and f.name in fields:
-                raise FieldError(
-                    "'{}' cannot be specified for {} model form as it is a "
-                    "non-editable field".format(f.name, model.__name__)
-                )
-            continue
+    for f in sorted(chain(opts.concrete_fields, opts.many_to_many)):
         if fields is not None and f.name not in fields:
             continue
 
@@ -189,8 +169,8 @@ class ModelFormMetaclass(DeclarativeFieldsMetaclass):
             if opts.fields is None:
                 raise ImproperlyConfigured(
                     "Creating a ModelForm without the 'fields' attribute "
-                    "is prohibited; form %s "
-                    "needs updating." % name
+                    f"is prohibited; form {name} "
+                    "needs updating."
                 )
 
             fields = fields_for_model(
@@ -219,11 +199,11 @@ class ModelFormMetaclass(DeclarativeFieldsMetaclass):
         return new_class
 
 
-class BaseModelForm(BaseForm, AltersData):
+class BaseModelForm(BaseForm):
     def __init__(
         self,
-        data=None,
-        files=None,
+        *,
+        request,
         auto_id="id_%s",
         prefix=None,
         initial=None,
@@ -247,11 +227,10 @@ class BaseModelForm(BaseForm, AltersData):
         # super will stop validate_unique from being called.
         self._validate_unique = False
         super().__init__(
-            data,
-            files,
-            auto_id,
-            prefix,
-            object_data,
+            request=request,
+            auto_id=auto_id,
+            prefix=prefix,
+            initial=object_data,
         )
 
     def _get_validation_exclusions(self):
@@ -290,7 +269,7 @@ class BaseModelForm(BaseForm, AltersData):
                 form_field = self.fields[field]
                 field_value = self.cleaned_data.get(field)
                 if (
-                    not f.blank
+                    f.required
                     and not form_field.required
                     and field_value in form_field.empty_values
                 ):
@@ -382,10 +361,8 @@ class BaseModelForm(BaseForm, AltersData):
         cleaned_data = self.cleaned_data
         fields = self._meta.fields
         opts = self.instance._meta
-        # Note that for historical reasons we want to include also
-        # private_fields here. (GenericRelation was previously a fake
-        # m2m field).
-        for f in chain(opts.many_to_many, opts.private_fields):
+
+        for f in opts.many_to_many:
             if not hasattr(f, "save_form_data"):
                 continue
             if fields and f.name not in fields:
@@ -416,8 +393,6 @@ class BaseModelForm(BaseForm, AltersData):
             self.save_m2m = self._save_m2m
         return self.instance
 
-    save.alters_data = True
-
 
 class ModelForm(BaseModelForm, metaclass=ModelFormMetaclass):
     pass
@@ -535,7 +510,6 @@ class ModelChoiceField(ChoiceField):
         required=True,
         initial=None,
         to_field_name=None,
-        blank=False,
         **kwargs,
     ):
         # Call Field instead of ChoiceField __init__() because we don't need
@@ -613,8 +587,6 @@ class ModelChoiceField(ChoiceField):
         return Field.validate(self, value)
 
     def has_changed(self, initial, data):
-        if self.disabled:
-            return False
         initial_value = initial if initial is not None else ""
         data_value = data if data is not None else ""
         return str(self.prepare_value(initial_value)) != str(data_value)
@@ -680,7 +652,7 @@ class ModelMultipleChoiceField(ModelChoiceField):
                     code="invalid_pk_value",
                     params={"pk": pk},
                 )
-        qs = self.queryset.filter(**{"%s__in" % key: value})
+        qs = self.queryset.filter(**{f"{key}__in": value})
         pks = {str(getattr(o, key)) for o in qs}
         for val in value:
             if str(val) not in pks:
@@ -702,8 +674,6 @@ class ModelMultipleChoiceField(ModelChoiceField):
         return super().prepare_value(value)
 
     def has_changed(self, initial, data):
-        if self.disabled:
-            return False
         if initial is None:
             initial = []
         if data is None:
@@ -718,15 +688,11 @@ class ModelMultipleChoiceField(ModelChoiceField):
         return data.getlist(html_name)
 
 
-def modelform_defines_fields(form_class):
-    return hasattr(form_class, "_meta") and (form_class._meta.fields is not None)
-
-
 def modelfield_to_formfield(
     modelfield, form_class=None, choices_form_class=None, **kwargs
 ):
     defaults = {
-        "required": not modelfield.blank,
+        "required": modelfield.required,
     }
 
     if modelfield.has_default():
@@ -734,12 +700,12 @@ def modelfield_to_formfield(
 
     if modelfield.choices is not None:
         # Fields with choices get special treatment.
-        include_blank = modelfield.blank or not (
+        include_blank = not modelfield.required or not (
             modelfield.has_default() or "initial" in kwargs
         )
         defaults["choices"] = modelfield.get_choices(include_blank=include_blank)
         defaults["coerce"] = modelfield.to_python
-        if modelfield.null:
+        if modelfield.allow_null:
             defaults["empty_value"] = None
         if choices_form_class is not None:
             form_class = choices_form_class
@@ -756,7 +722,6 @@ def modelfield_to_formfield(
                 "required",
                 "initial",
                 "error_messages",
-                "disabled",
             ):
                 del kwargs[k]
 
@@ -775,7 +740,9 @@ def modelfield_to_formfield(
         return None
 
     if isinstance(modelfield, models.BooleanField):
-        form_class = fields.NullBooleanField if modelfield.null else fields.BooleanField
+        form_class = (
+            fields.NullBooleanField if modelfield.allow_null else fields.BooleanField
+        )
         # In HTML checkboxes, 'required' means "must be checked" which is
         # different from the choices case ("must select some value").
         # required=False allows unchecked checkboxes.
@@ -792,9 +759,6 @@ def modelfield_to_formfield(
     if issubclass(modelfield.__class__, models.fields.PositiveIntegerRelDbTypeMixin):
         return fields.IntegerField(min_value=0, **defaults)
 
-    if isinstance(modelfield, models.SlugField):
-        return fields.SlugField(allow_unicode=modelfield.allow_unicode, **defaults)
-
     if isinstance(modelfield, models.TextField):
         # Passing max_length to fields.CharField means that the value's length
         # will be validated twice. This is considered acceptable since we want
@@ -805,13 +769,7 @@ def modelfield_to_formfield(
         # Passing max_length to forms.CharField means that the value's length
         # will be validated twice. This is considered acceptable since we want
         # the value in the form field (to pass into widget for example).
-        # TODO: Handle multiple backends with different feature flags.
-        from plain.models.db import connection
-
-        if (
-            modelfield.null
-            and not connection.features.interprets_empty_strings_as_nulls
-        ):
+        if modelfield.allow_null:
             defaults["empty_value"] = None
         return fields.CharField(
             max_length=modelfield.max_length,
@@ -827,7 +785,6 @@ def modelfield_to_formfield(
         return ModelChoiceField(
             queryset=modelfield.remote_field.model._default_manager,
             to_field_name=modelfield.remote_field.field_name,
-            blank=modelfield.blank,
             **defaults,
         )
 
