@@ -1,11 +1,10 @@
 import functools
 import re
 from graphlib import TopologicalSorter
-from itertools import chain
 
 from plain import models
 from plain.models.migrations import operations
-from plain.models.migrations.migration import Migration
+from plain.models.migrations.migration import Migration, SettingsTuple
 from plain.models.migrations.operations.models import AlterModelOptions
 from plain.models.migrations.optimizer import MigrationOptimizer
 from plain.models.migrations.questioner import MigrationQuestioner
@@ -128,19 +127,13 @@ class MigrationAutodetector:
 
         # Prepare some old/new state and model lists, ignoring unmigrated packages.
         self.old_model_keys = set()
-        self.old_unmanaged_keys = set()
         self.new_model_keys = set()
-        self.new_unmanaged_keys = set()
         for (package_label, model_name), model_state in self.from_state.models.items():
-            if not model_state.options.get("managed", True):
-                self.old_unmanaged_keys.add((package_label, model_name))
-            elif package_label not in self.from_state.real_packages:
+            if package_label not in self.from_state.real_packages:
                 self.old_model_keys.add((package_label, model_name))
 
         for (package_label, model_name), model_state in self.to_state.models.items():
-            if not model_state.options.get("managed", True):
-                self.new_unmanaged_keys.add((package_label, model_name))
-            elif package_label not in self.from_state.real_packages or (
+            if package_label not in self.from_state.real_packages or (
                 convert_packages and package_label in convert_packages
             ):
                 self.new_model_keys.add((package_label, model_name))
@@ -182,7 +175,6 @@ class MigrationAutodetector:
         self.generate_removed_fields()
         self.generate_added_fields()
         self.generate_altered_fields()
-        self.generate_altered_order_with_respect_to()
         self.generate_added_indexes()
         self.generate_added_constraints()
         self.generate_altered_db_table()
@@ -200,7 +192,6 @@ class MigrationAutodetector:
         deletion to the field that uses it.
         """
         self.kept_model_keys = self.old_model_keys & self.new_model_keys
-        self.kept_unmanaged_keys = self.old_unmanaged_keys & self.new_unmanaged_keys
         self.through_users = {}
         self.old_field_keys = {
             (package_label, model_name, field_name)
@@ -240,9 +231,9 @@ class MigrationAutodetector:
     def _resolve_dependency(dependency):
         """
         Return the resolved dependency and a boolean denoting whether or not
-        it was swappable.
+        it was a settings dependency.
         """
-        if dependency[0] != "__setting__":
+        if not isinstance(dependency, SettingsTuple):
             return dependency, False
         resolved_package_label, resolved_object_name = getattr(
             settings, dependency[1]
@@ -279,12 +270,12 @@ class MigrationAutodetector:
                     deps_satisfied = True
                     operation_dependencies = set()
                     for dep in operation._auto_deps:
-                        # Temporarily resolve the swappable dependency to
+                        # Temporarily resolve the settings dependency to
                         # prevent circular references. While keeping the
                         # dependency checks on the resolved model, add the
-                        # swappable dependencies.
+                        # settings dependencies.
                         original_dep = dep
-                        dep, is_swappable_dep = self._resolve_dependency(dep)
+                        dep, is_settings_dep = self._resolve_dependency(dep)
                         if dep[0] != package_label:
                             # External app dependency. See if it's not yet
                             # satisfied.
@@ -297,7 +288,7 @@ class MigrationAutodetector:
                             if not deps_satisfied:
                                 break
                             else:
-                                if is_swappable_dep:
+                                if is_settings_dep:
                                     operation_dependencies.add(
                                         (original_dep[0], original_dep[1])
                                     )
@@ -342,7 +333,7 @@ class MigrationAutodetector:
                             {"operations": [], "dependencies": []},
                         )
                         instance = subclass(
-                            "auto_%i"
+                            "auto_%i"  # noqa: UP031
                             % (len(self.migrations.get(package_label, [])) + 1),
                             package_label,
                         )
@@ -361,8 +352,7 @@ class MigrationAutodetector:
                     chop_mode = True
                 else:
                     raise ValueError(
-                        "Cannot resolve operation dependencies: %r"
-                        % self.generated_operations
+                        f"Cannot resolve operation dependencies: {self.generated_operations!r}"
                     )
             num_ops = new_num_ops
 
@@ -377,7 +367,7 @@ class MigrationAutodetector:
                 ts.add(op)
                 for dep in op._auto_deps:
                     # Resolve intra-app dependencies to handle circular
-                    # references involving a swappable model.
+                    # references involving a settings model.
                     dep = self._resolve_dependency(dep)[0]
                     if dep[0] != package_label:
                         continue
@@ -444,14 +434,6 @@ class MigrationAutodetector:
                 and operation.model_name_lower == dependency[1].lower()
                 and operation.name_lower == dependency[2].lower()
             )
-        # order_with_respect_to being unset for a field
-        elif dependency[2] is not None and dependency[3] == "order_wrt_unset":
-            return (
-                isinstance(operation, operations.AlterOrderWithRespectTo)
-                and operation.name_lower == dependency[1].lower()
-                and (operation.order_with_respect_to or "").lower()
-                != dependency[2].lower()
-            )
         # Unknown dependency. Raise an error.
         else:
             raise ValueError(f"Can't handle dependency {dependency!r}")
@@ -467,29 +449,6 @@ class MigrationAutodetector:
         else:
             self.generated_operations.setdefault(package_label, []).append(operation)
 
-    def swappable_first_key(self, item):
-        """
-        Place potential swappable models first in lists of created models (only
-        real way to solve #22783).
-        """
-        try:
-            model_state = self.to_state.models[item]
-            base_names = {
-                base if isinstance(base, str) else base.__name__
-                for base in model_state.bases
-            }
-            string_version = f"{item[0]}.{item[1]}"
-            if (
-                model_state.options.get("swappable")
-                or "BaseUser" in base_names
-                or "AbstractBaseUser" in base_names
-                or settings.AUTH_USER_MODEL.lower() == string_version.lower()
-            ):
-                return ("___" + item[0], "___" + item[1])
-        except LookupError:
-            pass
-        return item
-
     def generate_renamed_models(self):
         """
         Find any renamed models, generate the operations for them, and remove
@@ -542,16 +501,13 @@ class MigrationAutodetector:
                                 ),
                                 dependencies=dependencies,
                             )
-                            self.renamed_models[
-                                package_label, model_name
-                            ] = rem_model_name
-                            renamed_models_rel_key = "{}.{}".format(
-                                rem_model_state.package_label,
-                                rem_model_state.name_lower,
+                            self.renamed_models[package_label, model_name] = (
+                                rem_model_name
+                            )
+                            renamed_models_rel_key = f"{rem_model_state.package_label}.{rem_model_state.name_lower}"
+                            self.renamed_models_rel[renamed_models_rel_key] = (
+                                f"{model_state.package_label}.{model_state.name_lower}"
                             )
-                            self.renamed_models_rel[
-                                renamed_models_rel_key
-                            ] = f"{model_state.package_label}.{model_state.name_lower}"
                             self.old_model_keys.remove(
                                 (rem_package_label, rem_model_name)
                             )
@@ -560,7 +516,7 @@ class MigrationAutodetector:
 
     def generate_created_models(self):
         """
-        Find all new models (both managed and unmanaged) and make create
+        Find all new models and make create
         operations for them as well as separate operations to create any
         foreign key or M2M relationships (these are optimized later, if
         possible).
@@ -568,14 +524,9 @@ class MigrationAutodetector:
         Defer any model options that refer to collections of fields that might
         be deferred.
         """
-        old_keys = self.old_model_keys | self.old_unmanaged_keys
-        added_models = self.new_model_keys - old_keys
-        added_unmanaged_models = self.new_unmanaged_keys - old_keys
-        all_added_models = chain(
-            sorted(added_models, key=self.swappable_first_key, reverse=True),
-            sorted(added_unmanaged_models, key=self.swappable_first_key, reverse=True),
-        )
-        for package_label, model_name in all_added_models:
+        added_models = self.new_model_keys - self.old_model_keys
+
+        for package_label, model_name in added_models:
             model_state = self.to_state.models[package_label, model_name]
             # Gather related fields
             related_fields = {}
@@ -593,9 +544,6 @@ class MigrationAutodetector:
             # Are there indexes to defer?
             indexes = model_state.options.pop("indexes")
             constraints = model_state.options.pop("constraints")
-            order_with_respect_to = model_state.options.pop(
-                "order_with_respect_to", None
-            )
             # Depend on the deletion of any possible proxy version of us
             dependencies = [
                 (package_label, model_name, None, False),
@@ -658,10 +606,6 @@ class MigrationAutodetector:
                 beginning=True,
             )
 
-            # Don't add operations which modify the database for unmanaged models
-            if not model_state.options.get("managed", True):
-                continue
-
             # Generate operations for each related field
             for name, field in sorted(related_fields.items()):
                 dependencies = self._get_dependencies_for_foreign_key(
@@ -682,19 +626,7 @@ class MigrationAutodetector:
                     ),
                     dependencies=list(set(dependencies)),
                 )
-            # Generate other opns
-            if order_with_respect_to:
-                self.add_operation(
-                    package_label,
-                    operations.AlterOrderWithRespectTo(
-                        name=model_name,
-                        order_with_respect_to=order_with_respect_to,
-                    ),
-                    dependencies=[
-                        (package_label, model_name, order_with_respect_to, True),
-                        (package_label, model_name, None, True),
-                    ],
-                )
+
             related_dependencies = [
                 (package_label, model_name, name, True)
                 for name in sorted(related_fields)
@@ -721,7 +653,7 @@ class MigrationAutodetector:
 
     def generate_deleted_models(self):
         """
-        Find all deleted models (managed and unmanaged) and make delete
+        Find all deleted models and make delete
         operations for them as well as separate operations to delete any
         foreign key or M2M relationships (these are optimized later, if
         possible).
@@ -729,13 +661,9 @@ class MigrationAutodetector:
         Also bring forward removal of any model options that refer to
         collections of fields - the inverse of generate_created_models().
         """
-        new_keys = self.new_model_keys | self.new_unmanaged_keys
-        deleted_models = self.old_model_keys - new_keys
-        deleted_unmanaged_models = self.old_unmanaged_keys - new_keys
-        all_deleted_models = chain(
-            sorted(deleted_models), sorted(deleted_unmanaged_models)
-        )
-        for package_label, model_name in all_deleted_models:
+        deleted_models = self.old_model_keys - self.new_model_keys
+
+        for package_label, model_name in sorted(deleted_models):
             model_state = self.from_state.models[package_label, model_name]
             # Gather related fields
             related_fields = {}
@@ -924,10 +852,10 @@ class MigrationAutodetector:
         # which don't allow empty strings as default.
         time_fields = (models.DateField, models.DateTimeField, models.TimeField)
         preserve_default = (
-            field.null
+            field.allow_null
             or field.has_default()
             or field.many_to_many
-            or (field.blank and field.empty_strings_allowed)
+            or (not field.required and field.empty_strings_allowed)
             or (isinstance(field, time_fields) and field.auto_now)
         )
         if not preserve_default:
@@ -941,7 +869,7 @@ class MigrationAutodetector:
                     field_name, model_name
                 )
         if (
-            field.unique
+            field.primary_key
             and field.default is not models.NOT_PROVIDED
             and callable(field.default)
         ):
@@ -971,12 +899,6 @@ class MigrationAutodetector:
                 model_name=model_name,
                 name=field_name,
             ),
-            # We might need to depend on the removal of an
-            # order_with_respect_to or index operation;
-            # this is safely ignored if there isn't one
-            dependencies=[
-                (package_label, model_name, field_name, "order_wrt_unset"),
-            ],
         )
 
     def generate_altered_fields(self):
@@ -1069,8 +991,8 @@ class MigrationAutodetector:
                     # Either both fields are m2m or neither is
                     preserve_default = True
                     if (
-                        old_field.null
-                        and not new_field.null
+                        old_field.allow_null
+                        and not new_field.allow_null
                         and not new_field.has_default()
                         and not new_field.many_to_many
                     ):
@@ -1258,17 +1180,11 @@ class MigrationAutodetector:
                 ):
                     remote_field_model = f"{remote_package_label}.{remote_model_name}"
                     break
-        # Account for FKs to swappable models
-        swappable_setting = getattr(field, "swappable_setting", None)
-        if swappable_setting is not None:
-            dep_package_label = "__setting__"
-            dep_object_name = swappable_setting
-        else:
-            dep_package_label, dep_object_name = resolve_relation(
-                remote_field_model,
-                package_label,
-                model_name,
-            )
+        dep_package_label, dep_object_name = resolve_relation(
+            remote_field_model,
+            package_label,
+            model_name,
+        )
         dependencies = [(dep_package_label, dep_object_name, None, True)]
         if getattr(field.remote_field, "through", None):
             through_package_label, through_object_name = resolve_relation(
@@ -1297,74 +1213,8 @@ class MigrationAutodetector:
                 )
         return dependencies
 
-    def _get_altered_foo_together_operations(self, option_name):
-        for package_label, model_name in sorted(self.kept_model_keys):
-            old_model_name = self.renamed_models.get(
-                (package_label, model_name), model_name
-            )
-            old_model_state = self.from_state.models[package_label, old_model_name]
-            new_model_state = self.to_state.models[package_label, model_name]
-
-            # We run the old version through the field renames to account for those
-            old_value = old_model_state.options.get(option_name)
-            old_value = (
-                {
-                    tuple(
-                        self.renamed_fields.get((package_label, model_name, n), n)
-                        for n in unique
-                    )
-                    for unique in old_value
-                }
-                if old_value
-                else set()
-            )
-
-            new_value = new_model_state.options.get(option_name)
-            new_value = set(new_value) if new_value else set()
-
-            if old_value != new_value:
-                dependencies = []
-                for foo_togethers in new_value:
-                    for field_name in foo_togethers:
-                        field = new_model_state.get_field(field_name)
-                        if field.remote_field and field.remote_field.model:
-                            dependencies.extend(
-                                self._get_dependencies_for_foreign_key(
-                                    package_label,
-                                    model_name,
-                                    field,
-                                    self.to_state,
-                                )
-                            )
-                yield (
-                    old_value,
-                    new_value,
-                    package_label,
-                    model_name,
-                    dependencies,
-                )
-
-    def _generate_removed_altered_foo_together(self, operation):
-        for (
-            old_value,
-            new_value,
-            package_label,
-            model_name,
-            dependencies,
-        ) in self._get_altered_foo_together_operations(operation.option_name):
-            removal_value = new_value.intersection(old_value)
-            if removal_value or old_value:
-                self.add_operation(
-                    package_label,
-                    operation(
-                        name=model_name, **{operation.option_name: removal_value}
-                    ),
-                    dependencies=dependencies,
-                )
-
     def generate_altered_db_table(self):
-        models_to_check = self.kept_model_keys.union(self.kept_unmanaged_keys)
-        for package_label, model_name in sorted(models_to_check):
+        for package_label, model_name in sorted(self.kept_model_keys):
             old_model_name = self.renamed_models.get(
                 (package_label, model_name), model_name
             )
@@ -1382,8 +1232,7 @@ class MigrationAutodetector:
                 )
 
     def generate_altered_db_table_comment(self):
-        models_to_check = self.kept_model_keys.union(self.kept_unmanaged_keys)
-        for package_label, model_name in sorted(models_to_check):
+        for package_label, model_name in sorted(self.kept_model_keys):
             old_model_name = self.renamed_models.get(
                 (package_label, model_name), model_name
             )
@@ -1407,15 +1256,7 @@ class MigrationAutodetector:
         operation to represent them in state changes (in case Python code in
         migrations needs them).
         """
-        models_to_check = self.kept_model_keys.union(
-            self.kept_unmanaged_keys,
-            # unmanaged converted to managed
-            self.old_unmanaged_keys & self.new_model_keys,
-            # managed converted to unmanaged
-            self.old_model_keys & self.new_unmanaged_keys,
-        )
-
-        for package_label, model_name in sorted(models_to_check):
+        for package_label, model_name in sorted(self.kept_model_keys):
             old_model_name = self.renamed_models.get(
                 (package_label, model_name), model_name
             )
@@ -1440,40 +1281,6 @@ class MigrationAutodetector:
                     ),
                 )
 
-    def generate_altered_order_with_respect_to(self):
-        for package_label, model_name in sorted(self.kept_model_keys):
-            old_model_name = self.renamed_models.get(
-                (package_label, model_name), model_name
-            )
-            old_model_state = self.from_state.models[package_label, old_model_name]
-            new_model_state = self.to_state.models[package_label, model_name]
-            if old_model_state.options.get(
-                "order_with_respect_to"
-            ) != new_model_state.options.get("order_with_respect_to"):
-                # Make sure it comes second if we're adding
-                # (removal dependency is part of RemoveField)
-                dependencies = []
-                if new_model_state.options.get("order_with_respect_to"):
-                    dependencies.append(
-                        (
-                            package_label,
-                            model_name,
-                            new_model_state.options["order_with_respect_to"],
-                            True,
-                        )
-                    )
-                # Actually generate the operation
-                self.add_operation(
-                    package_label,
-                    operations.AlterOrderWithRespectTo(
-                        name=model_name,
-                        order_with_respect_to=new_model_state.options.get(
-                            "order_with_respect_to"
-                        ),
-                    ),
-                    dependencies=dependencies,
-                )
-
     def generate_altered_managers(self):
         for package_label, model_name in sorted(self.kept_model_keys):
             old_model_name = self.renamed_models.get(
@@ -1526,7 +1333,7 @@ class MigrationAutodetector:
             for i, migration in enumerate(migrations):
                 if i == 0 and app_leaf:
                     migration.dependencies.append(app_leaf)
-                new_name_parts = ["%04i" % next_number]
+                new_name_parts = ["%04i" % next_number]  # noqa: UP031
                 if migration_name:
                     new_name_parts.append(migration_name)
                 elif i == 0 and not app_leaf:
