@@ -1,8 +1,9 @@
+from functools import cached_property
+
 from plain import models
 from plain.models.migrations.operations.base import Operation
 from plain.models.migrations.state import ModelState
 from plain.models.migrations.utils import field_references, resolve_relation
-from plain.utils.functional import cached_property
 
 from .fields import AddField, AlterField, FieldOperation, RemoveField, RenameField
 
@@ -90,15 +91,10 @@ class CreateModel(ModelOperation):
         )
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = to_state.models_registry.get_model(package_label, self.name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             schema_editor.create_model(model)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = from_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            schema_editor.delete_model(model)
-
     def describe(self):
         return f"Create model {self.name}"
 
@@ -178,22 +174,6 @@ class CreateModel(ModelOperation):
                     managers=operation.managers,
                 ),
             ]
-        elif (
-            isinstance(operation, AlterOrderWithRespectTo)
-            and self.name_lower == operation.name_lower
-        ):
-            return [
-                CreateModel(
-                    self.name,
-                    fields=self.fields,
-                    options={
-                        **self.options,
-                        "order_with_respect_to": operation.order_with_respect_to,
-                    },
-                    bases=self.bases,
-                    managers=self.managers,
-                ),
-            ]
         elif (
             isinstance(operation, FieldOperation)
             and self.name_lower == operation.model_name_lower
@@ -224,9 +204,6 @@ class CreateModel(ModelOperation):
             elif isinstance(operation, RemoveField):
                 options = self.options.copy()
 
-                order_with_respect_to = options.get("order_with_respect_to")
-                if order_with_respect_to == operation.name_lower:
-                    del options["order_with_respect_to"]
                 return [
                     CreateModel(
                         self.name,
@@ -243,9 +220,6 @@ class CreateModel(ModelOperation):
             elif isinstance(operation, RenameField):
                 options = self.options.copy()
 
-                order_with_respect_to = options.get("order_with_respect_to")
-                if order_with_respect_to == operation.old_name:
-                    options["order_with_respect_to"] = operation.new_name
                 return [
                     CreateModel(
                         self.name,
@@ -274,26 +248,21 @@ class DeleteModel(ModelOperation):
         state.remove_model(package_label, self.name_lower)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = from_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = from_state.models_registry.get_model(package_label, self.name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             schema_editor.delete_model(model)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            schema_editor.create_model(model)
-
     def references_model(self, name, package_label):
         # The deleted model could be referencing the specified model through
         # related fields.
         return True
 
     def describe(self):
-        return "Delete model %s" % self.name
+        return f"Delete model {self.name}"
 
     @property
     def migration_name_fragment(self):
-        return "delete_%s" % self.name_lower
+        return f"delete_{self.name_lower}"
 
 
 class RenameModel(ModelOperation):
@@ -323,9 +292,11 @@ class RenameModel(ModelOperation):
         state.rename_model(package_label, self.old_name, self.new_name)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        new_model = to_state.packages.get_model(package_label, self.new_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, new_model):
-            old_model = from_state.packages.get_model(package_label, self.old_name)
+        new_model = to_state.models_registry.get_model(package_label, self.new_name)
+        if self.allow_migrate_model(schema_editor.connection, new_model):
+            old_model = from_state.models_registry.get_model(
+                package_label, self.old_name
+            )
             # Move the main table
             schema_editor.alter_db_table(
                 new_model,
@@ -343,47 +314,14 @@ class RenameModel(ModelOperation):
                         related_object.related_model._meta.package_label,
                         related_object.related_model._meta.model_name,
                     )
-                to_field = to_state.packages.get_model(*related_key)._meta.get_field(
-                    related_object.field.name
-                )
+                to_field = to_state.models_registry.get_model(
+                    *related_key
+                )._meta.get_field(related_object.field.name)
                 schema_editor.alter_field(
                     model,
                     related_object.field,
                     to_field,
                 )
-            # Rename M2M fields whose name is based on this model's name.
-            fields = zip(
-                old_model._meta.local_many_to_many, new_model._meta.local_many_to_many
-            )
-            for old_field, new_field in fields:
-                # Skip self-referential fields as these are renamed above.
-                if (
-                    new_field.model == new_field.related_model
-                    or not new_field.remote_field.through._meta.auto_created
-                ):
-                    continue
-                # Rename columns and the M2M table.
-                schema_editor._alter_many_to_many(
-                    new_model,
-                    old_field,
-                    new_field,
-                    strict=False,
-                )
-
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        self.new_name_lower, self.old_name_lower = (
-            self.old_name_lower,
-            self.new_name_lower,
-        )
-        self.new_name, self.old_name = self.old_name, self.new_name
-
-        self.database_forwards(package_label, schema_editor, from_state, to_state)
-
-        self.new_name_lower, self.old_name_lower = (
-            self.old_name_lower,
-            self.new_name_lower,
-        )
-        self.new_name, self.old_name = self.old_name, self.new_name
 
     def references_model(self, name, package_label):
         return (
@@ -445,29 +383,14 @@ class AlterModelTable(ModelOptionOperation):
         )
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        new_model = to_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, new_model):
-            old_model = from_state.packages.get_model(package_label, self.name)
+        new_model = to_state.models_registry.get_model(package_label, self.name)
+        if self.allow_migrate_model(schema_editor.connection, new_model):
+            old_model = from_state.models_registry.get_model(package_label, self.name)
             schema_editor.alter_db_table(
                 new_model,
                 old_model._meta.db_table,
                 new_model._meta.db_table,
             )
-            # Rename M2M fields whose name is based on this model's db_table
-            for old_field, new_field in zip(
-                old_model._meta.local_many_to_many, new_model._meta.local_many_to_many
-            ):
-                if new_field.remote_field.through._meta.auto_created:
-                    schema_editor.alter_db_table(
-                        new_field.remote_field.through,
-                        old_field.remote_field.through._meta.db_table,
-                        new_field.remote_field.through._meta.db_table,
-                    )
-
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        return self.database_forwards(
-            package_label, schema_editor, from_state, to_state
-        )
 
     def describe(self):
         return "Rename table for {} to {}".format(
@@ -477,7 +400,7 @@ class AlterModelTable(ModelOptionOperation):
 
     @property
     def migration_name_fragment(self):
-        return "alter_%s_table" % self.name_lower
+        return f"alter_{self.name_lower}_table"
 
 
 class AlterModelTableComment(ModelOptionOperation):
@@ -498,20 +421,15 @@ class AlterModelTableComment(ModelOptionOperation):
         )
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        new_model = to_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, new_model):
-            old_model = from_state.packages.get_model(package_label, self.name)
+        new_model = to_state.models_registry.get_model(package_label, self.name)
+        if self.allow_migrate_model(schema_editor.connection, new_model):
+            old_model = from_state.models_registry.get_model(package_label, self.name)
             schema_editor.alter_db_table_comment(
                 new_model,
                 old_model._meta.db_table_comment,
                 new_model._meta.db_table_comment,
             )
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        return self.database_forwards(
-            package_label, schema_editor, from_state, to_state
-        )
-
     def describe(self):
         return f"Alter {self.name} table comment"
 
@@ -520,74 +438,6 @@ class AlterModelTableComment(ModelOptionOperation):
         return f"alter_{self.name_lower}_table_comment"
 
 
-class AlterOrderWithRespectTo(ModelOptionOperation):
-    """Represent a change with the order_with_respect_to option."""
-
-    option_name = "order_with_respect_to"
-
-    def __init__(self, name, order_with_respect_to):
-        self.order_with_respect_to = order_with_respect_to
-        super().__init__(name)
-
-    def deconstruct(self):
-        kwargs = {
-            "name": self.name,
-            "order_with_respect_to": self.order_with_respect_to,
-        }
-        return (self.__class__.__qualname__, [], kwargs)
-
-    def state_forwards(self, package_label, state):
-        state.alter_model_options(
-            package_label,
-            self.name_lower,
-            {self.option_name: self.order_with_respect_to},
-        )
-
-    def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        to_model = to_state.packages.get_model(package_label, self.name)
-        if self.allow_migrate_model(schema_editor.connection.alias, to_model):
-            from_model = from_state.packages.get_model(package_label, self.name)
-            # Remove a field if we need to
-            if (
-                from_model._meta.order_with_respect_to
-                and not to_model._meta.order_with_respect_to
-            ):
-                schema_editor.remove_field(
-                    from_model, from_model._meta.get_field("_order")
-                )
-            # Add a field if we need to (altering the column is untouched as
-            # it's likely a rename)
-            elif (
-                to_model._meta.order_with_respect_to
-                and not from_model._meta.order_with_respect_to
-            ):
-                field = to_model._meta.get_field("_order")
-                if not field.has_default():
-                    field.default = 0
-                schema_editor.add_field(
-                    from_model,
-                    field,
-                )
-
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        self.database_forwards(package_label, schema_editor, from_state, to_state)
-
-    def references_field(self, model_name, name, package_label):
-        return self.references_model(model_name, package_label) and (
-            self.order_with_respect_to is None or name == self.order_with_respect_to
-        )
-
-    def describe(self):
-        return "Set order_with_respect_to on {} to {}".format(
-            self.name,
-            self.order_with_respect_to,
-        )
-
-    @property
-    def migration_name_fragment(self):
-        return "alter_%s_order_with_respect_to" % self.name_lower
-
-
 class AlterModelOptions(ModelOptionOperation):
     """
     Set new model options that don't directly affect the database schema
@@ -601,9 +451,7 @@ class AlterModelOptions(ModelOptionOperation):
         "default_manager_name",
         "default_related_name",
         "get_latest_by",
-        "managed",
         "ordering",
-        "select_on_save",
     ]
 
     def __init__(self, name, options):
@@ -628,15 +476,12 @@ class AlterModelOptions(ModelOptionOperation):
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
         pass
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        pass
-
     def describe(self):
-        return "Change Meta options on %s" % self.name
+        return f"Change Meta options on {self.name}"
 
     @property
     def migration_name_fragment(self):
-        return "alter_%s_options" % self.name_lower
+        return f"alter_{self.name_lower}_options"
 
 
 class AlterModelManagers(ModelOptionOperation):
@@ -657,15 +502,12 @@ class AlterModelManagers(ModelOptionOperation):
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
         pass
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        pass
-
     def describe(self):
-        return "Change managers on %s" % self.name
+        return f"Change managers on {self.name}"
 
     @property
     def migration_name_fragment(self):
-        return "alter_%s_managers" % self.name_lower
+        return f"alter_{self.name_lower}_managers"
 
 
 class IndexOperation(Operation):
@@ -684,7 +526,7 @@ class AddIndex(IndexOperation):
         if not index.name:
             raise ValueError(
                 "Indexes passed to AddIndex operations require a name "
-                "argument. %r doesn't have one." % index
+                f"argument. {index!r} doesn't have one."
             )
         self.index = index
 
@@ -692,15 +534,10 @@ class AddIndex(IndexOperation):
         state.add_index(package_label, self.model_name_lower, self.index)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = to_state.models_registry.get_model(package_label, self.model_name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             schema_editor.add_index(model, self.index)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = from_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            schema_editor.remove_index(model, self.index)
-
     def deconstruct(self):
         kwargs = {
             "model_name": self.model_name,
@@ -741,19 +578,12 @@ class RemoveIndex(IndexOperation):
         state.remove_index(package_label, self.model_name_lower, self.name)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = from_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = from_state.models_registry.get_model(package_label, self.model_name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             from_model_state = from_state.models[package_label, self.model_name_lower]
             index = from_model_state.get_index_by_name(self.name)
             schema_editor.remove_index(model, index)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            to_model_state = to_state.models[package_label, self.model_name_lower]
-            index = to_model_state.get_index_by_name(self.name)
-            schema_editor.add_index(model, index)
-
     def deconstruct(self):
         kwargs = {
             "model_name": self.model_name,
@@ -823,12 +653,14 @@ class RenameIndex(IndexOperation):
             )
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if not self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = to_state.models_registry.get_model(package_label, self.model_name)
+        if not self.allow_migrate_model(schema_editor.connection, model):
             return
 
         if self.old_fields:
-            from_model = from_state.packages.get_model(package_label, self.model_name)
+            from_model = from_state.models_registry.get_model(
+                package_label, self.model_name
+            )
             columns = [
                 from_model._meta.get_field(field).column for field in self.old_fields
             ]
@@ -858,25 +690,6 @@ class RenameIndex(IndexOperation):
         new_index = to_model_state.get_index_by_name(self.new_name)
         schema_editor.rename_index(model, old_index, new_index)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        if self.old_fields:
-            # Backward operation with unnamed index is a no-op.
-            return
-
-        self.new_name_lower, self.old_name_lower = (
-            self.old_name_lower,
-            self.new_name_lower,
-        )
-        self.new_name, self.old_name = self.old_name, self.new_name
-
-        self.database_forwards(package_label, schema_editor, from_state, to_state)
-
-        self.new_name_lower, self.old_name_lower = (
-            self.old_name_lower,
-            self.new_name_lower,
-        )
-        self.new_name, self.old_name = self.old_name, self.new_name
-
     def describe(self):
         if self.old_name:
             return (
@@ -926,15 +739,10 @@ class AddConstraint(IndexOperation):
         state.add_constraint(package_label, self.model_name_lower, self.constraint)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = to_state.models_registry.get_model(package_label, self.model_name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             schema_editor.add_constraint(model, self.constraint)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            schema_editor.remove_constraint(model, self.constraint)
-
     def deconstruct(self):
         return (
             self.__class__.__name__,
@@ -964,19 +772,12 @@ class RemoveConstraint(IndexOperation):
         state.remove_constraint(package_label, self.model_name_lower, self.name)
 
     def database_forwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
+        model = to_state.models_registry.get_model(package_label, self.model_name)
+        if self.allow_migrate_model(schema_editor.connection, model):
             from_model_state = from_state.models[package_label, self.model_name_lower]
             constraint = from_model_state.get_constraint_by_name(self.name)
             schema_editor.remove_constraint(model, constraint)
 
-    def database_backwards(self, package_label, schema_editor, from_state, to_state):
-        model = to_state.packages.get_model(package_label, self.model_name)
-        if self.allow_migrate_model(schema_editor.connection.alias, model):
-            to_model_state = to_state.models[package_label, self.model_name_lower]
-            constraint = to_model_state.get_constraint_by_name(self.name)
-            schema_editor.add_constraint(model, constraint)
-
     def deconstruct(self):
         return (
             self.__class__.__name__,
