@@ -1,7 +1,7 @@
 import copy
 from collections import defaultdict
 from contextlib import contextmanager
-from functools import partial
+from functools import cached_property, partial
 
 from plain import models
 from plain.exceptions import FieldDoesNotExist
@@ -9,12 +9,9 @@ from plain.models.fields import NOT_PROVIDED
 from plain.models.fields.related import RECURSIVE_RELATIONSHIP_CONSTANT
 from plain.models.migrations.utils import field_is_referenced, get_references
 from plain.models.options import DEFAULT_NAMES
-from plain.models.utils import make_model_tuple
-from plain.packages import PackageConfig
-from plain.packages.registry import Packages
-from plain.packages.registry import packages as global_packages
-from plain.runtime import settings
-from plain.utils.functional import cached_property
+from plain.models.registry import ModelsRegistry
+from plain.models.registry import models_registry as global_models
+from plain.packages import packages_registry
 from plain.utils.module_loading import import_string
 
 from .exceptions import InvalidBasesError
@@ -37,7 +34,7 @@ def _get_related_models(m):
         if issubclass(subclass, models.Model)
     ]
     related_fields_models = set()
-    for f in m._meta.get_fields(include_parents=True, include_hidden=True):
+    for f in m._meta.get_fields(include_hidden=True):
         if (
             f.is_relation
             and f.related_model is not None
@@ -65,10 +62,8 @@ def get_related_models_recursive(model):
     to the given model.
 
     Relationships are either defined by explicit relational fields, like
-    ForeignKey, ManyToManyField or OneToOneField, or by inheriting from another
-    model (a superclass is related to its subclasses, but not vice versa). Note,
-    however, that a model inheriting from a concrete model is also related to
-    its superclass through the implicit *_ptr OneToOneField on the subclass.
+    ForeignKey or ManyToManyField, or by inheriting from another
+    model (a superclass is related to its subclasses, but not vice versa).
     """
     seen = set()
     queue = _get_related_models(model)
@@ -114,7 +109,7 @@ class ProjectState:
         self.models[model_key] = model_state
         if self._relations is not None:
             self.resolve_model_relations(model_key)
-        if "packages" in self.__dict__:  # hasattr would cache the property
+        if "models_registry" in self.__dict__:  # hasattr would cache the property
             self.reload_model(*model_key)
 
     def remove_model(self, package_label, model_name):
@@ -127,11 +122,11 @@ class ProjectState:
                 model_relations.pop(model_key, None)
                 if not model_relations:
                     del self._relations[related_model_key]
-        if "packages" in self.__dict__:  # hasattr would cache the property
-            self.packages.unregister_model(*model_key)
+        if "models_registry" in self.__dict__:  # hasattr would cache the property
+            self.models_registry.unregister_model(*model_key)
             # Need to do this explicitly since unregister_model() doesn't clear
             # the cache automatically (#24513)
-            self.packages.clear_cache()
+            self.models_registry.clear_cache()
 
     def rename_model(self, package_label, old_name, new_name):
         # Add a new model.
@@ -181,16 +176,6 @@ class ProjectState:
                     model_state.options.pop(key, False)
         self.reload_model(package_label, model_name, delay=True)
 
-    def remove_model_options(
-        self, package_label, model_name, option_name, value_to_remove
-    ):
-        model_state = self.models[package_label, model_name]
-        if objs := model_state.options.get(option_name):
-            model_state.options[option_name] = [
-                obj for obj in objs if tuple(obj) != tuple(value_to_remove)
-            ]
-        self.reload_model(package_label, model_name, delay=True)
-
     def alter_model_managers(self, package_label, model_name, managers):
         model_state = self.models[package_label, model_name]
         model_state.managers = list(managers)
@@ -340,12 +325,12 @@ class ProjectState:
         related_models = set()
 
         try:
-            old_model = self.packages.get_model(package_label, model_name)
+            old_model = self.models_registry.get_model(package_label, model_name)
         except LookupError:
             pass
         else:
             # Get all relations to and from the old model before reloading,
-            # as _meta.packages may change
+            # as _meta.models_registry may change
             if delay:
                 related_models = get_related_models_tuples(old_model)
             else:
@@ -353,8 +338,7 @@ class ProjectState:
 
         # Get all outgoing references from the model to be rendered
         model_state = self.models[(package_label, model_name)]
-        # Directly related models are the models pointed to by ForeignKeys,
-        # OneToOneFields, and ManyToManyFields.
+        # Directly related models are the models pointed to by ForeignKeys and ManyToManyFields.
         direct_related_models = set()
         for field in model_state.fields.values():
             if field.is_relation:
@@ -369,7 +353,9 @@ class ProjectState:
         related_models.update(direct_related_models)
         for rel_package_label, rel_model_name in direct_related_models:
             try:
-                rel_model = self.packages.get_model(rel_package_label, rel_model_name)
+                rel_model = self.models_registry.get_model(
+                    rel_package_label, rel_model_name
+                )
             except LookupError:
                 pass
             else:
@@ -384,12 +370,12 @@ class ProjectState:
         return related_models
 
     def reload_model(self, package_label, model_name, delay=False):
-        if "packages" in self.__dict__:  # hasattr would cache the property
+        if "models_registry" in self.__dict__:  # hasattr would cache the property
             related_models = self._find_reload_model(package_label, model_name, delay)
             self._reload(related_models)
 
     def reload_models(self, models, delay=True):
-        if "packages" in self.__dict__:  # hasattr would cache the property
+        if "models_registry" in self.__dict__:  # hasattr would cache the property
             related_models = set()
             for package_label, model_name in models:
                 related_models.update(
@@ -399,15 +385,15 @@ class ProjectState:
 
     def _reload(self, related_models):
         # Unregister all related models
-        with self.packages.bulk_update():
+        with self.models_registry.bulk_update():
             for rel_package_label, rel_model_name in related_models:
-                self.packages.unregister_model(rel_package_label, rel_model_name)
+                self.models_registry.unregister_model(rel_package_label, rel_model_name)
 
         states_to_be_rendered = []
         # Gather all models states of those models that will be rerendered.
         # This includes:
         # 1. All related models of unmigrated packages
-        for model_state in self.packages.real_models:
+        for model_state in self.models_registry.real_models:
             if (model_state.package_label, model_state.name_lower) in related_models:
                 states_to_be_rendered.append(model_state)
 
@@ -421,7 +407,7 @@ class ProjectState:
                 states_to_be_rendered.append(model_state)
 
         # Render all models
-        self.packages.render_multiple(states_to_be_rendered)
+        self.models_registry.render_multiple(states_to_be_rendered)
 
     def update_model_field_relation(
         self,
@@ -498,11 +484,6 @@ class ProjectState:
         for model_key in concretes:
             self.resolve_model_relations(model_key, concretes)
 
-    def get_concrete_model_key(self, model):
-        (concrete_models_mapping,) = self._get_concrete_models_mapping()
-        model_key = make_model_tuple(model)
-        return concrete_models_mapping[model_key]
-
     def _get_concrete_models_mapping(self):
         concrete_models_mapping = {}
         for model_key, model_state in self.models.items():
@@ -515,57 +496,41 @@ class ProjectState:
             models={k: v.clone() for k, v in self.models.items()},
             real_packages=self.real_packages,
         )
-        if "packages" in self.__dict__:
-            new_state.packages = self.packages.clone()
+        if "models_registry" in self.__dict__:
+            new_state.models_registry = self.models_registry.clone()
         new_state.is_delayed = self.is_delayed
         return new_state
 
-    def clear_delayed_packages_cache(self):
-        if self.is_delayed and "packages" in self.__dict__:
-            del self.__dict__["packages"]
+    def clear_delayed_models_cache(self):
+        if self.is_delayed and "models_registry" in self.__dict__:
+            del self.__dict__["models_registry"]
 
     @cached_property
-    def packages(self):
-        return StatePackages(self.real_packages, self.models)
+    def models_registry(self):
+        return StateModelsRegistry(self.real_packages, self.models)
 
     @classmethod
-    def from_packages(cls, packages):
+    def from_models_registry(cls, models_registry):
         """Take an Packages and return a ProjectState matching it."""
         app_models = {}
-        for model in packages.get_models(include_swapped=True):
+        for model in models_registry.get_models():
             model_state = ModelState.from_model(model)
-            app_models[
-                (model_state.package_label, model_state.name_lower)
-            ] = model_state
+            app_models[(model_state.package_label, model_state.name_lower)] = (
+                model_state
+            )
         return cls(app_models)
 
     def __eq__(self, other):
         return self.models == other.models and self.real_packages == other.real_packages
 
 
-class PackageConfigStub(PackageConfig):
-    """Stub of an PackageConfig. Only provides a label and a dict of models."""
-
-    def __init__(self, label):
-        self.packages = None
-        self.models = {}
-        # Package-label and package-name are not the same thing, so technically passing
-        # in the label here is wrong. In practice, migrations don't care about
-        # the package name, but we need something unique, and the label works fine.
-        self.label = label
-        self.name = label
-
-    def import_models(self):
-        self.models = self.packages.all_models[self.label]
-
-
-class StatePackages(Packages):
+class StateModelsRegistry(ModelsRegistry):
     """
     Subclass of the global Packages registry class to better handle dynamic model
     additions and removals.
     """
 
-    def __init__(self, real_packages, models, ignore_swappable=False):
+    def __init__(self, real_packages, models):
         # Any packages in self.real_packages should have all their models included
         # in the render. We don't use the original model instances as there
         # are some variables that refer to the Packages object.
@@ -573,32 +538,19 @@ class StatePackages(Packages):
         # mess things up with partial states (due to lack of dependencies)
         self.real_models = []
         for package_label in real_packages:
-            app = global_packages.get_package_config(package_label)
-            for model in app.get_models():
+            for model in global_models.get_models(package_label=package_label):
                 self.real_models.append(ModelState.from_model(model, exclude_rels=True))
-        # Populate the app registry with a stub for each application.
-        package_labels = {model_state.package_label for model_state in models.values()}
-        package_configs = [
-            PackageConfigStub(label)
-            for label in sorted([*real_packages, *package_labels])
-        ]
-        super().__init__(package_configs)
 
-        # These locks get in the way of copying as implemented in clone(),
-        # which is called whenever Plain duplicates a StatePackages before
-        # updating it.
-        self._lock = None
+        super().__init__()
 
         self.render_multiple([*models.values(), *self.real_models])
 
+        self.ready = True
+
         # There shouldn't be any operations pending at this point.
         from plain.models.preflight import _check_lazy_references
 
-        ignore = (
-            {make_model_tuple(settings.AUTH_USER_MODEL)} if ignore_swappable else set()
-        )
-        errors = _check_lazy_references(self, ignore=ignore)
-        if errors:
+        if errors := _check_lazy_references(self, packages_registry):
             raise ValueError("\n".join(error.msg for error in errors))
 
     @contextmanager
@@ -632,41 +584,29 @@ class StatePackages(Packages):
                         new_unrendered_models.append(model)
                 if len(new_unrendered_models) == len(unrendered_models):
                     raise InvalidBasesError(
-                        "Cannot resolve bases for %r\nThis can happen if you are "
+                        f"Cannot resolve bases for {new_unrendered_models!r}\nThis can happen if you are "
                         "inheriting models from an app with migrations (e.g. "
                         "contrib.auth)\n in an app with no migrations"
-                        % new_unrendered_models
                     )
                 unrendered_models = new_unrendered_models
 
     def clone(self):
         """Return a clone of this registry."""
-        clone = StatePackages([], {})
+        clone = StateModelsRegistry([], {})
         clone.all_models = copy.deepcopy(self.all_models)
 
-        for package_label in self.package_configs:
-            package_config = PackageConfigStub(package_label)
-            package_config.packages = clone
-            package_config.import_models()
-            clone.package_configs[package_label] = package_config
-
         # No need to actually clone them, they'll never change
         clone.real_models = self.real_models
         return clone
 
     def register_model(self, package_label, model):
         self.all_models[package_label][model._meta.model_name] = model
-        if package_label not in self.package_configs:
-            self.package_configs[package_label] = PackageConfigStub(package_label)
-            self.package_configs[package_label].packages = self
-        self.package_configs[package_label].models[model._meta.model_name] = model
         self.do_pending_operations(model)
         self.clear_cache()
 
     def unregister_model(self, package_label, model_name):
         try:
             del self.all_models[package_label][model_name]
-            del self.package_configs[package_label].models[model_name]
         except KeyError:
             pass
 
@@ -697,25 +637,25 @@ class ModelState:
             # Sanity-check that fields are NOT already bound to a model.
             if hasattr(field, "model"):
                 raise ValueError(
-                    'ModelState.fields cannot be bound to a model - "%s" is.' % name
+                    f'ModelState.fields cannot be bound to a model - "{name}" is.'
                 )
             # Sanity-check that relation fields are NOT referring to a model class.
             if field.is_relation and hasattr(field.related_model, "_meta"):
                 raise ValueError(
-                    'ModelState.fields cannot refer to a model class - "%s.to" does. '
-                    "Use a string reference instead." % name
+                    f'ModelState.fields cannot refer to a model class - "{name}.to" does. '
+                    "Use a string reference instead."
                 )
             if field.many_to_many and hasattr(field.remote_field.through, "_meta"):
                 raise ValueError(
-                    'ModelState.fields cannot refer to a model class - "%s.through" '
-                    "does. Use a string reference instead." % name
+                    f'ModelState.fields cannot refer to a model class - "{name}.through" '
+                    "does. Use a string reference instead."
                 )
         # Sanity-check that indexes have their name set.
         for index in self.options["indexes"]:
             if not index.name:
                 raise ValueError(
                     "Indexes passed to ModelState require a name attribute. "
-                    "%r doesn't have one." % index
+                    f"{index!r} doesn't have one."
                 )
 
     @cached_property
@@ -723,8 +663,6 @@ class ModelState:
         return self.name.lower()
 
     def get_field(self, field_name):
-        if field_name == "_order":
-            field_name = self.options.get("order_with_respect_to", field_name)
         return self.fields[field_name]
 
     @classmethod
@@ -735,8 +673,6 @@ class ModelState:
         for field in model._meta.local_fields:
             if getattr(field, "remote_field", None) and exclude_rels:
                 continue
-            if isinstance(field, models.OrderWrt):
-                continue
             name = field.name
             try:
                 fields.append((name, field.clone()))
@@ -751,17 +687,13 @@ class ModelState:
                     fields.append((name, field.clone()))
                 except TypeError as e:
                     raise TypeError(
-                        "Couldn't reconstruct m2m field {} on {}: {}".format(
-                            name,
-                            model._meta.object_name,
-                            e,
-                        )
+                        f"Couldn't reconstruct m2m field {name} on {model._meta.object_name}: {e}"
                     )
         # Extract the options
         options = {}
         for name in DEFAULT_NAMES:
             # Ignore some special options
-            if name in ["packages", "package_label"]:
+            if name in ["models_registry", "package_label"]:
                 continue
             elif name in model._meta.original_attrs:
                 if name == "indexes":
@@ -776,24 +708,11 @@ class ModelState:
                     ]
                 else:
                     options[name] = model._meta.original_attrs[name]
-        # If we're ignoring relationships, remove all field-listing model
-        # options (that option basically just means "make a stub model")
-        if exclude_rels:
-            if "order_with_respect_to" in options:
-                del options["order_with_respect_to"]
-        # Private fields are ignored, so remove options that refer to them.
-        elif options.get("order_with_respect_to") in {
-            field.name for field in model._meta.private_fields
-        }:
-            del options["order_with_respect_to"]
 
         def flatten_bases(model):
             bases = []
             for base in model.__bases__:
-                if hasattr(base, "_meta") and base._meta.abstract:
-                    bases.extend(flatten_bases(base))
-                else:
-                    bases.append(base)
+                bases.append(base)
             return bases
 
         # We can't rely on __mro__ directly because we only want to flatten
@@ -880,19 +799,19 @@ class ModelState:
             managers=list(self.managers),
         )
 
-    def render(self, packages):
+    def render(self, models_registry):
         """Create a Model object from our current state into the given packages."""
         # First, make a Meta object
         meta_contents = {
             "package_label": self.package_label,
-            "packages": packages,
+            "models_registry": models_registry,
             **self.options,
         }
         meta = type("Meta", (), meta_contents)
         # Then, work out our bases
         try:
             bases = tuple(
-                (packages.get_model(base) if isinstance(base, str) else base)
+                (models_registry.get_model(base) if isinstance(base, str) else base)
                 for base in self.bases
             )
         except LookupError:
@@ -906,8 +825,15 @@ class ModelState:
 
         # Restore managers
         body.update(self.construct_managers())
-        # Then, make a Model object (packages.register_model is called in __new__)
-        return type(self.name, bases, body)
+        # Then, make a Model object (models_registry.register_model is called in __new__)
+        model_class = type(self.name, bases, body)
+        from plain.models import register_model
+
+        # Register it to the models_registry associated with the model meta
+        # (could probably do this directly right here too...)
+        register_model(model_class)
+
+        return model_class
 
     def get_index_by_name(self, name):
         for index in self.options["indexes"]:
