@@ -2,18 +2,15 @@ import bisect
 import copy
 import inspect
 from collections import defaultdict
+from functools import cached_property
 
 from plain.exceptions import FieldDoesNotExist
+from plain.models import models_registry
 from plain.models.constraints import UniqueConstraint
-from plain.models.db import connections
+from plain.models.db import db_connection
 from plain.models.fields import BigAutoField
-from plain.models.fields.proxy import OrderWrt
 from plain.models.manager import Manager
-from plain.models.query_utils import PathInfo
-from plain.packages import packages
-from plain.runtime import settings
-from plain.utils.datastructures import ImmutableList, OrderedSet
-from plain.utils.functional import cached_property
+from plain.utils.datastructures import ImmutableList
 
 PROXY_PARENTS = object()
 
@@ -29,15 +26,8 @@ DEFAULT_NAMES = (
     "db_table_comment",
     "ordering",
     "get_latest_by",
-    "order_with_respect_to",
     "package_label",
-    "db_tablespace",
-    "abstract",
-    "managed",
-    "swappable",
-    "auto_created",
-    "packages",
-    "select_on_save",
+    "models_registry",
     "default_related_name",
     "required_db_features",
     "required_db_vendor",
@@ -67,13 +57,12 @@ class Options:
     }
     REVERSE_PROPERTIES = {"related_objects", "fields_map", "_relation_tree"}
 
-    default_packages = packages
+    default_models_registry = models_registry
 
     def __init__(self, meta, package_label=None):
         self._get_fields_cache = {}
         self.local_fields = []
         self.local_many_to_many = []
-        self.private_fields = []
         self.local_managers = []
         self.base_manager_name = None
         self.default_manager_name = None
@@ -81,36 +70,28 @@ class Options:
         self.db_table = ""
         self.db_table_comment = ""
         self.ordering = []
-        self._ordering_clash = False
         self.indexes = []
         self.constraints = []
-        self.select_on_save = False
         self.object_name = None
         self.package_label = package_label
         self.get_latest_by = None
-        self.order_with_respect_to = None
-        self.db_tablespace = settings.DEFAULT_TABLESPACE
         self.required_db_features = []
         self.required_db_vendor = None
         self.meta = meta
         self.pk = None
         self.auto_field = None
-        self.abstract = False
-        self.managed = True
+
         # For any non-abstract class, the concrete class is the model
         # in the end of the proxy_for_model chain. In particular, for
         # concrete models, the concrete_model is always the class itself.
         self.concrete_model = None
-        self.swappable = None
-        self.parents = {}
-        self.auto_created = False
 
         # List of all lookups defined in ForeignKey 'limit_choices_to' options
         # from *other* models. Needed for some admin checks. Internal use only.
         self.related_fkey_lookups = []
 
         # A custom app registry to use, if you're making a separate model set.
-        self.packages = self.default_packages
+        self.models_registry = self.default_models_registry
 
         self.default_related_name = None
 
@@ -122,14 +103,8 @@ class Options:
     def label_lower(self):
         return f"{self.package_label}.{self.model_name}"
 
-    @property
-    def package_config(self):
-        # Don't go through get_package_config to avoid triggering imports.
-        return self.packages.package_configs.get(self.package_label)
-
     def contribute_to_class(self, cls, name):
         from plain.models.backends.utils import truncate_name
-        from plain.models.db import connection
 
         cls._meta = self
         self.model = cls
@@ -160,18 +135,16 @@ class Options:
 
             # Package label/class name interpolation for names of constraints and
             # indexes.
-            if not getattr(cls._meta, "abstract", False):
-                for attr_name in {"constraints", "indexes"}:
-                    objs = getattr(self, attr_name, [])
-                    setattr(self, attr_name, self._format_names_with_class(cls, objs))
-
-            # order_with_respect_and ordering are mutually exclusive.
-            self._ordering_clash = bool(self.ordering and self.order_with_respect_to)
+            for attr_name in {"constraints", "indexes"}:
+                objs = getattr(self, attr_name, [])
+                setattr(self, attr_name, self._format_names_with_class(cls, objs))
 
             # Any leftover attributes must be invalid.
             if meta_attrs != {}:
                 raise TypeError(
-                    "'class Meta' got invalid attribute(s): %s" % ",".join(meta_attrs)
+                    "'class Meta' got invalid attribute(s): {}".format(
+                        ",".join(meta_attrs)
+                    )
                 )
 
         del self.meta
@@ -180,7 +153,8 @@ class Options:
         if not self.db_table:
             self.db_table = f"{self.package_label}_{self.model_name}"
             self.db_table = truncate_name(
-                self.db_table, connection.ops.max_name_length()
+                self.db_table,
+                db_connection.ops.max_name_length(),
             )
 
     def _format_names_with_class(self, cls, objs):
@@ -196,47 +170,9 @@ class Options:
         return new_objs
 
     def _prepare(self, model):
-        if self.order_with_respect_to:
-            # The app registry will not be ready at this point, so we cannot
-            # use get_field().
-            query = self.order_with_respect_to
-            try:
-                self.order_with_respect_to = next(
-                    f
-                    for f in self._get_fields(reverse=False)
-                    if f.name == query or f.attname == query
-                )
-            except StopIteration:
-                raise FieldDoesNotExist(
-                    f"{self.object_name} has no field named '{query}'"
-                )
-
-            self.ordering = ("_order",)
-            if not any(
-                isinstance(field, OrderWrt) for field in model._meta.local_fields
-            ):
-                model.add_to_class("_order", OrderWrt())
-        else:
-            self.order_with_respect_to = None
-
         if self.pk is None:
-            if self.parents:
-                # Promote the first parent link in lieu of adding yet another
-                # field.
-                field = next(iter(self.parents.values()))
-                # Look for a local field with the same name as the
-                # first parent link. If a local field has already been
-                # created, use it instead of promoting the parent
-                already_created = [
-                    fld for fld in self.local_fields if fld.name == field.name
-                ]
-                if already_created:
-                    field = already_created[0]
-                field.primary_key = True
-                self.setup_pk(field)
-            else:
-                auto = BigAutoField(primary_key=True, auto_created=True)
-                model.add_to_class("id", auto)
+            auto = BigAutoField(primary_key=True, auto_created=True)
+            model.add_to_class("id", auto)
 
     def add_manager(self, manager):
         self.local_managers.append(manager)
@@ -247,9 +183,7 @@ class Options:
         # the "creation_counter" attribute of the field.
         # Move many-to-many related fields from self.fields into
         # self.many_to_many.
-        if private:
-            self.private_fields.append(field)
-        elif field.is_relation and field.many_to_many:
+        if field.is_relation and field.many_to_many:
             bisect.insort(self.local_many_to_many, field)
         else:
             bisect.insort(self.local_fields, field)
@@ -281,20 +215,16 @@ class Options:
             self.pk = field
 
     def __repr__(self):
-        return "<Options for %s>" % self.object_name
+        return f"<Options for {self.object_name}>"
 
     def __str__(self):
         return self.label_lower
 
     def can_migrate(self, connection):
         """
-        Return True if the model can/should be migrated on the `connection`.
-        `connection` can be either a real connection or a connection alias.
+        Return True if the model can/should be migrated on the given
+        `connection` object.
         """
-        if self.swapped or not self.managed:
-            return False
-        if isinstance(connection, str):
-            connection = connections[connection]
         if self.required_db_vendor:
             return self.required_db_vendor == connection.vendor
         if self.required_db_features:
@@ -304,31 +234,6 @@ class Options:
             )
         return True
 
-    @property
-    def swapped(self):
-        """
-        Has this model been swapped out for another? If so, return the model
-        name of the replacement; otherwise, return None.
-
-        For historical reasons, model name lookups using get_model() are
-        case insensitive, so we make sure we are case insensitive here.
-        """
-        if self.swappable:
-            swapped_for = getattr(settings, self.swappable, None)
-            if swapped_for:
-                try:
-                    swapped_label, swapped_object = swapped_for.split(".")
-                except ValueError:
-                    # setting not in the format package_label.model_name
-                    # raising ImproperlyConfigured here causes problems with
-                    # test cleanup code - instead it is raised in get_user_model
-                    # or as part of validation.
-                    return swapped_for
-
-                if f"{swapped_label}.{swapped_object.lower()}" != self.label_lower:
-                    return swapped_for
-        return None
-
     @cached_property
     def managers(self):
         managers = []
@@ -549,11 +454,11 @@ class Options:
         except KeyError:
             # If the app registry is not ready, reverse fields are
             # unavailable, therefore we throw a FieldDoesNotExist exception.
-            if not self.packages.models_ready:
+            if not self.models_registry.ready:
                 raise FieldDoesNotExist(
-                    "{} has no field named '{}'. The app cache isn't ready yet, "
+                    f"{self.object_name} has no field named '{field_name}'. The app cache isn't ready yet, "
                     "so if this is an auto-created related field, it won't "
-                    "be available yet.".format(self.object_name, field_name)
+                    "be available yet."
                 )
 
         try:
@@ -565,108 +470,6 @@ class Options:
                 f"{self.object_name} has no field named '{field_name}'"
             )
 
-    def get_base_chain(self, model):
-        """
-        Return a list of parent classes leading to `model` (ordered from
-        closest to most distant ancestor). This has to handle the case where
-        `model` is a grandparent or even more distant relation.
-        """
-        if not self.parents:
-            return []
-        if model in self.parents:
-            return [model]
-        for parent in self.parents:
-            res = parent._meta.get_base_chain(model)
-            if res:
-                res.insert(0, parent)
-                return res
-        return []
-
-    def get_parent_list(self):
-        """
-        Return all the ancestors of this model as a list ordered by MRO.
-        Useful for determining if something is an ancestor, regardless of lineage.
-        """
-        result = OrderedSet(self.parents)
-        for parent in self.parents:
-            for ancestor in parent._meta.get_parent_list():
-                result.add(ancestor)
-        return list(result)
-
-    def get_ancestor_link(self, ancestor):
-        """
-        Return the field on the current model which points to the given
-        "ancestor". This is possible an indirect link (a pointer to a parent
-        model, which points, eventually, to the ancestor). Used when
-        constructing table joins for model inheritance.
-
-        Return None if the model isn't an ancestor of this one.
-        """
-        if ancestor in self.parents:
-            return self.parents[ancestor]
-        for parent in self.parents:
-            # Tries to get a link field from the immediate parent
-            parent_link = parent._meta.get_ancestor_link(ancestor)
-            if parent_link:
-                # In case of a proxied model, the first link
-                # of the chain to the ancestor is that parent
-                # links
-                return self.parents[parent] or parent_link
-
-    def get_path_to_parent(self, parent):
-        """
-        Return a list of PathInfos containing the path from the current
-        model to the parent model, or an empty list if parent is not a
-        parent of the current model.
-        """
-        if self.model is parent:
-            return []
-        # Skip the chain of proxy to the concrete proxied model.
-        proxied_model = self.concrete_model
-        path = []
-        opts = self
-        for int_model in self.get_base_chain(parent):
-            if int_model is proxied_model:
-                opts = int_model._meta
-            else:
-                final_field = opts.parents[int_model]
-                targets = (final_field.remote_field.get_related_field(),)
-                opts = int_model._meta
-                path.append(
-                    PathInfo(
-                        from_opts=final_field.model._meta,
-                        to_opts=opts,
-                        target_fields=targets,
-                        join_field=final_field,
-                        m2m=False,
-                        direct=True,
-                        filtered_relation=None,
-                    )
-                )
-        return path
-
-    def get_path_from_parent(self, parent):
-        """
-        Return a list of PathInfos containing the path from the parent
-        model to the current model, or an empty list if parent is not a
-        parent of the current model.
-        """
-        if self.model is parent:
-            return []
-        model = self.concrete_model
-        # Get a reversed base chain including both the current and parent
-        # models.
-        chain = model._meta.get_base_chain(parent)
-        chain.reverse()
-        chain.append(model)
-        # Construct a list of the PathInfos between models in chain.
-        path = []
-        for i, ancestor in enumerate(chain[:-1]):
-            child = chain[i + 1]
-            link = child._meta.get_ancestor_link(ancestor)
-            path.extend(link.reverse_path_infos)
-        return path
-
     def _populate_directed_relation_graph(self):
         """
         This method is used by each model to find its reverse objects. As this
@@ -676,16 +479,13 @@ class Options:
         """
         related_objects_graph = defaultdict(list)
 
-        all_models = self.packages.get_models(include_auto_created=True)
+        all_models = self.models_registry.get_models()
         for model in all_models:
             opts = model._meta
-            # Abstract model's fields are copied to child models, hence we will
-            # see the fields from the child models.
-            if opts.abstract:
-                continue
+
             fields_with_relations = (
                 f
-                for f in opts._get_fields(reverse=False, include_parents=False)
+                for f in opts._get_fields(reverse=False)
                 if f.is_relation and f.related_model is not None
             )
             for f in fields_with_relations:
@@ -718,33 +518,27 @@ class Options:
             for cache_key in self.FORWARD_PROPERTIES:
                 if cache_key in self.__dict__:
                     delattr(self, cache_key)
-        if reverse and not self.abstract:
+        if reverse:
             for cache_key in self.REVERSE_PROPERTIES:
                 if cache_key in self.__dict__:
                     delattr(self, cache_key)
         self._get_fields_cache = {}
 
-    def get_fields(self, include_parents=True, include_hidden=False):
+    def get_fields(self, include_hidden=False):
         """
         Return a list of fields associated to the model. By default, include
         forward and reverse fields, fields derived from inheritance, but not
         hidden fields. The returned fields can be changed using the parameters:
 
-        - include_parents: include fields derived from inheritance
         - include_hidden:  include fields that have a related_name that
                            starts with a "+"
         """
-        if include_parents is False:
-            include_parents = PROXY_PARENTS
-        return self._get_fields(
-            include_parents=include_parents, include_hidden=include_hidden
-        )
+        return self._get_fields(include_hidden=include_hidden)
 
     def _get_fields(
         self,
         forward=True,
         reverse=True,
-        include_parents=True,
         include_hidden=False,
         seen_models=None,
     ):
@@ -753,14 +547,8 @@ class Options:
         * If forward=True, then fields defined on this model are returned.
         * If reverse=True, then relations pointing to this model are returned.
         * If include_hidden=True, then fields with is_hidden=True are returned.
-        * The include_parents argument toggles if fields from parent models
-          should be included. It has three values: True, False, and
-          PROXY_PARENTS. When set to PROXY_PARENTS, the call will return all
-          fields defined for the current model or any of its parents in the
-          parent chain to the model's concrete model.
         """
-        if include_parents not in (True, False, PROXY_PARENTS):
-            raise TypeError(f"Invalid argument for include_parents: {include_parents}")
+
         # This helper function is used to allow recursion in ``get_fields()``
         # implementation and to provide a fast way for Plain's internals to
         # access specific subsets of fields.
@@ -773,7 +561,7 @@ class Options:
         seen_models.add(self.model)
 
         # Creates a cache key composed of all arguments
-        cache_key = (forward, reverse, include_parents, include_hidden, topmost_call)
+        cache_key = (forward, reverse, include_hidden, topmost_call)
 
         try:
             # In order to avoid list manipulation. Always return a shallow copy
@@ -783,32 +571,7 @@ class Options:
             pass
 
         fields = []
-        # Recursively call _get_fields() on each parent, with the same
-        # options provided in this call.
-        if include_parents is not False:
-            for parent in self.parents:
-                # In diamond inheritance it is possible that we see the same
-                # model from two different routes. In that case, avoid adding
-                # fields from the same parent again.
-                if parent in seen_models:
-                    continue
-                if (
-                    parent._meta.concrete_model != self.concrete_model
-                    and include_parents == PROXY_PARENTS
-                ):
-                    continue
-                for obj in parent._meta._get_fields(
-                    forward=forward,
-                    reverse=reverse,
-                    include_parents=include_parents,
-                    include_hidden=include_hidden,
-                    seen_models=seen_models,
-                ):
-                    if (
-                        not getattr(obj, "parent_link", False)
-                        or obj.model == self.concrete_model
-                    ):
-                        fields.append(obj)
+
         if reverse:
             # Tree is computed once and cached until the app cache is expired.
             # It is composed of a list of fields pointing to the current model
@@ -823,13 +586,6 @@ class Options:
         if forward:
             fields += self.local_fields
             fields += self.local_many_to_many
-            # Private fields are recopied to each child model, and they get a
-            # different model as field.model in each child. Hence we have to
-            # add the private fields separately from the topmost call. If we
-            # did this recursively similar to local_fields, we would get field
-            # instances with field.model != self.model.
-            if topmost_call:
-                fields += self.private_fields
 
         # In order to avoid list manipulation. Always
         # return a shallow copy of the results
@@ -886,8 +642,6 @@ class Options:
         """
         return [
             field
-            for field in self._get_fields(
-                forward=True, reverse=False, include_parents=PROXY_PARENTS
-            )
+            for field in self._get_fields(forward=True, reverse=False)
             if getattr(field, "db_returning", False)
         ]
