@@ -5,6 +5,7 @@ The main QuerySet implementation. This provides the public API for the ORM.
 import copy
 import operator
 import warnings
+from functools import cached_property
 from itertools import chain, islice
 
 import plain.runtime
@@ -19,8 +20,7 @@ from plain.models.db import (
     PLAIN_VERSION_PICKLE_KEY,
     IntegrityError,
     NotSupportedError,
-    connections,
-    router,
+    db_connection,
 )
 from plain.models.expressions import Case, F, Value, When
 from plain.models.fields import (
@@ -33,13 +33,11 @@ from plain.models.functions import Cast, Trunc
 from plain.models.query_utils import FilteredRelation, Q
 from plain.models.sql.constants import CURSOR, GET_ITERATOR_CHUNK_SIZE
 from plain.models.utils import (
-    AltersData,
     create_namedtuple_class,
     resolve_callables,
 )
-from plain.runtime import settings
 from plain.utils import timezone
-from plain.utils.functional import cached_property, partition
+from plain.utils.functional import partition
 
 # The maximum number of results to fetch in a get() query.
 MAX_GET_RESULTS = 21
@@ -62,8 +60,7 @@ class ModelIterable(BaseIterable):
 
     def __iter__(self):
         queryset = self.queryset
-        db = queryset.db
-        compiler = queryset.query.get_compiler(using=db)
+        compiler = queryset.query.get_compiler()
         # Execute the query. This will also fill compiler.select, klass_info,
         # and annotations.
         results = compiler.execute_sql(
@@ -80,7 +77,7 @@ class ModelIterable(BaseIterable):
         init_list = [
             f[0].target.attname for f in select[model_fields_start:model_fields_end]
         ]
-        related_populators = get_related_populators(klass_info, select, db)
+        related_populators = get_related_populators(klass_info, select)
         known_related_objects = [
             (
                 field,
@@ -97,9 +94,7 @@ class ModelIterable(BaseIterable):
             for field, related_objs in queryset._known_related_objects.items()
         ]
         for row in compiler.results_iter(results):
-            obj = model_cls.from_db(
-                db, init_list, row[model_fields_start:model_fields_end]
-            )
+            obj = model_cls.from_db(init_list, row[model_fields_start:model_fields_end])
             for rel_populator in related_populators:
                 rel_populator.populate(row, obj)
             if annotation_col_map:
@@ -129,10 +124,8 @@ class RawModelIterable(BaseIterable):
 
     def __iter__(self):
         # Cache some things for performance reasons outside the loop.
-        db = self.queryset.db
         query = self.queryset.query
-        connection = connections[db]
-        compiler = connection.ops.compiler("SQLCompiler")(query, connection, db)
+        compiler = db_connection.ops.compiler("SQLCompiler")(query, db_connection)
         query_iterator = iter(query)
 
         try:
@@ -155,7 +148,7 @@ class RawModelIterable(BaseIterable):
             for values in query_iterator:
                 # Associate fields to values
                 model_init_values = [values[pos] for pos in model_init_pos]
-                instance = model_cls.from_db(db, model_init_names, model_init_values)
+                instance = model_cls.from_db(model_init_names, model_init_values)
                 if annotation_fields:
                     for column, pos in annotation_fields:
                         setattr(instance, column, values[pos])
@@ -174,7 +167,7 @@ class ValuesIterable(BaseIterable):
     def __iter__(self):
         queryset = self.queryset
         query = queryset.query
-        compiler = query.get_compiler(queryset.db)
+        compiler = query.get_compiler()
 
         # extra(select=...) cols are always at the start of the row.
         names = [
@@ -198,7 +191,7 @@ class ValuesListIterable(BaseIterable):
     def __iter__(self):
         queryset = self.queryset
         query = queryset.query
-        compiler = query.get_compiler(queryset.db)
+        compiler = query.get_compiler()
 
         if queryset._fields:
             # extra(select=...) cols are always at the start of the row.
@@ -259,19 +252,18 @@ class FlatValuesListIterable(BaseIterable):
 
     def __iter__(self):
         queryset = self.queryset
-        compiler = queryset.query.get_compiler(queryset.db)
+        compiler = queryset.query.get_compiler()
         for row in compiler.results_iter(
             chunked_fetch=self.chunked_fetch, chunk_size=self.chunk_size
         ):
             yield row[0]
 
 
-class QuerySet(AltersData):
+class QuerySet:
     """Represent a lazy database lookup for a set of objects."""
 
-    def __init__(self, model=None, query=None, using=None, hints=None):
+    def __init__(self, model=None, query=None, hints=None):
         self.model = model
-        self._db = using
         self._hints = hints or {}
         self._query = query or sql.Query(self.model)
         self._result_cache = None
@@ -334,10 +326,8 @@ class QuerySet(AltersData):
         if pickled_version:
             if pickled_version != plain.runtime.__version__:
                 warnings.warn(
-                    "Pickled queryset instance's Plain version {} does not "
-                    "match the current version {}.".format(
-                        pickled_version, plain.runtime.__version__
-                    ),
+                    f"Pickled queryset instance's Plain version {pickled_version} does not "
+                    f"match the current version {plain.runtime.__version__}.",
                     RuntimeWarning,
                     stacklevel=2,
                 )
@@ -385,8 +375,7 @@ class QuerySet(AltersData):
         """Retrieve an item or slice from the set of results."""
         if not isinstance(k, int | slice):
             raise TypeError(
-                "QuerySet indices must be integers or slices, not %s."
-                % type(k).__name__
+                f"QuerySet indices must be integers or slices, not {type(k).__name__}."
             )
         if (isinstance(k, int) and k < 0) or (
             isinstance(k, slice)
@@ -504,7 +493,7 @@ class QuerySet(AltersData):
                 )
         elif chunk_size <= 0:
             raise ValueError("Chunk size must be strictly positive.")
-        use_chunked_fetch = not connections[self.db].settings_dict.get(
+        use_chunked_fetch = not db_connection.settings_dict.get(
             "DISABLE_SERVER_SIDE_CURSORS"
         )
         return self._iterator(use_chunked_fetch, chunk_size)
@@ -532,7 +521,7 @@ class QuerySet(AltersData):
                 raise TypeError("Complex aggregates require an alias")
             kwargs[arg.default_alias] = arg
 
-        return self.query.chain().get_aggregation(self.db, kwargs)
+        return self.query.chain().get_aggregation(kwargs)
 
     def count(self):
         """
@@ -545,7 +534,7 @@ class QuerySet(AltersData):
         if self._result_cache is not None:
             return len(self._result_cache)
 
-        return self.query.get_count(using=self.db)
+        return self.query.get_count()
 
     def get(self, *args, **kwargs):
         """
@@ -554,8 +543,8 @@ class QuerySet(AltersData):
         """
         if self.query.combinator and (args or kwargs):
             raise NotSupportedError(
-                "Calling QuerySet.get(...) with filters after %s() is not "
-                "supported." % self.query.combinator
+                f"Calling QuerySet.get(...) with filters after {self.query.combinator}() is not "
+                "supported."
             )
         clone = self._chain() if self.query.combinator else self.filter(*args, **kwargs)
         if self.query.can_filter() and not self.query.distinct_fields:
@@ -563,7 +552,7 @@ class QuerySet(AltersData):
         limit = None
         if (
             not clone.query.select_for_update
-            or connections[clone.db].features.supports_select_for_update_with_limit
+            or db_connection.features.supports_select_for_update_with_limit
         ):
             limit = MAX_GET_RESULTS
             clone.query.set_limits(high=limit)
@@ -572,7 +561,7 @@ class QuerySet(AltersData):
             return clone._result_cache[0]
         if not num:
             raise self.model.DoesNotExist(
-                "%s matching query does not exist." % self.model._meta.object_name
+                f"{self.model._meta.object_name} matching query does not exist."
             )
         raise self.model.MultipleObjectsReturned(
             "get() returned more than one {} -- it returned {}!".format(
@@ -588,7 +577,7 @@ class QuerySet(AltersData):
         """
         obj = self.model(**kwargs)
         self._for_write = True
-        obj.save(force_insert=True, using=self.db)
+        obj.save(force_insert=True)
         return obj
 
     def _prepare_for_bulk_create(self, objs):
@@ -599,20 +588,10 @@ class QuerySet(AltersData):
             obj._prepare_related_fields_for_save(operation_name="bulk_create")
 
     def _check_bulk_create_options(
-        self, ignore_conflicts, update_conflicts, update_fields, unique_fields
+        self, update_conflicts, update_fields, unique_fields
     ):
-        if ignore_conflicts and update_conflicts:
-            raise ValueError(
-                "ignore_conflicts and update_conflicts are mutually exclusive."
-            )
-        db_features = connections[self.db].features
-        if ignore_conflicts:
-            if not db_features.supports_ignore_conflicts:
-                raise NotSupportedError(
-                    "This database backend does not support ignoring conflicts."
-                )
-            return OnConflict.IGNORE
-        elif update_conflicts:
+        db_features = db_connection.features
+        if update_conflicts:
             if not db_features.supports_update_conflicts:
                 raise NotSupportedError(
                     "This database backend does not support updating conflicts."
@@ -640,8 +619,7 @@ class QuerySet(AltersData):
                 )
             if any(f.primary_key for f in update_fields):
                 raise ValueError(
-                    "bulk_create() cannot be used with primary keys in "
-                    "update_fields."
+                    "bulk_create() cannot be used with primary keys in update_fields."
                 )
             if unique_fields:
                 if any(not f.concrete or f.many_to_many for f in unique_fields):
@@ -656,15 +634,13 @@ class QuerySet(AltersData):
         self,
         objs,
         batch_size=None,
-        ignore_conflicts=False,
         update_conflicts=False,
         update_fields=None,
         unique_fields=None,
     ):
         """
         Insert each of the instances into the database. Do *not* call
-        save() on each of the instances, do not send any pre/post_save
-        signals, and do not set the primary key attribute if it is an
+        save() on each of the instances, and do not set the primary key attribute if it is an
         autoincrement field (except if features.can_return_rows_from_bulk_insert=True).
         Multi-table models are not supported.
         """
@@ -682,13 +658,7 @@ class QuerySet(AltersData):
         # trickier so it's not done yet.
         if batch_size is not None and batch_size <= 0:
             raise ValueError("Batch size must be a positive integer.")
-        # Check that the parents share the same concrete model with the our
-        # model to detect the inheritance pattern ConcreteGrandParent ->
-        # MultiTableParent -> ProxyChild. Simply checking self.model._meta.proxy
-        # would not identify that case as involving multiple tables.
-        for parent in self.model._meta.get_parent_list():
-            if parent._meta.concrete_model is not self.model._meta.concrete_model:
-                raise ValueError("Can't bulk create a multi-table inherited model")
+
         if not objs:
             return objs
         opts = self.model._meta
@@ -701,7 +671,6 @@ class QuerySet(AltersData):
         if update_fields:
             update_fields = [self.model._meta.get_field(name) for name in update_fields]
         on_conflict = self._check_bulk_create_options(
-            ignore_conflicts,
             update_conflicts,
             update_fields,
             unique_fields,
@@ -710,7 +679,7 @@ class QuerySet(AltersData):
         fields = opts.concrete_fields
         objs = list(objs)
         self._prepare_for_bulk_create(objs)
-        with transaction.atomic(using=self.db, savepoint=False):
+        with transaction.atomic(savepoint=False):
             objs_with_pk, objs_without_pk = partition(lambda o: o.pk is None, objs)
             if objs_with_pk:
                 returned_columns = self._batched_insert(
@@ -727,7 +696,6 @@ class QuerySet(AltersData):
                             setattr(obj_with_pk, field.attname, result)
                 for obj_with_pk in objs_with_pk:
                     obj_with_pk._state.adding = False
-                    obj_with_pk._state.db = self.db
             if objs_without_pk:
                 fields = [f for f in fields if not isinstance(f, AutoField)]
                 returned_columns = self._batched_insert(
@@ -738,9 +706,8 @@ class QuerySet(AltersData):
                     update_fields=update_fields,
                     unique_fields=unique_fields,
                 )
-                connection = connections[self.db]
                 if (
-                    connection.features.can_return_rows_from_bulk_insert
+                    db_connection.features.can_return_rows_from_bulk_insert
                     and on_conflict is None
                 ):
                     assert len(returned_columns) == len(objs_without_pk)
@@ -748,7 +715,6 @@ class QuerySet(AltersData):
                     for result, field in zip(results, opts.db_returning_fields):
                         setattr(obj_without_pk, field.attname, result)
                     obj_without_pk._state.adding = False
-                    obj_without_pk._state.db = self.db
 
         return objs
 
@@ -777,10 +743,9 @@ class QuerySet(AltersData):
         # PK is used twice in the resulting update query, once in the filter
         # and once in the WHEN. Each field will also have one CAST.
         self._for_write = True
-        connection = connections[self.db]
-        max_batch_size = connection.ops.bulk_batch_size(["pk", "pk"] + fields, objs)
+        max_batch_size = db_connection.ops.bulk_batch_size(["pk", "pk"] + fields, objs)
         batch_size = min(batch_size, max_batch_size) if batch_size else max_batch_size
-        requires_casting = connection.features.requires_casted_case_in_updates
+        requires_casting = db_connection.features.requires_casted_case_in_updates
         batches = (objs[i : i + batch_size] for i in range(0, len(objs), batch_size))
         updates = []
         for batch_objs in batches:
@@ -798,14 +763,12 @@ class QuerySet(AltersData):
                 update_kwargs[field.attname] = case_statement
             updates.append(([obj.pk for obj in batch_objs], update_kwargs))
         rows_updated = 0
-        queryset = self.using(self.db)
-        with transaction.atomic(using=self.db, savepoint=False):
+        queryset = self._chain()
+        with transaction.atomic(savepoint=False):
             for pks, update_kwargs in updates:
                 rows_updated += queryset.filter(pk__in=pks).update(**update_kwargs)
         return rows_updated
 
-    bulk_update.alters_data = True
-
     def get_or_create(self, defaults=None, **kwargs):
         """
         Look up an object with the given kwargs, creating one if necessary.
@@ -821,7 +784,7 @@ class QuerySet(AltersData):
             params = self._extract_model_params(defaults, **kwargs)
             # Try to create an object using passed params.
             try:
-                with transaction.atomic(using=self.db):
+                with transaction.atomic():
                     params = dict(resolve_callables(params))
                     return self.create(**params), True
             except (IntegrityError, ValidationError):
@@ -852,7 +815,7 @@ class QuerySet(AltersData):
         else:
             update_defaults = defaults or {}
         self._for_write = True
-        with transaction.atomic(using=self.db):
+        with transaction.atomic():
             # Lock the row so that a concurrent update is blocked until
             # update_or_create() has performed its save.
             obj, created = self.select_for_update().get_or_create(
@@ -878,9 +841,9 @@ class QuerySet(AltersData):
                         update_fields.add(field.name)
                         if field.name != field.attname:
                             update_fields.add(field.attname)
-                obj.save(using=self.db, update_fields=update_fields)
+                obj.save(update_fields=update_fields)
             else:
-                obj.save(using=self.db)
+                obj.save()
         return obj, False
 
     def _extract_model_params(self, defaults, **kwargs):
@@ -980,19 +943,18 @@ class QuerySet(AltersData):
         ]
         if (
             field_name != "pk"
-            and not opts.get_field(field_name).unique
+            and not opts.get_field(field_name).primary_key
             and field_name not in unique_fields
             and self.query.distinct_fields != (field_name,)
         ):
             raise ValueError(
-                "in_bulk()'s field_name must be a unique field but %r isn't."
-                % field_name
+                f"in_bulk()'s field_name must be a unique field but {field_name!r} isn't."
             )
         if id_list is not None:
             if not id_list:
                 return {}
             filter_key = f"{field_name}__in"
-            batch_size = connections[self.db].features.max_query_params
+            batch_size = db_connection.features.max_query_params
             id_list = tuple(id_list)
             # If the database has a limit on the number of query parameters
             # (e.g. SQLite), retrieve objects in batches if necessary.
@@ -1031,7 +993,7 @@ class QuerySet(AltersData):
 
         from plain.models.deletion import Collector
 
-        collector = Collector(using=del_query.db, origin=self)
+        collector = Collector(origin=self)
         collector.collect(del_query)
         deleted, _rows_count = collector.delete()
 
@@ -1039,24 +1001,21 @@ class QuerySet(AltersData):
         self._result_cache = None
         return deleted, _rows_count
 
-    delete.alters_data = True
     delete.queryset_only = True
 
-    def _raw_delete(self, using):
+    def _raw_delete(self):
         """
         Delete objects found from the given queryset in single direct SQL
         query. No signals are sent and there is no protection for cascades.
         """
         query = self.query.clone()
         query.__class__ = sql.DeleteQuery
-        cursor = query.get_compiler(using).execute_sql(CURSOR)
+        cursor = query.get_compiler().execute_sql(CURSOR)
         if cursor:
             with cursor:
                 return cursor.rowcount
         return 0
 
-    _raw_delete.alters_data = True
-
     def update(self, **kwargs):
         """
         Update all elements in the current QuerySet, setting all the given
@@ -1091,13 +1050,11 @@ class QuerySet(AltersData):
 
         # Clear any annotations so that they won't be present in subqueries.
         query.annotations = {}
-        with transaction.mark_for_rollback_on_error(using=self.db):
-            rows = query.get_compiler(self.db).execute_sql(CURSOR)
+        with transaction.mark_for_rollback_on_error():
+            rows = query.get_compiler().execute_sql(CURSOR)
         self._result_cache = None
         return rows
 
-    update.alters_data = True
-
     def _update(self, values):
         """
         A version of update() that accepts field objects instead of field names.
@@ -1112,9 +1069,8 @@ class QuerySet(AltersData):
         # Clear any annotations so that they won't be present in subqueries.
         query.annotations = {}
         self._result_cache = None
-        return query.get_compiler(self.db).execute_sql(CURSOR)
+        return query.get_compiler().execute_sql(CURSOR)
 
-    _update.alters_data = True
     _update.queryset_only = False
 
     def exists(self):
@@ -1122,7 +1078,7 @@ class QuerySet(AltersData):
         Return True if the QuerySet would have any results, False otherwise.
         """
         if self._result_cache is None:
-            return self.query.has_results(using=self.db)
+            return self.query.has_results()
         return bool(self._result_cache)
 
     def contains(self, obj):
@@ -1156,21 +1112,18 @@ class QuerySet(AltersData):
         Runs an EXPLAIN on the SQL query this QuerySet would perform, and
         returns the results.
         """
-        return self.query.explain(using=self.db, format=format, **options)
+        return self.query.explain(format=format, **options)
 
     ##################################################
     # PUBLIC METHODS THAT RETURN A QUERYSET SUBCLASS #
     ##################################################
 
-    def raw(self, raw_query, params=(), translations=None, using=None):
-        if using is None:
-            using = self.db
+    def raw(self, raw_query, params=(), translations=None):
         qs = RawQuerySet(
             raw_query,
             model=self.model,
             params=params,
             translations=translations,
-            using=using,
         )
         qs._prefetch_related_lookups = self._prefetch_related_lookups[:]
         return qs
@@ -1259,11 +1212,10 @@ class QuerySet(AltersData):
             )
         if order not in ("ASC", "DESC"):
             raise ValueError("'order' must be either 'ASC' or 'DESC'.")
-        if settings.USE_TZ:
-            if tzinfo is None:
-                tzinfo = timezone.get_current_timezone()
-        else:
-            tzinfo = None
+
+        if tzinfo is None:
+            tzinfo = timezone.get_current_timezone()
+
         return (
             self.annotate(
                 datetimefield=Trunc(
@@ -1476,8 +1428,8 @@ class QuerySet(AltersData):
             try:
                 if arg.default_alias in kwargs:
                     raise ValueError(
-                        "The named annotation '%s' conflicts with the "
-                        "default name for another annotation." % arg.default_alias
+                        f"The named annotation '{arg.default_alias}' conflicts with the "
+                        "default name for another annotation."
                     )
             except TypeError:
                 raise TypeError("Complex annotations require an alias")
@@ -1499,8 +1451,7 @@ class QuerySet(AltersData):
         for alias, annotation in annotations.items():
             if alias in names:
                 raise ValueError(
-                    "The annotation '%s' conflicts with a field on "
-                    "the model." % alias
+                    f"The annotation '{alias}' conflicts with a field on the model."
                 )
             if isinstance(annotation, FilteredRelation):
                 clone.query.add_filtered_relation(annotation, alias)
@@ -1605,12 +1556,6 @@ class QuerySet(AltersData):
         clone.query.add_immediate_loading(fields)
         return clone
 
-    def using(self, alias):
-        """Select which database this QuerySet should execute against."""
-        clone = self._chain()
-        clone._db = alias
-        return clone
-
     ###################################
     # PUBLIC INTROSPECTION ATTRIBUTES #
     ###################################
@@ -1636,13 +1581,6 @@ class QuerySet(AltersData):
         else:
             return False
 
-    @property
-    def db(self):
-        """Return the database used if this query is executed now."""
-        if self._for_write:
-            return self._db or router.db_for_write(self.model, **self._hints)
-        return self._db or router.db_for_read(self.model, **self._hints)
-
     ###################
     # PRIVATE METHODS #
     ###################
@@ -1653,7 +1591,6 @@ class QuerySet(AltersData):
         fields,
         returning_fields=None,
         raw=False,
-        using=None,
         on_conflict=None,
         update_fields=None,
         unique_fields=None,
@@ -1663,8 +1600,6 @@ class QuerySet(AltersData):
         the InsertQuery class and is how Model.save() is implemented.
         """
         self._for_write = True
-        if using is None:
-            using = self.db
         query = sql.InsertQuery(
             self.model,
             on_conflict=on_conflict,
@@ -1672,9 +1607,8 @@ class QuerySet(AltersData):
             unique_fields=unique_fields,
         )
         query.insert_values(fields, objs, raw=raw)
-        return query.get_compiler(using=using).execute_sql(returning_fields)
+        return query.get_compiler().execute_sql(returning_fields)
 
-    _insert.alters_data = True
     _insert.queryset_only = False
 
     def _batched_insert(
@@ -1689,19 +1623,17 @@ class QuerySet(AltersData):
         """
         Helper method for bulk_create() to insert objs one batch at a time.
         """
-        connection = connections[self.db]
-        ops = connection.ops
+        ops = db_connection.ops
         max_batch_size = max(ops.bulk_batch_size(fields, objs), 1)
         batch_size = min(batch_size, max_batch_size) if batch_size else max_batch_size
         inserted_rows = []
-        bulk_return = connection.features.can_return_rows_from_bulk_insert
+        bulk_return = db_connection.features.can_return_rows_from_bulk_insert
         for item in [objs[i : i + batch_size] for i in range(0, len(objs), batch_size)]:
             if bulk_return and on_conflict is None:
                 inserted_rows.extend(
                     self._insert(
                         item,
                         fields=fields,
-                        using=self.db,
                         returning_fields=self.model._meta.db_returning_fields,
                     )
                 )
@@ -1709,7 +1641,6 @@ class QuerySet(AltersData):
                 self._insert(
                     item,
                     fields=fields,
-                    using=self.db,
                     on_conflict=on_conflict,
                     update_fields=update_fields,
                     unique_fields=unique_fields,
@@ -1735,7 +1666,6 @@ class QuerySet(AltersData):
         c = self.__class__(
             model=self.model,
             query=self.query.chain(),
-            using=self._db,
             hints=self._hints,
         )
         c._sticky_filter = self._sticky_filter
@@ -1774,8 +1704,7 @@ class QuerySet(AltersData):
             or set(self.query.annotation_select) != set(other.query.annotation_select)
         ):
             raise TypeError(
-                "Merging '%s' classes must involve the same values in each case."
-                % self.__class__.__name__
+                f"Merging '{self.__class__.__name__}' classes must involve the same values in each case."
             )
 
     def _merge_known_related_objects(self, other):
@@ -1791,7 +1720,6 @@ class QuerySet(AltersData):
             # if they are set up to select only a single field.
             raise TypeError("Cannot use multi-field values as a filter value.")
         query = self.query.resolve_expression(*args, **kwargs)
-        query._db = self._db
         return query
 
     resolve_expression.queryset_only = True
@@ -1827,9 +1755,7 @@ class QuerySet(AltersData):
     def _not_support_combined_queries(self, operation_name):
         if self.query.combinator:
             raise NotSupportedError(
-                "Calling QuerySet.{}() after {}() is not supported.".format(
-                    operation_name, self.query.combinator
-                )
+                f"Calling QuerySet.{operation_name}() after {self.query.combinator}() is not supported."
             )
 
     def _check_operator_queryset(self, other, operator_):
@@ -1874,14 +1800,12 @@ class RawQuerySet:
         query=None,
         params=(),
         translations=None,
-        using=None,
         hints=None,
     ):
         self.raw_query = raw_query
         self.model = model
-        self._db = using
         self._hints = hints or {}
-        self.query = query or sql.RawQuery(sql=raw_query, using=self.db, params=params)
+        self.query = query or sql.RawQuery(sql=raw_query, params=params)
         self.params = params
         self.translations = translations or {}
         self._result_cache = None
@@ -1890,7 +1814,7 @@ class RawQuerySet:
 
     def resolve_model_init_order(self):
         """Resolve the init field names and value positions."""
-        converter = connections[self.db].introspection.identifier_converter
+        converter = db_connection.introspection.identifier_converter
         model_init_fields = [
             f for f in self.model._meta.fields if converter(f.column) in self.columns
         ]
@@ -1926,7 +1850,6 @@ class RawQuerySet:
             query=self.query,
             params=self.params,
             translations=self.translations,
-            using=self._db,
             hints=self._hints,
         )
         c._prefetch_related_lookups = self._prefetch_related_lookups[:]
@@ -1959,22 +1882,6 @@ class RawQuerySet:
     def __getitem__(self, k):
         return list(self)[k]
 
-    @property
-    def db(self):
-        """Return the database used if this query is executed now."""
-        return self._db or router.db_for_read(self.model, **self._hints)
-
-    def using(self, alias):
-        """Select the database this RawQuerySet should execute against."""
-        return RawQuerySet(
-            self.raw_query,
-            model=self.model,
-            query=self.query.chain(using=alias),
-            params=self.params,
-            translations=self.translations,
-            using=alias,
-        )
-
     @cached_property
     def columns(self):
         """
@@ -1996,7 +1903,7 @@ class RawQuerySet:
     @cached_property
     def model_fields(self):
         """A dict mapping column names to model field names."""
-        converter = connections[self.db].introspection.identifier_converter
+        converter = db_connection.introspection.identifier_converter
         model_fields = {}
         for field in self.model._meta.fields:
             name, column = field.get_attname_column()
@@ -2099,9 +2006,8 @@ def prefetch_related_objects(model_instances, *related_lookups):
         if lookup.prefetch_to in done_queries:
             if lookup.queryset is not None:
                 raise ValueError(
-                    "'%s' lookup was already seen with a different queryset. "
+                    f"'{lookup.prefetch_to}' lookup was already seen with a different queryset. "
                     "You may need to adjust the ordering of your lookups."
-                    % lookup.prefetch_to
                 )
 
             continue
@@ -2154,12 +2060,8 @@ def prefetch_related_objects(model_instances, *related_lookups):
 
             if not attr_found:
                 raise AttributeError(
-                    "Cannot find '{}' on {} object, '{}' is an invalid "
-                    "parameter to prefetch_related()".format(
-                        through_attr,
-                        first_obj.__class__.__name__,
-                        lookup.prefetch_through,
-                    )
+                    f"Cannot find '{through_attr}' on {first_obj.__class__.__name__} object, '{lookup.prefetch_through}' is an invalid "
+                    "parameter to prefetch_related()"
                 )
 
             if level == len(through_attrs) - 1 and prefetcher is None:
@@ -2167,9 +2069,9 @@ def prefetch_related_objects(model_instances, *related_lookups):
                 # prefetching, otherwise there is no point adding it and the
                 # developer asking for it has made a mistake.
                 raise ValueError(
-                    "'%s' does not resolve to an item that supports "
+                    f"'{lookup.prefetch_through}' does not resolve to an item that supports "
                     "prefetching - this is an invalid parameter to "
-                    "prefetch_related()." % lookup.prefetch_through
+                    "prefetch_related()."
                 )
 
             obj_to_fetch = None
@@ -2411,8 +2313,7 @@ class RelatedPopulator:
     model instance.
     """
 
-    def __init__(self, klass_info, select, db):
-        self.db = db
+    def __init__(self, klass_info, select):
         # Pre-compute needed attributes. The attributes are:
         #  - model_cls: the possibly deferred model class to instantiate
         #  - either:
@@ -2437,31 +2338,17 @@ class RelatedPopulator:
         #    the object being populated and on the remote object. Usually
         #    these are Field.set_cached_value() methods.
         select_fields = klass_info["select_fields"]
-        from_parent = klass_info["from_parent"]
-        if not from_parent:
-            self.cols_start = select_fields[0]
-            self.cols_end = select_fields[-1] + 1
-            self.init_list = [
-                f[0].target.attname for f in select[self.cols_start : self.cols_end]
-            ]
-            self.reorder_for_init = None
-        else:
-            attname_indexes = {
-                select[idx][0].target.attname: idx for idx in select_fields
-            }
-            model_init_attnames = (
-                f.attname for f in klass_info["model"]._meta.concrete_fields
-            )
-            self.init_list = [
-                attname for attname in model_init_attnames if attname in attname_indexes
-            ]
-            self.reorder_for_init = operator.itemgetter(
-                *[attname_indexes[attname] for attname in self.init_list]
-            )
+
+        self.cols_start = select_fields[0]
+        self.cols_end = select_fields[-1] + 1
+        self.init_list = [
+            f[0].target.attname for f in select[self.cols_start : self.cols_end]
+        ]
+        self.reorder_for_init = None
 
         self.model_cls = klass_info["model"]
         self.pk_idx = self.init_list.index(self.model_cls._meta.pk.attname)
-        self.related_populators = get_related_populators(klass_info, select, self.db)
+        self.related_populators = get_related_populators(klass_info, select)
         self.local_setter = klass_info["local_setter"]
         self.remote_setter = klass_info["remote_setter"]
 
@@ -2473,7 +2360,7 @@ class RelatedPopulator:
         if obj_data[self.pk_idx] is None:
             obj = None
         else:
-            obj = self.model_cls.from_db(self.db, self.init_list, obj_data)
+            obj = self.model_cls.from_db(self.init_list, obj_data)
             for rel_iter in self.related_populators:
                 rel_iter.populate(row, obj)
         self.local_setter(from_obj, obj)
@@ -2481,10 +2368,10 @@ class RelatedPopulator:
             self.remote_setter(obj, from_obj)
 
 
-def get_related_populators(klass_info, select, db):
+def get_related_populators(klass_info, select):
     iterators = []
     related_klass_infos = klass_info.get("related_klass_infos", [])
     for rel_klass_info in related_klass_infos:
-        rel_cls = RelatedPopulator(rel_klass_info, select, db)
+        rel_cls = RelatedPopulator(rel_klass_info, select)
         iterators.append(rel_cls)
     return iterators
