@@ -5,6 +5,7 @@ Factored out from plain.models.query to avoid making the main module very
 large and/or so that they can be used by other modules without getting into
 circular import difficulties.
 """
+
 import functools
 import inspect
 import logging
@@ -12,7 +13,7 @@ from collections import namedtuple
 
 from plain.exceptions import FieldError
 from plain.models.constants import LOOKUP_SEP
-from plain.models.db import DEFAULT_DB_ALIAS, DatabaseError, connections
+from plain.models.db import DatabaseError, db_connection
 from plain.utils import tree
 
 logger = logging.getLogger("plain.models")
@@ -110,7 +111,7 @@ class Q(tree.Node):
             else:
                 yield child
 
-    def check(self, against, using=DEFAULT_DB_ALIAS):
+    def check(self, against):
         """
         Do a database query to check if the expressions of the Q instance
         matches against the expressions.
@@ -129,11 +130,11 @@ class Q(tree.Node):
             query.add_annotation(value, name, select=False)
         query.add_annotation(Value(1), "_check")
         # This will raise a FieldError if a field is missing in "against".
-        if connections[using].features.supports_comparing_boolean_expr:
+        if db_connection.features.supports_comparing_boolean_expr:
             query.add_q(Q(Coalesce(self, True, output_field=BooleanField())))
         else:
             query.add_q(self)
-        compiler = query.get_compiler(using=using)
+        compiler = query.get_compiler()
         try:
             return compiler.execute_sql(SINGLE) is not None
         except DatabaseError as e:
@@ -172,27 +173,9 @@ class DeferredAttribute:
         data = instance.__dict__
         field_name = self.field.attname
         if field_name not in data:
-            # Let's see if the field is part of the parent chain. If so we
-            # might be able to reuse the already loaded value. Refs #18343.
-            val = self._check_parent_chain(instance)
-            if val is None:
-                instance.refresh_from_db(fields=[field_name])
-            else:
-                data[field_name] = val
+            instance.refresh_from_db(fields=[field_name])
         return data[field_name]
 
-    def _check_parent_chain(self, instance):
-        """
-        Check if the field value can be fetched from a parent field already
-        loaded in the instance. This can be done if the to-be fetched
-        field is a primary key field.
-        """
-        opts = instance._meta
-        link_field = opts.get_ancestor_link(self.field.model)
-        if self.field.primary_key and self.field != link_field:
-            return getattr(instance, link_field.attname)
-        return None
-
 
 class class_or_instance_method:
     """
@@ -337,7 +320,7 @@ def select_related_descend(field, restricted, requested, select_mask, reverse=Fa
             return False
         if not reverse and field.name not in requested:
             return False
-    if not restricted and field.null:
+    if not restricted and field.allow_null:
         return False
     if (
         restricted
@@ -374,11 +357,7 @@ def check_rel_lookup_compatibility(model, target_opts, field):
     """
 
     def check(opts):
-        return (
-            model._meta.concrete_model == opts.concrete_model
-            or opts.concrete_model in model._meta.get_parent_list()
-            or model in opts.get_parent_list()
-        )
+        return model._meta.concrete_model == opts.concrete_model
 
     # If the field is a primary key, then doing a query against the field's
     # model is ok, too. Consider the case:
