@@ -1,7 +1,7 @@
 import collections
 import json
 import re
-from functools import partial
+from functools import cached_property, partial
 from itertools import chain
 
 from plain.exceptions import EmptyResultSet, FieldError, FullResultSet
@@ -22,7 +22,6 @@ from plain.models.sql.constants import (
 from plain.models.sql.query import Query, get_order_dir
 from plain.models.sql.where import AND
 from plain.models.transaction import TransactionManagementError
-from plain.utils.functional import cached_property
 from plain.utils.hashable import make_hashable
 from plain.utils.regex_helper import _lazy_re_compile
 
@@ -43,10 +42,9 @@ class SQLCompiler:
         re.MULTILINE | re.DOTALL,
     )
 
-    def __init__(self, query, connection, using, elide_empty=True):
+    def __init__(self, query, connection, elide_empty=True):
         self.query = query
         self.connection = connection
-        self.using = using
         # Some queries, e.g. coalesced aggregation, need to be executed even if
         # they would return an empty result set.
         self.elide_empty = elide_empty
@@ -64,7 +62,7 @@ class SQLCompiler:
         return (
             f"<{self.__class__.__qualname__} "
             f"model={self.query.model.__qualname__} "
-            f"connection={self.connection!r} using={self.using!r}>"
+            f"connection={self.connection!r}>"
         )
 
     def setup_query(self, with_col_aliases=False):
@@ -203,8 +201,6 @@ class SQLCompiler:
             # present in the grouped columns. This is done by identifying all
             # tables that have their primary key included in the grouped
             # columns and removing non-primary key columns referring to them.
-            # Unmanaged models are excluded because they could be representing
-            # database views on which the optimization might not be allowed.
             pks = {
                 expr
                 for expr in expressions
@@ -279,16 +275,6 @@ class SQLCompiler:
             related_klass_infos = self.get_related_selections(select, select_mask)
             klass_info["related_klass_infos"] = related_klass_infos
 
-            def get_select_from_parent(klass_info):
-                for ki in klass_info["related_klass_infos"]:
-                    if ki["from_parent"]:
-                        ki["select_fields"] = (
-                            klass_info["select_fields"] + ki["select_fields"]
-                        )
-                    get_select_from_parent(ki)
-
-            get_select_from_parent(klass_info)
-
         ret = []
         col_idx = 1
         for col, alias in select:
@@ -545,7 +531,7 @@ class SQLCompiler:
     def get_combinator_sql(self, combinator, all):
         features = self.connection.features
         compilers = [
-            query.get_compiler(self.using, self.connection, self.elide_empty)
+            query.get_compiler(elide_empty=self.elide_empty)
             for query in self.query.combined_queries
         ]
         if not features.supports_slicing_ordering_in_compound:
@@ -673,9 +659,7 @@ class SQLCompiler:
                     {expr: Ref(alias, expr) for expr, alias in replacements.items()}
                 )
             )
-        inner_query_compiler = inner_query.get_compiler(
-            self.using, connection=self.connection, elide_empty=self.elide_empty
-        )
+        inner_query_compiler = inner_query.get_compiler(elide_empty=self.elide_empty)
         inner_sql, inner_params = inner_query_compiler.as_sql(
             # The limits must be applied to the outer query to avoid pruning
             # results too eagerly.
@@ -851,7 +835,7 @@ class SQLCompiler:
                     result.append(for_update_part)
 
                 if where:
-                    result.append("WHERE %s" % where)
+                    result.append(f"WHERE {where}")
                     params.extend(w_params)
 
                 grouping = []
@@ -864,11 +848,11 @@ class SQLCompiler:
                             "annotate() + distinct(fields) is not implemented."
                         )
                     order_by = order_by or self.connection.ops.force_no_ordering()
-                    result.append("GROUP BY %s" % ", ".join(grouping))
+                    result.append("GROUP BY {}".format(", ".join(grouping)))
                     if self._meta_ordering:
                         order_by = None
                 if having:
-                    result.append("HAVING %s" % having)
+                    result.append(f"HAVING {having}")
                     params.extend(h_params)
 
             if self.query.explain_info:
@@ -885,7 +869,7 @@ class SQLCompiler:
                 for _, (o_sql, o_params, _) in order_by:
                     ordering.append(o_sql)
                     params.extend(o_params)
-                order_by_sql = "ORDER BY %s" % ", ".join(ordering)
+                order_by_sql = "ORDER BY {}".format(", ".join(ordering))
                 if combinator and features.requires_compound_order_by_subquery:
                     result = ["SELECT * FROM (", *result, ")", order_by_sql]
                 else:
@@ -937,9 +921,7 @@ class SQLCompiler:
             # Finally do cleanup - get rid of the joins we created above.
             self.query.reset_refcounts(refcounts_before)
 
-    def get_default_columns(
-        self, select_mask, start_alias=None, opts=None, from_parent=None
-    ):
+    def get_default_columns(self, select_mask, start_alias=None, opts=None):
         """
         Compute the default columns for selecting every field in the base
         model. Will sometimes be called to pull in related models (e.g. via
@@ -956,10 +938,6 @@ class SQLCompiler:
             if (opts := self.query.get_meta()) is None:
                 return result
         start_alias = start_alias or self.query.get_initial_alias()
-        # The 'seen_models' is used to optimize checking the needed parent
-        # alias for a given field. This also includes None -> start_alias to
-        # be used by local fields.
-        seen_models = {None: start_alias}
 
         for field in opts.concrete_fields:
             model = field.model._meta.concrete_model
@@ -967,23 +945,10 @@ class SQLCompiler:
             # will assign None if the field belongs to this model.
             if model == opts.model:
                 model = None
-            if (
-                from_parent
-                and model is not None
-                and issubclass(
-                    from_parent._meta.concrete_model, model._meta.concrete_model
-                )
-            ):
-                # Avoid loading data for already loaded parents.
-                # We end up here in the case select_related() resolution
-                # proceeds from parent model to child model. In that case the
-                # parent model data is already present in the SELECT clause,
-                # and we want to avoid reloading the same data again.
-                continue
             if select_mask and field not in select_mask:
                 continue
-            alias = self.query.join_parent_model(opts, model, start_alias, seen_models)
-            column = field.get_col(alias)
+
+            column = field.get_col(start_alias)
             result.append(column)
         return result
 
@@ -1127,7 +1092,7 @@ class SQLCompiler:
                 alias not in self.query.alias_map
                 or self.query.alias_refcount[alias] == 1
             ):
-                result.append(", %s" % self.quote_name_unless_alias(alias))
+                result.append(f", {self.quote_name_unless_alias(alias)}")
         return result, params
 
     def get_related_selections(
@@ -1152,7 +1117,7 @@ class SQLCompiler:
             reverse_choices = (
                 f.field.related_query_name()
                 for f in opts.related_objects
-                if f.field.unique
+                if f.field.primary_key
             )
             return chain(
                 direct_choices, reverse_choices, self.query._filtered_relations
@@ -1206,9 +1171,8 @@ class SQLCompiler:
                 "reverse": False,
                 "local_setter": f.set_cached_value,
                 "remote_setter": f.remote_field.set_cached_value
-                if f.unique
+                if f.primary_key
                 else lambda x, y: None,
-                "from_parent": False,
             }
             related_klass_infos.append(klass_info)
             select_fields = []
@@ -1236,7 +1200,7 @@ class SQLCompiler:
             related_fields = [
                 (o.field, o.related_model)
                 for o in opts.related_objects
-                if o.field.unique and not o.many_to_many
+                if o.field.primary_key and not o.many_to_many
             ]
             for related_field, model in related_fields:
                 related_select_mask = select_mask.get(related_field) or {}
@@ -1256,14 +1220,12 @@ class SQLCompiler:
                     [related_field_name], opts, root_alias
                 )
                 alias = join_info.joins[-1]
-                from_parent = issubclass(model, opts.model) and model is not opts.model
                 klass_info = {
                     "model": model,
                     "field": related_field,
                     "reverse": True,
                     "local_setter": related_field.remote_field.set_cached_value,
                     "remote_setter": related_field.set_cached_value,
-                    "from_parent": from_parent,
                 }
                 related_klass_infos.append(klass_info)
                 select_fields = []
@@ -1271,7 +1233,6 @@ class SQLCompiler:
                     related_select_mask,
                     start_alias=alias,
                     opts=model._meta,
-                    from_parent=opts.model,
                 )
                 for col in columns:
                     select_fields.append(len(select))
@@ -1311,9 +1272,6 @@ class SQLCompiler:
                     )
                     model = join_opts.model
                     alias = joins[-1]
-                    from_parent = (
-                        issubclass(model, opts.model) and model is not opts.model
-                    )
                     klass_info = {
                         "model": model,
                         "field": final_field,
@@ -1324,7 +1282,6 @@ class SQLCompiler:
                             else local_setter_noop
                         ),
                         "remote_setter": partial(remote_setter, name),
-                        "from_parent": from_parent,
                     }
                     related_klass_infos.append(klass_info)
                     select_fields = []
@@ -1333,7 +1290,6 @@ class SQLCompiler:
                         field_select_mask,
                         start_alias=alias,
                         opts=model._meta,
-                        from_parent=opts.model,
                     )
                     for col in columns:
                         select_fields.append(len(select))
@@ -1352,7 +1308,7 @@ class SQLCompiler:
                     get_related_klass_infos(klass_info, next_klass_infos)
             fields_not_found = set(requested).difference(fields_found)
             if fields_not_found:
-                invalid_fields = ("'%s'" % s for s in fields_not_found)
+                invalid_fields = (f"'{s}'" for s in fields_not_found)
                 raise FieldError(
                     "Invalid field name(s) given in select_related: {}. "
                     "Choices are: {}".format(
@@ -1368,25 +1324,6 @@ class SQLCompiler:
         the query.
         """
 
-        def _get_parent_klass_info(klass_info):
-            concrete_model = klass_info["model"]._meta.concrete_model
-            for parent_model, parent_link in concrete_model._meta.parents.items():
-                parent_list = parent_model._meta.get_parent_list()
-                yield {
-                    "model": parent_model,
-                    "field": parent_link,
-                    "reverse": False,
-                    "select_fields": [
-                        select_index
-                        for select_index in klass_info["select_fields"]
-                        # Selected columns from a model or its parents.
-                        if (
-                            self.select[select_index][0].target.model == parent_model
-                            or self.select[select_index][0].target.model in parent_list
-                        )
-                    ],
-                }
-
         def _get_first_selected_col_from_model(klass_info):
             """
             Find the first selected column from a model. If it doesn't exist,
@@ -1414,10 +1351,6 @@ class SQLCompiler:
                         field = field.remote_field
                     path = parent_path + [field.name]
                     yield LOOKUP_SEP.join(path)
-                queue.extend(
-                    (path, klass_info)
-                    for klass_info in _get_parent_klass_info(klass_info)
-                )
                 queue.extend(
                     (path, klass_info)
                     for klass_info in klass_info.get("related_klass_infos", [])
@@ -1433,10 +1366,7 @@ class SQLCompiler:
                 col = _get_first_selected_col_from_model(klass_info)
             else:
                 for part in name.split(LOOKUP_SEP):
-                    klass_infos = (
-                        *klass_info.get("related_klass_infos", []),
-                        *_get_parent_klass_info(klass_info),
-                    )
+                    klass_infos = (*klass_info.get("related_klass_infos", []),)
                     for related_klass_info in klass_infos:
                         field = related_klass_info["field"]
                         if related_klass_info["reverse"]:
@@ -1592,7 +1522,7 @@ class SQLCompiler:
             self.query.where.add(RawSQL(f"{lhs_sql} = {rhs}", lhs_params), AND)
 
         sql, params = self.as_sql()
-        return "EXISTS (%s)" % sql, params
+        return f"EXISTS ({sql})", params
 
     def explain_query(self):
         result = list(self.execute_sql())
@@ -1657,8 +1587,8 @@ class SQLInsertCompiler(SQLCompiler):
             # doesn't exist yet.
             if value.contains_column_references:
                 raise ValueError(
-                    'Failed to insert expression "{}" on {}. F() expressions '
-                    "can only be used to update, not to insert.".format(value, field)
+                    f'Failed to insert expression "{value}" on {field}. F() expressions '
+                    "can only be used to update, not to insert."
                 )
             if value.contains_aggregate:
                 raise FieldError(
@@ -1667,9 +1597,7 @@ class SQLInsertCompiler(SQLCompiler):
                 )
             if value.contains_over_clause:
                 raise FieldError(
-                    "Window expressions are not allowed in this query ({}={!r}).".format(
-                        field.name, value
-                    )
+                    f"Window expressions are not allowed in this query ({field.name}={value!r})."
                 )
         return field.get_db_prep_save(value, connection=self.connection)
 
@@ -1727,7 +1655,7 @@ class SQLInsertCompiler(SQLCompiler):
         )
         result = [f"{insert_statement} {qn(opts.db_table)}"]
         fields = self.query.fields or [opts.pk]
-        result.append("(%s)" % ", ".join(qn(f.column) for f in fields))
+        result.append("({})".format(", ".join(qn(f.column) for f in fields)))
 
         if self.query.fields:
             value_rows = [
@@ -1770,7 +1698,7 @@ class SQLInsertCompiler(SQLCompiler):
                 )
                 params = param_rows
             else:
-                result.append("VALUES (%s)" % ", ".join(placeholder_rows[0]))
+                result.append("VALUES ({})".format(", ".join(placeholder_rows[0])))
                 params = [param_rows[0]]
             if on_conflict_suffix_sql:
                 result.append(on_conflict_suffix_sql)
@@ -1793,7 +1721,7 @@ class SQLInsertCompiler(SQLCompiler):
             if on_conflict_suffix_sql:
                 result.append(on_conflict_suffix_sql)
             return [
-                (" ".join(result + ["VALUES (%s)" % ", ".join(p)]), vals)
+                (" ".join(result + ["VALUES ({})".format(", ".join(p))]), vals)
                 for p, vals in zip(placeholder_rows, param_rows)
             ]
 
@@ -1868,7 +1796,7 @@ class SQLDeleteCompiler(SQLCompiler):
         )
 
     def _as_sql(self, query):
-        delete = "DELETE FROM %s" % self.quote_name_unless_alias(query.base_table)
+        delete = f"DELETE FROM {self.quote_name_unless_alias(query.base_table)}"
         try:
             where, params = self.compile(query.where)
         except FullResultSet:
@@ -1891,8 +1819,8 @@ class SQLDeleteCompiler(SQLCompiler):
         if not self.connection.features.update_can_self_select:
             # Force the materialization of the inner query to allow reference
             # to the target table on MySQL.
-            sql, params = innerq.get_compiler(connection=self.connection).as_sql()
-            innerq = RawSQL("SELECT * FROM (%s) subquery" % sql, params)
+            sql, params = innerq.get_compiler().as_sql()
+            innerq = RawSQL(f"SELECT * FROM ({sql}) subquery", params)
         outerq.add_filter("pk__in", innerq)
         return self._as_sql(outerq)
 
@@ -1928,10 +1856,8 @@ class SQLUpdateCompiler(SQLCompiler):
                     val = val.prepare_database_save(field)
                 else:
                     raise TypeError(
-                        "Tried to update field {} with a model instance, {!r}. "
-                        "Use a value compatible with {}.".format(
-                            field, val, field.__class__.__name__
-                        )
+                        f"Tried to update field {field} with a model instance, {val!r}. "
+                        f"Use a value compatible with {field.__class__.__name__}."
                     )
             val = field.get_db_prep_save(val, connection=self.connection)
 
@@ -1949,10 +1875,10 @@ class SQLUpdateCompiler(SQLCompiler):
                 values.append(f"{qn(name)} = {placeholder}")
                 update_params.append(val)
             else:
-                values.append("%s = NULL" % qn(name))
+                values.append(f"{qn(name)} = NULL")
         table = self.query.base_table
         result = [
-            "UPDATE %s SET" % qn(table),
+            f"UPDATE {qn(table)} SET",
             ", ".join(values),
         ]
         try:
@@ -1960,7 +1886,7 @@ class SQLUpdateCompiler(SQLCompiler):
         except FullResultSet:
             params = []
         else:
-            result.append("WHERE %s" % where)
+            result.append(f"WHERE {where}")
         return " ".join(result), tuple(update_params + params)
 
     def execute_sql(self, result_type):
@@ -1978,7 +1904,7 @@ class SQLUpdateCompiler(SQLCompiler):
             if cursor:
                 cursor.close()
         for query in self.query.get_related_updates():
-            aux_rows = query.get_compiler(self.using).execute_sql(result_type)
+            aux_rows = query.get_compiler().execute_sql(result_type)
             if is_empty and aux_rows:
                 rows = aux_rows
                 is_empty = False
@@ -2008,18 +1934,10 @@ class SQLUpdateCompiler(SQLCompiler):
         fields = [meta.pk.name]
         related_ids_index = []
         for related in self.query.related_updates:
-            if all(
-                path.join_field.primary_key for path in meta.get_path_to_parent(related)
-            ):
-                # If a primary key chain exists to the targeted related update,
-                # then the meta.pk value can be used for it.
-                related_ids_index.append((related, 0))
-            else:
-                # This branch will only be reached when updating a field of an
-                # ancestor that is not part of the primary key chain of a MTI
-                # tree.
-                related_ids_index.append((related, len(fields)))
-                fields.append(related._meta.pk.name)
+            # If a primary key chain exists to the targeted related update,
+            # then the meta.pk value can be used for it.
+            related_ids_index.append((related, 0))
+
         query.add_fields(fields)
         super().pre_sql_setup()
 
@@ -2036,7 +1954,7 @@ class SQLUpdateCompiler(SQLCompiler):
             # selecting from the updating table (e.g. MySQL).
             idents = []
             related_ids = collections.defaultdict(list)
-            for rows in query.get_compiler(self.using).execute_sql(MULTI):
+            for rows in query.get_compiler().execute_sql(MULTI):
                 idents.extend(r[0] for r in rows)
                 for parent, index in related_ids_index:
                     related_ids[parent].extend(r[index] for r in rows)
@@ -2065,7 +1983,6 @@ class SQLAggregateCompiler(SQLCompiler):
         params = tuple(params)
 
         inner_query_sql, inner_query_params = self.query.inner_query.get_compiler(
-            self.using,
             elide_empty=self.elide_empty,
         ).as_sql(with_col_aliases=True)
         sql = f"SELECT {sql} FROM ({inner_query_sql}) subquery"
